<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_OPN_MAILER_INCLUDED') ) {
	define ('_OPN_CLASS_OPN_MAILER_INCLUDED', 1);

	class opn_mailer {
		var $_to = '';
		var $_toname = '';
		var $_from = '';
		var $_fromname = '';
		var $_headers = array ();
		var $_subject = '';
		var $_filepath = '';
		var $_body = '';
		var $_debug = false;
		var $_vars = array ();
		var $_error = '';
		var $_filename = '';
		var $_parsebody = false;
		var $_parts;
		var $_mailer = false;
		var $_sendhtml = false;

		/**
		* Should the clas to a debugoutput or not.
		*
		* @param bool $debug True gives the debugoutput and the class will
		* stops with a die.
		* @access public
		*/

		function setDebug ($debug) {

			$this->_debug = $debug;

		}
		// end function

		function SetToHTML_Send ($var = false) {

			$this->_sendhtml = $var;

		}
		// end function

		/**
		* Set's a headerparter.
		*
		* @param string $index Index for the headerarray
		* @param string $value The value to set.
		* @access public
		*/

		function setHeader ($index, $value) {

			$this->_headers[$index] = $value;

		}

		/**
		* Set's the body for an email when no parsing is needed.
		*
		* @param string $body The body to set.
		* @access public
		*/

		function setBody ($body) {

			$this->_body = $body;

		}

		/**
		* Clean the last error
		*
		* @access public
		*/

		function clear_error () {

			$this->_error = '';

		}

		/**
		* Fill's the class vars with the needed values.
		*
		* @param string $toemail Reciepient of the email.
		* @param string $subject Subject of the email.
		* @param string $filepath Path to the mailfile.
		* @param string $filename Name of the mailfile or when filename is
		* empty the body to parse.
		* @param mixed $vars The vars array that should replaced the
		* placeholders in the body or when $fillbody is true the body of the
		* email as string.
		* @param string $fromname The name of the sender.
		* @param string $frommail The email of the sender.
		* @param bool $fillbody Optional. True = $vars contains the body.
		* @param bool $parsebody Optional. True = $filename cotains the
		* messagebody.
		* @access public
		*/

		function opn_mail_fill ($toemail, $subject, $filepath, $filename, $vars, $fromname, $frommail, $fillbody = false, $parsebody = false) {

			$this->_to = $toemail;
			$this->_subject = $subject;
			if ($parsebody) {
				$this->_parsebody = true;
				$this->_body = $filename;
				$this->_vars = $vars;
			} elseif ($fillbody) {
				$this->_body = $vars;
			} else {
				$this->_filepath = $filepath;
				$this->_filename = $filename;
				$this->_vars = $vars;
			}
			$this->_from = $frommail;
			$this->_fromname = $fromname;

		}

		/**
		* class init
		*
		* @access public
		*/

		function init () {

			global $opnConfig;

			$this->_to = '';
			$this->_toname = '';
			$this->_error = '';
			$this->_from = '';
			$this->_fromname = '';
			$this->_subject = '';
			$this->_filepath = '';
			$this->_filename = '';
			$this->_vars = array ();
			$this->_body = '';
			$this->_debug = false;
			$this->_parsebody = false;
			$this->_headers = array ();
			$this->_parts = array ();
			$this->_sendhtml = false;
			switch ($opnConfig['opn_emailpriority']) {
				case '1':
					$emailpriority = 'High';
					break;
				case '3':
					$emailpriority = 'Normal';
					break;
				case '4':
					$emailpriority = 'Low';
					break;
				default:
					$emailpriority = 'ERROR ;-)';
					break;
			}
			$this->_headers['X-MSMail-Priority'] = $emailpriority;
			$this->_headers['X-Importance'] = $emailpriority;
			$this->_headers['X-Priority'] = $opnConfig['opn_emailpriority'];
			$this->_headers['X-MimeOLE'] = 'Produced by openPHPnuke 2.3';
			$this->_headers['X-Mailer'] = 'openPHPnuke';
			$this->_headers['X-openPHPnuke-Site'] = $opnConfig['sitename'];
			$this->_headers['X-openPHPnuke-URL'] = $opnConfig['opn_url'];

		}

		/**
		* the class starting
		*
		* @access public
		*/

		function opn_mailer () {

			include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'mail/class.opn_mail.php');
			$this->init ();

		}

		/**
		* add an attachment to mail
		*
		* @param string $file The filename with path to read and add.
		* @param string $name The displayed filename in the email. Optional.
		* @param string $ctype The content type. Optional.
		* @param string $encode Attachment encoding. Optional.
		* @access public
		*/

		function add_attachment ($file, $name = '', $ctype = 'application/octet-stream', $encode = 'base64') {

			$filedata = $this->_file2str ($file);
			// Force the name the user supplied, otherwise use $file
			$filename = (!empty ($name) )? $name : $file;
			if (empty ($filename) ) {
				opnErrorHandler (E_WARNING, 'The supplied filename for the attachment can\'t be empty', __FILE__, __LINE__);
			}
			$filename = basename ($filename);
			if (!$filedata) {
				return false;
			}
			$this->_parts[] = array ('c_type' => $ctype,
						'body' => $filedata,
						'encoding' => $encode,
						'name' => $filename);
			unset ($filedata);
			unset ($filename);
			return true;

		}

		/**
		* Creates a message ID.
		*
		* @return string The message ID.
		* @access private
		*/

		function _create_id () {

			$id = time () . '.';
			for ($c = 1; $c<16; $c++) {
				$randnum = rand (1, 9);
				$id .= $randnum;
			}
			return $id;

		}

		/**
		* Parses the Mailfile and replaces the placeholders.
		*
		* @param string $path Modulepath for the mailfile.
		* @param string $name The mailfile.
		* @param array $vars The Replacements.
		* @return string The parsed mailfile.
		* @access private
		*/

		function _ParseFile ($path, $name, $vars) {

			global $opnConfig, $opnTables;

			$message = '';

			if ($opnConfig['installedPlugins']->isplugininstalled ('admin/customizer_email_template') ) {

				$_name = $opnConfig['opnSQL']->qstr ($name. '.mail');
				$_module = $opnConfig['opnSQL']->qstr ($path);
				$_language = $opnConfig['opnSQL']->qstr ($opnConfig['language']);

				$result = $opnConfig['database']->Execute ('SELECT template FROM ' . $opnTables['opn_cmi_email_template']. ' WHERE (module='.$_module.') AND (name='.$_name.') AND (language='.$_language.')');
				if ($result !== false) {
					while (! $result->EOF) {
						$message .= $result->fields['template'];
						$result->MoveNext ();
					}
				}

			}

			if ($message == '') {

				if (file_exists (_OPN_ROOT_PATH . $path . '/mail/' . $opnConfig['language'] . '/' . $name . '.mail') ) {
					$fp = file (_OPN_ROOT_PATH . $path . '/mail/' . $opnConfig['language'] . '/' . $name . '.mail');
				} else {
					$fp = file (_OPN_ROOT_PATH . $path . '/mail/english/' . $name . '.mail');
				}
				$max = count ($fp);
				for ($i = 0; $i< $max; $i++) {
					$message .= $fp[$i];
				}

			}
			$message .= _OPN_HTML_NL;
			foreach ($vars as $key => $value) {
				$message = str_replace (strtoupper ($key), $value, $message);
			}
			$message = str_replace ('{SITENAME}', $opnConfig['sitename'], $message);
			$message = str_replace ('{SITEURL}', $opnConfig['opn_url'], $message);
			$message = str_replace ('{SITESLOGAN}', $opnConfig['slogan'], $message);
			$message = str_replace ('{SITEMAIL}', $opnConfig['adminmail'], $message);
			unset ($fp);
			return $message;

		}

		/**
		* Parse the Body and replaces the Placeholders.
		*
		* @param string $body The message to be parsed.
		* @param array $vars The Replacements.
		* @return string The parsed Body.
		* @access private
		*/

		function _ParseBody ($body, $vars) {

			global $opnConfig;

			$message = $body;
			foreach ($vars as $key => $value) {
				$message = str_replace (strtoupper ($key), $value, $message);
			}
			$message = str_replace ('{SITENAME}', $opnConfig['sitename'], $message);
			$message = str_replace ('{SITEURL}', $opnConfig['opn_url'], $message);
			$message = str_replace ('{SITESLOGAN}', $opnConfig['slogan'], $message);
			$message = str_replace ('{SITEMAIL}', $opnConfig['adminmail'], $message);
			return $message;

		}

		/**
		* Send the mail
		*
		* @return int The Errorflag. (0 = No Error with to and from, 1 Error
		* with the To Address, 2 = Error with the From Address.
		* @access public
		*/

		function send () {

			global $opnConfig;
			if (empty ($this->_filepath) ) {
				$b = $this->_body;
			}
			$frominheader = false;
			if (isset ($this->_headers['From']) ) {
				$frominheader = true;
			}
			if (!$frominheader) {
				if (substr_count ($this->_to, 'freenet')>0) {
					$this->_headers['From'] = $this->_from;
				} else {
					if (!empty ($this->_fromname) ) {
						$this->_headers['From'] = $this->_fromname . ' <' . $this->_from . '>';
					} else {
						$this->_headers['From'] = $this->_from . ' <' . $this->_from . '>';
					}
				}
			}
			if (!isset ($this->_headers['Errors-To']) ) {
				$this->_headers['Errors-To'] = $this->_from;
			}
			if (!isset ($this->_headers['Return-Path']) ) {
				if ($opnConfig['send_email_via'] != 'smtp') {
					$this->_headers['Return-Path'] = $this->_from;
				}
			}
			if (!isset ($this->_headers['Reply-To']) ) {
				$this->_headers['Reply-To'] = $this->_from;
			}
			if ( (isset ($opnConfig['opn_emailgeneratemessageid']) ) && ($opnConfig['opn_emailgeneratemessageid'] == 1) ) {
				$msgidinheader = false;
				if (isset ($this->_headers['Message-ID']) ) {
					$msgidinheader = true;
				}
				if (!$msgidinheader) {
					$temp = explode ('@', $this->_from);
					if (!isset ($temp[1])) {
						$temp[1] = 'example.com';
					}
					$this->_headers['Message-ID'] = '<' . $this->_create_id () . '@' . $temp[1] . '>';
				}
			}
			$body = '';
			if ($this->_parsebody) {
				$body = $this->_ParseBody ($b, $this->_vars);
			} elseif (!empty ($this->_filename) ) {
				$body = $this->_ParseFile ($this->_filepath, $this->_filename, $this->_vars);
			} elseif (!empty ($b) ) {
				$body = $b;
			}
			$this->_headers['Subject'] = $this->_subject;
			$this->_headers['Date'] = date ('D, d M y H:i:s O');
			if ($opnConfig['opn_use_checkemail'] == 1) {
				include (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'class.multichecker.php');
				$check =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ multichecker ();
				if ($this->_to != '') {
					if (!$check->email_validateexists ($this->_to) ) {
						return 1;
					}
				}
				if (!$check->email_validateexists ($this->_from) ) {
					// hier kommt noch was
					return 2;
				}
			}
			if (!defined ('MAIL_MIME_CRLF') ) {
				define ('MAIL_MIME_CRLF', "\n", true);
			}
			include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'mail/mime_part.php');
			$params = array ();
			$tparams = array ();
			$tparams['charset'] = 'iso-8859-1';
			if ($this->_sendhtml == false) {
				$tparams['content_type'] = 'text/plain';
			} else {
				$tparams['content_type'] = 'text/html';
			}
			$tparams['encoding'] = $opnConfig['opn_email_body_encoding'];
			if ($opnConfig['opn_email_charset_encoding'] != '') {
				$tparams['charset'] = $opnConfig['opn_email_charset_encoding'];
			}
			if (count ($this->_parts) ) {
				$params['content_type'] = 'multipart/mixed';
				$message =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ Mail_mimePart ('', $params);
				$message->addSubpart ($body, $tparams);
				$max = count ($this->_parts);
				for ($i = 0; $i< $max; $i++) {
					$this->_addAttachmentPart ($message, $this->_parts[$i]);
				}
			} else {
				$message =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ Mail_mimePart ($body, $tparams);
			}
			$output = $message->encode ();
			unset ($params);
			unset ($tparams);
			$this->_headers = array_merge ($this->_headers, $output['headers']);
			$body = $output['body'];
			unset ($output);
			$headers['MIME-Version'] = '1.0';
			$this->_headers = array_merge ($headers, $this->_headers);
			$headers = $this->_encodeHeaders ($this->_headers);
			if ($this->_debug) {
				echo 'The Mailheader:<br />' . print_array ($headers) . '<br /><br />';
				echo 'The Body:<br />' . nl2br ($body);
				echo '<br />';
				die ();
			}
			if ($opnConfig['opn_activate_email'] == 1) {
				if ($this->_mailer === false) {
 					$params = array ();
 					$this->_mailer = loadDriver ($opnConfig['send_email_via'], $params);
				}
				$result = $this->_mailer->send ($this->_to, $headers, $body);
				if ( (isset ($opnConfig['opn_all_email_to_admin']) ) && ($opnConfig['opn_all_email_to_admin'] == 1) ) {
					$result = $this->_mailer->send ($opnConfig['adminmail'], $headers, $body);
				}
			}
			unset ($body);
			unset ($headers);
			return 0;

		}

		/**
		* Adds an attachment subpart to a mimePart object and returns it during
		* the build process.
		*
		* @param  object  The mimePart to add the image to
		* @param  array   The attachment information
		* @return object  The image mimePart object
		* @access private
		*/

		function &_addAttachmentPart (&$obj, $value) {

			$params['content_type'] = $value['c_type'];
			$params['encoding'] = $value['encoding'];
			$params['disposition'] = 'attachment';
			$params['dfilename'] = $value['name'];
			$obj->addSubpart ($value['body'], $params);
			return $obj;

		}

		/**
		* Encodes a header as per RFC2047
		*
		* @param  string  $input The header data to encode
		* @return string		 Encoded data
		* @access private
		*/

		function _encodeHeaders ($input) {

			global $opnConfig;

			foreach ($input as $hdr_name => $hdr_value) {
				$matches = '';
				preg_match_all ('/(\w*[\x80-\xFF]+\w*)/', $hdr_value, $matches);
				foreach ($matches[1] as $value) {
					$replacement = preg_replace ('/([\x80-\xFF\x5F])/e', '"=" . strtoupper(dechex(ord("\1")))', $value);
					$hdr_value = str_replace ($value, '=?' . $opnConfig['opn_email_charset_encoding'] . '?Q?' . $replacement . '?=', $hdr_value);
				}
				/* catch special case that two encoded words follow each other - in this case the space must become part of the encoded word */
				$hdr_value = str_replace ('?= =?', '_?= =?', $hdr_value);
				$input[$hdr_name] = $hdr_value;
			}
			return $input;

		}

		/**
		* Returns the contents of the given file name as string
		* @param string $file_name
		* @return string
		* @acces private
		*/

		function &_file2str ($file_name) {

			$false = false;
			if (!is_readable ($file_name) ) {
				opnErrorHandler (E_WARNING, 'File is not readable ' . $file_name, __FILE__, __LINE__);
				return $false;
			}
			if (!$fd = fopen ($file_name, 'rb') ) {
				opnErrorHandler (E_WARNING, 'Could not open ' . $file_name, __FILE__, __LINE__);
				return $false;
			}
			$cont = fread ($fd, filesize ($file_name) );
			fclose ($fd);
			return $cont;

		}

	}
}
// if

?>