<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_OPN_MAIL_INCLUDED') ) {
	define ('_OPN_CLASS_OPN_MAIL_INCLUDED', 1);

	/**
	* FUNCTION: loadDriver
	* Provides an interface for generating opn_mail:: objects of various
	* types
	*
	* @param string $driver The kind of opn_mail:: object to instantiate.
	* @param array  $params The parameters to pass to the opn_mail:: object.
	* @access public
	* @return object
	*/

	function loadDriver ($driver, $params) {

		global $opnConfig;
		switch ($opnConfig['send_email_via']) {
			case 'smtp':
				$params['host'] = $opnConfig['smtp_host'];
				$params['port'] = $opnConfig['smtp_port'];
				$params['localhost'] = $opnConfig['opn_url'];
				if ( ($opnConfig['smtp_auth'] === true) || ($opnConfig['smtp_auth'] === 1) || ($opnConfig['smtp_auth'] == '1') ) {
					$params['auth'] = '';
				} else {
					$params['auth'] = $opnConfig['smtp_auth'];
				}
				$params['username'] = $opnConfig['smtp_username'];
				$params['password'] = open_the_secret (md5 ($opnConfig['encoder']), $opnConfig['smtp_password']);
				break;
			case 'sendmail':
				$params['sendmail_path'] = $opnConfig['sendmail_path'];
				$params['sendmail_args'] = $opnConfig['sendmail_args'];
				break;
			case 'mail':
				if ($opnConfig['opn_use_sendmail'] == 1) {
					// $params['f'] = '-f'.$this->_from;
				}
				break;
		}
		// end switch
		$driver = strtolower ($driver);
		include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'mail/class.opn_mail_' . $driver . '.php');
		$class = 'opn_mail_' . $driver;
		if (class_exists ($class) ) {
			$c =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ $class ($params);
			return $c;
		}
		opnErrorHandler (E_ERROR, 'Unable to find class for driver ' . $driver, __FILE__, __LINE__);
		$object = null;
		return (object) $object;

	}
	// end function

	class opn_mail {

		/**
		* Line terminator used for separating header lines.
		* @var string
		*/
		var $sep = "\r\n";

		/**
		* FUNCTION: send
		* Virtual send() function.
		*
		* @param mixed $recipients Either a comma-seperated list of recipients
		*			  (RFC822 compliant), or an array of recipients,
		*			  each RFC822 valid. This may contain recipients not
		*			  specified in the headers, for Bcc:, resending
		*			  messages, etc.
		*
		* @param array $headers The array of headers to send with the mail, in an
		*			  associative array, where the array key is the
		*			  header name (ie, 'Subject'), and the array value
		*			  is the header value (ie, 'test'). The header
		*			  produced from those values would be 'Subject:
		*			  test'.
		*
		* @param string $body The full text of the message body, including any
		*			   Mime parts, etc.
		*
		* @return mixed Returns true on success, or a PEAR_Error
		*			   containing a descriptive error message on
		*			   failure.
		* @access public
		*/

		function send ($recipients, $headers, $body) {

			$t = $recipients;
			$t = $headers;
			$t = $body;
			return false;

		}
		// end function

		/**
		* FUNCTION: _prepareHeaders
		* Take an array of mail headers and return a
		* string containing text usable in sending a message.
		*
		* @param array $headers The array of headers to prepare, in an associative
		*			  array, where the array key is the header name (ie,
		*			  'Subject'), and the array value is the header
		*			  value (ie, 'test'). The header produced from those
		*			  values would be 'Subject: test'.
		*
		* @return mixed Returns false if it encounters a bad address,
		*			   otherwise returns an array containing two
		*			   elements: Any From: address found in the headers,
		*			   and the plain text version of the headers.
		* @access private
		*/

		function _prepareHeaders ($headers) {

			$lines = array ();
			$from = null;
			foreach ($headers as $key => $value) {
				if (strcasecmp ($key, 'From') === 0) {
					$from = $value;
					$this->_getMailbox ($from);
					// Reject envelope From: addresses with spaces.
					if (strstr ($from, ' ') ) {
						return false;
					}
					$lines[] = $key . ': ' . $value;
				} elseif (strcasecmp ($key, 'Received') === 0) {
					// Put Received: headers at the top.  Spam detectors often
					// flag messages with Received: headers after the Subject:
					// as spam.
					array_unshift ($lines, $key . ': ' . $value);
				} else {
					// If $value is an array (i.e., a list of addresses), convert
					// it to a comma-delimited string of its elements (addresses).
					if (is_array ($value) ) {
						$value = implode (', ', $value);
					}
					$lines[] = $key . ': ' . $value;
				}
			}
			return array ($from,
					join ($this->sep,
					$lines) . $this->sep);

		}
		// end function

		/**
		* FUNCTION: _parseRecipients
		* Take a set of recipients and parse them, returning an array of
		* bare addresses (forward paths) that can be passed to sendmail
		* or an smtp server with the rcpt to: command.
		*
		* @param mixed Either a comma-seperated list of recipients
		*			  (RFC822 compliant), or an array of recipients,
		*			  each RFC822 valid.
		*
		* @return array An array of forward paths (bare addresses).
		* @access private
		*/

		function _parseRecipients ($recipients) {
			// if we're passed an array, assume addresses are valid and
			// implode them before parsing.
			if (is_array ($recipients) ) {
				$recipients = implode (', ', $recipients);
			}
			$recipients1 = explode (', ', $recipients);
			$recipients = array ();
			foreach ($recipients1 as $value) {
				$to = $value;
				$this->_getMailbox ($to);
				$recipients[] = $to;
			}
			unset ($recipients1);
			return $recipients;

		}

		/**
		* FUNCTION: _splitCheck
		* A common function that will check an exploded string.
		*
		* @param array $parts The exloded string.
		* @param string $char  The char that was exploded on.
		* @return mixed False if the string contains unclosed quotes/brackets, or the string on success.
		* @access private
		*/

		function _splitCheck ($parts, $char) {

			$string = $parts[0];
			$countparts = count ($parts);
			for ($i = 0; $i<$countparts; $i++) {
				if ($this->_hasUnclosedQuotes ($string) || $this->_hasUnclosedBrackets ($string, '<>') || $this->_hasUnclosedBrackets ($string, '[]') || $this->_hasUnclosedBrackets ($string, '()') || substr ($string, -1) == '\\') {
					if (isset ($parts[$i+1]) ) {
						$string = $string . $char . $parts[$i+1];
					} else {
						opnErrorHandler (E_WARNING, 'Invalid address spec. Unclosed bracket or quotes ' . $parts, __FILE__, __LINE__);
						return false;
					}
				} else {
					break;
				}
			}
			return $string;

		}

		/**
		* FUNCTION: _hasUnclosedQuotes
		* Checks if a string has an unclosed quotes or not.
		*
		* @param string $string The string to check.
		* @return boolean True if there are unclosed quotes inside the string, false otherwise.
		* @access private
		*/

		function _hasUnclosedQuotes ($string) {

			$string = explode ('"', $string);
			$string_cnt = count ($string);
			for ($i = 0; $i< (count ($string)-1); $i++)
			if (substr ($string[$i], -1) == '\\') {
				$string_cnt--;
			}
			return ($string_cnt%2 === 0);

		}

		/**
		* FUNCTION: _hasUnclosedBrackets
		* Checks if a string has an unclosed brackets or not. IMPORTANT:
		* This function handles both angle brackets and square brackets;
		*
		* @param string $string The string to check.
		* @param string $chars  The characters to check for.
		* @return boolean True if there are unclosed brackets inside the string, false otherwise.
		* @access private
		*/

		function _hasUnclosedBrackets ($string, $chars) {

			$num_angle_start = substr_count ($string, $chars[0]);
			$num_angle_end = substr_count ($string, $chars[1]);
			$this->_hasUnclosedBracketsSub ($string, $num_angle_start, $chars[0]);
			$this->_hasUnclosedBracketsSub ($string, $num_angle_end, $chars[1]);
			if ($num_angle_start<$num_angle_end) {
				opnErrorHandler (E_WARNING, 'Invalid address spec. Unmatched quote or bracket (' . $chars . ')', __FILE__, __LINE__);
				return false;
			}
			return ($num_angle_start> $num_angle_end);

		}

		/**
		* FUNCTION: _hasUnclosedBracketsSub
		* Sub function that is used only by hasUnclosedBrackets().
		*
		* @param string $string The string to check.
		* @param integer &$num	The number of occurences.
		* @param string $char   The character to count.
		* @return integer The number of occurences of $char in $string, adjusted for backslashes.
		* @access private
		*/

		function _hasUnclosedBracketsSub ($string, &$num, $char) {

			$parts = explode ($char, $string);
			$countparts = count ($parts);
			for ($i = 0; $i<$countparts; $i++) {
				if (substr ($parts[$i], -1) == '\\' || $this->_hasUnclosedQuotes ($parts[$i]) ) {
					$num--;
				}
				if (isset ($parts[$i+1]) ) {
					$parts[$i+1] = $parts[$i] . $char . $parts[$i+1];
				}
			}
			return $num;

		}

		/**
		* FUNCTION: _getMailbox
		* Function to get a mailbox, which is:
		* mailbox =   addr-spec; simple address
		* / phrase route- addr ; name and route-addr
		*
		* @param string &$mailbox The string to check.
		* @return boolean Success or failure.
		* @access private
		*/

		function _getMailbox (&$mailbox) {
			// A couple of defaults.
			$phrase = '';
			$comment = '';
			$comments = array ();
			// Catch any RFC822 comments and store them separately.
			$_mailbox = $mailbox;
			while (strlen (trim ($_mailbox) )>0) {
				$parts = explode ('(', $_mailbox);
				$before_comment = $this->_splitCheck ($parts, '(');
				if ($before_comment != $_mailbox) {
					// First char should be a (.
					$comment = substr (str_replace ($before_comment, '', $_mailbox), 1);
					$parts = explode (')', $comment);
					$comment = $this->_splitCheck ($parts, ')');
					$comments[] = $comment;
					// +1 is for the trailing )
					$_mailbox = substr ($_mailbox, strpos ($_mailbox, $comment)+strlen ($comment)+1);
				} else {
					break;
				}
			}
			foreach ($comments as $comment) {
				$mailbox = str_replace ("($comment)", '', $mailbox);
			}
			unset ($comments);
			$mailbox = trim ($mailbox);
			// Check for name + route-addr
			if (substr ($mailbox, -1) == '>' && substr ($mailbox, 0, 1) != '<') {
				$parts = explode ('<', $mailbox);
				$name = $this->_splitCheck ($parts, '<');
				$phrase = trim ($name);
				$route_addr = trim (substr ($mailbox, strlen ($name . '<'), -1) );
			} else {
				// First snip angle brackets if present.
				if (substr ($mailbox, 0, 1) == '<' && substr ($mailbox, -1) == '>') {
					$addr_spec = substr ($mailbox, 1, -1);
				} else {
					$addr_spec = $mailbox;
				}
			}
			if (isset ($route_addr) ) {
				$mailbox = $route_addr;
			} else {
				$mailbox = $addr_spec;
			}
			return true;

		}

	}
	// end class
}

?>