<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_OPN_MAIL_MAIL_INCLUDED') ) {
	define ('_OPN_CLASS_OPN_MAIL_MAIL_INCLUDED', 1);

	class opn_mail_mail extends opn_mail {

		/**
		* Any arguments to pass to the mail() function.
		* @var string
		*/
		var $_params = '';

		/**
		* FUNCTION: opn_mail_mail
		* Constructor.
		*
		* Instantiates a new opn_mail_mail:: object based on the parameters
		* passed in.
		*
		* @param array $params Extra arguments for the mail() function.
		* @access public
		*/

		function opn_mail_mail ($params = null) {

			/* The other mail implementations accept parameters as arrays.
			* In the interest of being consistent, explode an array into
			* a string of parameter arguments. */
			if (is_array ($params) ) {
				$this->_params = join (' ', $params);
			} else {
				$this->_params = $params;
			}

			/* Because the mail() function may pass headers as command
			* line arguments, we can't guarantee the use of the standard
			* "\r\n" separator.  Instead, we use the system's native line
			* separator. */

			$this->sep = (strstr (PHP_OS, 'WIN') )?"\r\n" : "\n";

		}
		// end function

		/**
		* FUNCTION: send
		* Implements opn_mail_mail::send() function using php's built-in mail()
		* command.
		*
		* @param mixed $recipients Either a comma-seperated list of recipients
		*			  (RFC822 compliant), or an array of recipients,
		*			  each RFC822 valid. This may contain recipients not
		*			  specified in the headers, for Bcc:, resending
		*			  messages, etc.
		*
		* @param array $headers The array of headers to send with the mail, in an
		*			  associative array, where the array key is the
		*			  header name (ie, 'Subject'), and the array value
		*			  is the header value (ie, 'test'). The header
		*			  produced from those values would be 'Subject:
		*			  test'.
		*
		* @param string $body The full text of the message body, including any
		*			   Mime parts, etc.
		*
		* @return mixed Returns true on success, or false on failure.
		*
		* @access public
		*/

		function send ($recipients, $headers, $body) {
			// If we're passed an array of recipients, implode it.
			if (is_array ($recipients) ) {
				$recipients = implode (', ', $recipients);
			}
			// Get the Subject out of the headers array so that we can
			// pass it as a seperate argument to mail().
			$subject = '';
			if (isset ($headers['Subject']) ) {
				$subject = $headers['Subject'];
				unset ($headers['Subject']);
			}
			// Flatten the headers out.
			$headerElements = $this->_prepareHeaders ($headers);
			if ($headerElements === false) {
				return false;
			}
			list (, $text_headers) = $headerElements;

			/*
			* We only use mail()'s optional fifth parameter if the additional
			* parameters have been provided and we're not running in safe mode.
			*/

			$result = false;
			if ($recipients == '') {
				$recipients = 'Undisclosed-recipients:;';
			}
			if (!defined ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN') ) {
				if (empty ($this->_params) || ini_get ('safe_mode') ) {
					$result = mail ($recipients, $subject, $body, $text_headers);
				} else {
					$result = mail ($recipients, $subject, $body, $text_headers, $this->_params);
				}
				if ($result === false) {
					$recipients = ' ';
					if (empty ($this->_params) || ini_get ('safe_mode') ) {
						$result = mail ($recipients, $subject, $body, $text_headers);
					} else {
						$result = mail ($recipients, $subject, $body, $text_headers, $this->_params);
					}
				}
			}
			if ($result === false && (!defined ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN') ) ) {
				define ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN', '1');
				opnErrorHandler (E_WARNING, 'mail() returned failure:'. $result, __FILE__, __LINE__);
			}
			return $result;

		}
		// end function

	}
	// end class
}

?>