<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_OPN_MAIL_SENDMAIL_INCLUDED') ) {
	define ('_OPN_CLASS_OPN_MAIL_SENDMAIL_INCLUDED', 1);

	class opn_mail_sendmail extends opn_mail {

		/**
		* The location of the sendmail or sendmail wrapper binary on the
		* filesystem.
		* @var string
		*/
		var $sendmail_path = '/usr/sbin/sendmail';

		/**
		* Any extra command-line parameters to pass to the sendmail or
		* sendmail wrapper binary.
		* @var string
		*/
		var $sendmail_args = '';

		/**
		* FUNCTION: opn_mail_sendmail
		* Constructor.
		*
		* Instantiates a new opn_mail_sendmail:: object based on the parameters
		* passed in. It looks for the following parameters:
		*	 sendmail_path	The location of the sendmail binary on the
		*					  filesystem. Defaults to '/usr/sbin/sendmail'.
		*
		*	 sendmail_args	Any extra parameters to pass to the sendmail
		*					  or sendmail wrapper binary.
		*
		* If a parameter is present in the $params array, it replaces the
		* default.
		*
		* @param array $params Hash containing any parameters different from the
		*			  defaults.
		* @access public
		*/

		function opn_mail_sendmail ($params) {
			if (isset ($params['sendmail_path']) ) {
				$this->sendmail_path = $params['sendmail_path'];
			}
			if (isset ($params['sendmail_args']) ) {
				$this->sendmail_args = $params['sendmail_args'];
			}

			/*
			* Because we need to pass message headers to the sendmail program on
			* the commandline, we can't guarantee the use of the standard "\r\n"
			* separator.  Instead, we use the system's native line separator.
			*/

			$this->sep = (strstr (PHP_OS, 'WIN') )?"\r\n" : "\n";

		}
		// end function

		/**
		* FUNCTION: send
		* Implements opn_mail_sendmail::send() function using the sendmail
		* command-line binary.
		*
		* @param mixed $recipients Either a comma-seperated list of recipients
		*			  (RFC822 compliant), or an array of recipients,
		*			  each RFC822 valid. This may contain recipients not
		*			  specified in the headers, for Bcc:, resending
		*			  messages, etc.
		*
		* @param array $headers The array of headers to send with the mail, in an
		*			  associative array, where the array key is the
		*			  header name (ie, 'Subject'), and the array value
		*			  is the header value (ie, 'test'). The header
		*			  produced from those values would be 'Subject:
		*			  test'.
		*
		* @param string $body The full text of the message body, including any
		*			   Mime parts, etc.
		*
		* @return mixed Returns true on success, or false on failure.
		*
		* @access public
		*/

		function send ($recipients, $headers, $body) {
			if ($recipients != '') {
				$recipients = $this->_parseRecipients ($recipients);
				$recipients = escapeshellcmd (implode (' ', $recipients) );
			} else {
				$recipients = 'Undisclosed-recipients:;';
				$recipients = escapeshellcmd ($recipients);
			}
			$headerElements = $this->_prepareHeaders ($headers);
			if (is_string ($headerElements) ) {
				opnErrorHandler (E_WARNING, $headerElements, __FILE__, __LINE__);
				return false;
			}
			list ($from, $text_headers) = $headerElements;
			if (!isset ($from) ) {
				opnErrorHandler (E_WARNING, 'No from address given.', __FILE__, __LINE__);
				return false;
			}
			if (strstr ($from, ' ') || strstr ($from, ';') || strstr ($from, '&') || strstr ($from, '`') ) {
				opnErrorHandler (E_WARNING, 'From address specified with dangerous characters', __FILE__, __LINE__);
				return false;
			}
			$result = 0;
			if (@is_file ($this->sendmail_path) ) {
				$from = escapeShellCmd ($from);
				$mail = popen ($this->sendmail_path . (!empty ($this->sendmail_args)?' ' . $this->sendmail_args : '') . "-f $from-- $recipients", 'w');
				fputs ($mail, $text_headers);
				fputs ($mail, $this->sep);
				// newline to end the headers section
				fputs ($mail, $body);
				$result = pclose ($mail) >> 8&0xFF;
				// need to shift the pclose result to get the exit code
			} else {
				opnErrorHandler (E_WARNING, 'sendmail [' . $this->sendmail_path . '] is not a valid file', __FILE__, __LINE__);
				return false;
			}
			if ($result != 0) {
				opnErrorHandler (E_WARNING, 'sendmail returned error code ' . $result, __FILE__, __LINE__);
				return false;
			}
			return true;

		}
		// end function

	}
	// end class
}

?>