<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_OPN_MAIL_SMTP_INCLUDED') ) {
	define ('_OPN_CLASS_OPN_MAIL_SMTP_INCLUDED', 1);

	class opn_mail_smtp extends opn_mail {

		/**
		* The SMTP host to connect to.
		* @var string
		*/
		var $host = 'localhost';

		/**
		* The port the SMTP server is on.
		* @var integer
		*/
		var $port = 25;

		/**
		* Should SMTP authentication be used?
		*
		* This value may be set to true, false or the name of a specific
		* authentication method.
		*
		* If the value is set to true, the Net_SMTP package will attempt to use
		* the best authentication method advertised by the remote SMTP server.
		*
		* @var mixed
		*/
		var $auth = false;

		/**
		* The username to use if the SMTP server requires authentication.
		* @var string
		*/
		var $username = '';

		/**
		* The password to use if the SMTP server requires authentication.
		* @var string
		*/
		var $password = '';

		/**
		* Hostname or domain that will be sent to the remote SMTP server in the
		* HELO / EHLO message.
		*
		* @var string
		*/
		var $localhost = 'localhost';

		/**
		* Whether to use VERP or not. If not a boolean, the string value
		* will be used as the VERP separators.
		*
		* @var mixed boolean or string
		*/
		var $verp = false;

		/**
		* Turn on Net_SMTP debugging?
		*
		* @var boolean $debug
		*/
		var $debug = false;

		/**
		* FUNCTION: opn_mail_smtp
		*  Constructor.
		*
		* Instantiates a new opn_mail_smtp:: object based on the parameters
		* passed in. It looks for the following parameters:
		*	 host		The server to connect to. Defaults to localhost.
		*	 port		The port to connect to. Defaults to 25.
		*	 auth		SMTP authentication.  Defaults to none.
		*	 username	The username to use for SMTP auth. No default.
		*	 password	The password to use for SMTP auth. No default.
		*	 localhost   The local hostname / domain. Defaults to localhost.
		*	 verp		Whether to use VERP or not. Defaults to false.
		*	 debug	   Activate SMTP debug mode? Defaults to false.
		*
		* If a parameter is present in the $params array, it replaces the
		* default.
		*
		* @param array $params Hash containing any parameters different from the
		*			  defaults.
		* @access public
		*/

		function opn_mail_smtp ($params) {
			if (isset ($params['host']) ) {
				$this->host = $params['host'];
			}
			if (isset ($params['port']) ) {
				$this->port = $params['port'];
			}
			if (isset ($params['auth']) ) {
				$this->auth = $params['auth'];
			}
			if (isset ($params['username']) ) {
				$this->username = $params['username'];
			}
			if (isset ($params['password']) ) {
				$this->password = $params['password'];
			}
			if (isset ($params['localhost']) ) {
				$this->localhost = $params['localhost'];
			}
			if (isset ($params['verp']) ) {
				$this->verp = $params['verp'];
			}
			if (isset ($params['debug']) ) {
				$this->debug = (boolean) $params['debug'];
			}

		}
		// end function

		/**
		* FUNCTION: send
		* Implements opn_mail_smtp::send() function using SMTP.
		*
		* @param mixed $recipients Either a comma-seperated list of recipients
		*			  (RFC822 compliant), or an array of recipients,
		*			  each RFC822 valid. This may contain recipients not
		*			  specified in the headers, for Bcc:, resending
		*			  messages, etc.
		*
		* @param array $headers The array of headers to send with the mail, in an
		*			  associative array, where the array key is the
		*			  header name (ie, 'Subject'), and the array value
		*			  is the header value (ie, 'test'). The header
		*			  produced from those values would be 'Subject:
		*			  test'.
		*
		* @param string $body The full text of the message body, including any
		* Mime parts, etc.
		*
		* @return mixed Returns true on success, or false on failure.
		*
		* @access public
		*/

		function send ($recipients, $headers, $body) {

			global $opnConfig;

			include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'net/class.opn_smtp.php');

			if (defined ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN') ) {
				return false;
			}
			
			if (! ($smtp =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opn_smtp ($this->host, $this->port, $this->localhost) ) ) {
				define ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN', '1');
				opnErrorHandler (E_WARNING, 'unable to instantiate opn_smtp object', __FILE__, __LINE__);
				return false;
			}
			if ($this->debug) {
				$smtp->setDebug (true);
			}
			if ( (isset($opnConfig['opn_activate_email_debug'])) && ($opnConfig['opn_activate_email_debug'] == 1) ) {
				$smtp->setDebug (true);
			}
			if (!$smtp->connectsmtp (5) ) {
				define ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN', '1');
				opnErrorHandler (E_WARNING, 'unable to connect to smtp server ' . $this->host . ':' . $this->port, __FILE__, __LINE__);
				return false;
			}
			if ($this->auth) {
				$method = is_string ($this->auth)? $this->auth : '';
				if (!$smtp->auth ($this->username, $this->password, $method) ) {
					$smtp->disconnect ();
					define ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN', '1');
					opnErrorHandler (E_WARNING, 'unable to authenticate to smtp server', __FILE__, __LINE__);
					return false;
				}
			}
			if ($recipients != '') {
				$recipients = $this->_parseRecipients ($recipients);
			} else {
				$recipients[] = 'Undisclosed-recipients:;';
			}
			if (empty ($headers['To']) ) {
				if (is_array ($recipients) ) {
					$headers['To'] = implode (', ', $recipients);
				} else {
					$headers['To'] = $recipients;
				}
			}
			$headerElements = $this->_prepareHeaders ($headers);
			if (is_string ($headerElements) ) {
				$smtp->disconnect ();
				define ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN', '1');
				opnErrorHandler (E_WARNING, $headerElements, __FILE__, __LINE__);
				return false;
			}
			list ($from, $text_headers) = $headerElements;
			/* Since few MTAs are going to allow this header to be forged
			* unless it's in the MAIL FROM: exchange, we'll use
			* Return-Path instead of From: if it's set. */
			if (!empty ($headers['Return-Path']) ) {
				$from = $headers['Return-Path'];
			}
			if (!isset ($from) ) {
				$smtp->disconnect ();
				define ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN', '1');
				opnErrorHandler (E_WARNING, 'No from address given', __FILE__, __LINE__);
				return false;
			}
			$args['verp'] = $this->verp;
			if (!$smtp->mailFrom ($from, $args) ) {
				$smtp->disconnect ();
				define ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN', '1');
				opnErrorHandler (E_WARNING, 'unable to set sender to [' . $from . ']', __FILE__, __LINE__);
				return false;
			}
			foreach ($recipients as $recipient) {
				if (!$smtp->rcptTo ($recipient) ) {
					$smtp->disconnect ();
					define ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN', '1');
					opnErrorHandler (E_WARNING, 'unable to add recipient [' . $recipient . ']', __FILE__, __LINE__);
					return false;
				}
			}
			if (!$smtp->data ($text_headers . "\r\n" . $body) ) {
				$smtp->disconnect ();
				define ('_OPNINTERNAL_DONOTTRYTOUSEMAILAGAIN', '1');
				opnErrorHandler (E_WARNING, 'unable to send data', __FILE__, __LINE__);
				return false;
			}
			$smtp->disconnect ();
			unset ($smtp);
			return true;

		}
		// end function

	}
	// end class
}

?>