<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_OPN_SOCKET_INCLUDED') ) {
	define ('_OPN_CLASS_OPN_SOCKET_INCLUDED', 1);
	define ('NET_SOCKET_READ', 1);
	define ('NET_SOCKET_WRITE', 2);
	define ('NET_SOCKET_ERROR', 3);

	/**
	* @abstract Defines a Wrapperclass for the access via fsockopen.
	*
	*/

	class opn_socket {

		/**
		* @var bool Whether the socket is blocking.
		* @access public
		*/
		var $blocking = true;

		/**
		* @var integer Number of seconds to wait on socket connections before
		*			  assuming there's no more data.
		* @access public
		*/
		var $timeout = false;

		/**
		* @var integer Number of bytes to read at a time in readLine() and
		*			  readAll().
		*/
		var $lineLength = 2048;

		/**
		* @var ressource Socket Filepointer
		* @access private
		*/
		var $_fp = null;

		/**
		* @var bool Whether the socket is persistent.
		* @access private
		*/
		var $_persistent = false;

		/**
		* @var string The IP address to connect to.
		* @access private
		*/
		var $_addr = '';

		/**
		* @var integer The port number to connect to.
		* @access private
		*/
		var $_port = 0;

		/**
		* FUNCTION: connectsocket
		* Connect to the specified port. If called when the socket is already
		* connected, it disconnects and connects again.
		*
		* @param string $adr IP address or host name
		* @param integer $port TCP port number
		* @param int (optional) $persisten whether the connection is
		* persistent (kept open between requests by the web server)
		* @param integer (optional) $timeout how long to wait for data
		* @param array $options See options for stream_context_create.
		* @return bool true on success otherwise false.
		* @access public
		*/

		function connectsocket ($addr, $port, $persistent = null, $timeout = null, $options = null) {
			if (is_resource ($this->_fp) ) {
				@fclose ($this->_fp);
				$this->_fp = null;
			}
			if (!$addr) {
				return false;
			}
			if (strspn ($addr, '.0123456789') == strlen ($addr) ) {
				$this->_addr = $addr;
			} else {
				$this->_addr = gethostbyname ($addr);
			}
			$this->_port = $port%65536;
			if ($persistent !== null) {
				$this->_persistent = $persistent;
			}
			if ($timeout !== null) {
				$this->timeout = $timeout;
			}
			$errno = 0;
			// Alex: binnicht sicher obs passt...
			$openfunc = $this->_persistent?'pfsockopen' :  'fsockopen';
			$errstr = '';
			if ($options && function_exists ('stream_context_create') ) {
				if ($this->timeout) {
					$timeout = $this->timeout;
				} else {
					$timeout = 0;
				}
				$context = stream_context_create ($options);
				$fp = @ $openfunc ($this->_addr, $this->_port, $errno, $errstr, $timeout, $context);
			} else {
				if ($this->timeout) {
					$fp = @ $openfunc ($this->_addr, $this->_port, $errno, $errstr, $this->timeout);
				} else {
					$fp = @ $openfunc ($this->_addr, $this->_port, $errno, $errstr);
				}
			}
			if (!$fp) {
				opnErrorHandler (E_WARNING, $errstr, __FILE__, __LINE__);
				return false;
			}
			$this->_fp = $fp;
			return $this->setBlocking ($this->blocking);

		}
		// end function

		/**
		* FUNCTION: disconnect
		* Disconnects from the peer, closes the socket.
		*
		* @return bool true on success otherwise false.
		* @access public
		*/

		function disconnect () {
			if (is_resource ($this->_fp) ) {
				fclose ($this->_fp);
				$this->_fp = null;
				return true;
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: isBlocking
		* Find out if the socket is in blocking mode.
		*
		* @return bool the current blocking mode.
		* @access public
		*/

		function isBlocking () {
			return $this->blocking;

		}
		// end function

		/**
		* FUNCTION: setBlocking
		* Sets whether the socket connection should be blocking or not. A read
		* call to a non-blocking socket will return immediately if there is no
		* data available, whereas it will block until there is data for
		* blocking sockets.
		*
		* @param bool $mode true for blocking sockets, false for nonblocking
		* @return bool true on success otherwise false.
		* @access public
		*/

		function setBlocking ($mode) {
			if (is_resource ($this->_fp) ) {
				$this->blocking = $mode;
				socket_set_blocking ($this->_fp, $this->blocking);
				return true;
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: setTimeout
		* Sets the timeout value on socket descriptor, expressed in the sum of
		* seconds and microseconds
		*
		* @param int $seconds seconds
		* @param int $microseconds microseconds
		* @return bool true on success otherwise false.
		* @access public
		*/

		function setTimeout ($seconds, $microseconds) {
			if (is_resource ($this->_fp) ) {
				socket_set_timeout ($this->_fp, $seconds, $microseconds);
				return true;
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: getStatus
		* Currently returns four entries in the result array:
		*
		* <p>
		* timed_out (bool) - The socket timed out waiting for data<br>
		* blocked (bool) - The socket was blocked<br>
		* eof (bool) - Indicates EOF event<br>
		* unread_bytes (int) - Number of bytes left in the socket buffer<br>
		* </p>
		*
		* @return mixed Array containing information about existing socket
		* resource or an false otherwise.
		* @access public
		*/

		function getStatus () {
			if (is_resource ($this->_fp) ) {
				return socket_get_status ($this->_fp);
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: gets
		* Get a specified line of data
		*
		* @param integer $size How many bytes to read.
		* @return mixed $size bytes of data from the socket, or false if not
		* connected.
		* @access public
		*/

		function gets ($size) {
			if (is_resource ($this->_fp) ) {
				return @fgets ($this->_fp, $size);
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: read
		* Read a specified amount of data. This is guaranteed to return,
		* and has the added benefit of getting everything in one fread()
		* chunk; if you know the size of the data you're getting
		* beforehand, this is definitely the way to go.
		*
		* @param integer $size The number of bytes to read from the socket.
		* @return mixed $size bytes of data from the socket, or false if not
		* connected.
		* @access public
		*/

		function read ($size) {
			if (is_resource ($this->_fp) ) {
				return @fread ($this->_fp, $size);
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: write
		* Write a specified amount of data.
		*
		* @param mixed $data The Data for writing.
		* @return bool true on success or false otherwise.
		* @access public
		*/

		function write ($data) {
			if (is_resource ($this->_fp) ) {
				return fwrite ($this->_fp, $data);
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: writeLine
		* Write a line of data to the socket, followed by a trailing "\r\n".
		*
		* @param mixed $data The Data for writing.
		* @return bool true on success or false otherwise.
		*/

		function writeLine ($data) {
			if (is_resource ($this->_fp) ) {
				return fwrite ($this->_fp, $data . "\r\n");
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: eof
		* Tests for end-of-file on a socket descriptor.
		*
		* @return bool
		* @access public
		*/

		function eof () {
			return (is_resource ($this->_fp) && feof ($this->_fp) );

		}
		// end function

		/**
		* FUNCTION: readByte
		* Reads a byte of data
		*
		* @return mixed 1 byte of data from the socket, or false if not
		* connected.
		* @access public
		*/

		function readByte () {
			if (is_resource ($this->_fp) ) {
				return ord (@fread ($this->_fp, 1) );
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: readWord
		* Reads a word of data
		*
		* @return mixed 1 word of data from the socket, or false if not
		* connected.
		* @access public
		*/

		function readWord () {
			if (is_resource ($this->_fp) ) {
				$buf = @fread ($this->_fp, 2);
				return (ord ($buf[0])+ (ord ($buf[1]) << 8) );
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: readInt
		* Reads an int of data
		*
		* @return mixed 1 int of data from the socket, or false if not
		* connected.
		* @access public
		*/

		function readInt () {
			if (is_resource ($this->_fp) ) {
				$buf = @fread ($this->_fp, 4);
				return (ord ($buf[0])+ (ord ($buf[1]) << 8)+ (ord ($buf[2]) << 16)+ (ord ($buf[3]) << 24) );
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: readString
		* Reads a zeroterminated string of data
		*
		* @return mixed string, or false if not connected.
		* @access public
		*/

		function readString () {
			if (is_resource ($this->_fp) ) {
				$string = '';
				while ( ($char = @fread ($this->_fp, 1) ) != "\x00") {
					$string .= $char;
				}
				return $string;
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: readIPAddress
		* Reads an IP Address and returns it in a dot formated string
		*
		* @return mixed Dot formated string, or false if not connected.
		* @access private
		*/

		function readIPAddress () {
			if (is_resource ($this->_fp) ) {
				$buf = @fread ($this->_fp, 4);
				return sprintf ("%s.%s.%s.%s", ord ($buf[0]), ord ($buf[1]), ord ($buf[2]), ord ($buf[3]) );
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: readLine
		* Read until either the end of the socket or a newline, whichever
		* comes first. Strips the trailing newline from the returned data.
		*
		* @return mixed All available data up to a newline, without that
		* newline, or until the end of the socket, or false if not connected.
		* @access public
		*/

		function readLine () {
			if (is_resource ($this->_fp) ) {
				$line = '';
				$timeout = time ()+ $this->timeout;
				while (!feof ($this->_fp) && (! $this->timeout || time ()< $timeout) ) {
					$line .= @fgets ($this->_fp, $this->lineLength);
					if (substr ($line, -2) == "\r\n" || substr ($line, -1) == "\n") {
						return rtrim ($line, "\r\n");
					}
				}
				return $line;
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: readAll
		* Read until the socket closes. THIS FUNCTION WILL NOT EXIT if the
		* socket is in blocking mode until the socket closes.
		*
		* @return mixed All data until the socket closes, or false if not
		* connected.
		* @access public
		*/

		function readAll () {
			if (is_resource ($this->_fp) ) {
				$data = '';
				while (!feof ($this->_fp) ) {
					$data .= @fread ($this->_fp, $this->lineLength);
				}
				return $data;
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: select
		* Runs the equivalent of the select() system call on the socket
		* with a timeout specified by tv_sec and tv_usec.
		*
		* @param integer $state	Which of read/write/error to check for.
		* @param integer $tv_sec   Number of seconds for timeout.
		* @param integer $tv_usec  Number of microseconds for timeout.
		* @return mixed False if select fails, integer describing which of
		* read/write/error are ready, or false if not connected.
		*/

		function select ($state, $tv_sec, $tv_usec = 0) {
			if (is_resource ($this->_fp) ) {
				$read = null;
				$write = null;
				$except = null;
				if ($state&NET_SOCKET_READ) {
					$read[] = $this->_fp;
				}
				if ($state&NET_SOCKET_WRITE) {
					$write[] = $this->_fp;
				}
				if ($state&NET_SOCKET_ERROR) {
					$except[] = $this->_fp;
				}
				if (false === ($sr = stream_select ($read, $write, $except, $tv_sec, $tv_usec) ) ) {
					return false;
				}
				$result = 0;
				if (count ($read) ) {
					$result |= NET_SOCKET_READ;
				}
				if (count ($write) ) {
					$result |= NET_SOCKET_WRITE;
				}
				if (count ($except) ) {
					$result |= NET_SOCKET_ERROR;
				}
				return $result;
			}
			return false;

		}
		// end function

	}
	// end class
}

?>