<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_SQLLAYER_INCLUDED') ) {
	define ('_OPN_CLASS_SQLLAYER_INCLUDED', 1);

	/**
	* Load the needed layerclass
	*
	* @access public
	* @param string $db The driver to be included.
	* @return object
	*/

	function &loadSQLLayer ($db) {

		$db = strtolower ($db);
		switch ($db) {
			case 'ifx':
			case 'maxsql':
				$db = 'mysqlt';
				break;
			case 'postgres':
			case 'postgres8':
			case 'pgsql':
				$db = 'postgres7';
				break;
		}
		$class = 'SQLLayer_' . $db;
		@include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'sql/drivers/driver-' . $db . '.php');
		if (!class_exists ($class) ) {
			opnErrorHandler (E_WARNING, 'Couldn\'t find class ' . $class, __FILE__, __LINE__);
		}
		$errorfn = (defined ('SQLLAYER_ERROR_HANDLER') )?SQLLAYER_ERROR_HANDLER : false;
		$obj =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ $class ();
		if ($obj) {
			if ($errorfn) {
				$obj->raiseErrorFn = $errorfn;
			}
		}
		return $obj;

	}

	/**
	* Error output if a Transaction will not start.
	*
	* @param unknown	$dbms
	* @param unknown	$fn
	* @param unknown	$errno
	* @param string	$errmsg
	* @param unknown	$p1
	* @param unknown	$p2
	* @param object	$thisConnection
	* @return void
	*/

	function SQLLayer_TransMonitor ($dbms, $fn, $errno, $errmsg, $p1, $p2, &$thisConnection) {
		// print "Errorno ($fn errno=$errno m=$errmsg) ";
		$thisConnection->_transOK = false;
		if ($thisConnection->_oldRaiseFn) {
			$fn = $thisConnection->_oldRaiseFn;
			$fn ($dbms, $fn, $errno, $errmsg, $p1, $p2, $thisConnection);
		}

	}

	/**
	* SQL Connection class (will be extended for db specific use)
	*
	*
	*/

	class SQLConnection {

		/*<{php4}>
		var $dataProvider = 'native';
		var $databaseType = '';		/// RDBMS currently in use, eg. odbc, mysql, mssql
		var $database = '';			/// Name of database to be used.
		var $host = ''; 			/// The hostname of the database server
		var $user = ''; 			/// The username which is used to connect to the database server.
		var $password = ''; 		/// Password for the username. For security, we no longer store it.
		var $debug = false; 		/// if set to true will output sql statements
		var $maxblobsize = 262144; 	/// maximum size of blobs or large text fields (262144 = 256K)-- some db's die otherwise like foxpro
		var $true = '1'; 			/// string that represents TRUE for a database
		var $false = '0'; 			/// string that represents FALSE for a database
		var $replaceQuote = "\\'"; 	/// string to use to replace quotes
		var $charSet=false; 		/// character set to use - only for interbase, postgres and oci8
		var $forceNewConnect = false;
		var $metaDatabasesSQL = '';
		var $metaTablesSQL = '';
		//--
		var $hasTop = false;			/// support mssql/access SELECT TOP 10 * FROM TABLE
		//--
		var $raiseErrorFn = false; 	/// error function to call
		var $arrayClass = 'SQLLayerRecordSet_array'; /// name of class used to generate array recordsets, which are pre-downloaded recordsets

		var $autoRollback = false; // autoRollback on PConnect().
		var $metaColumnsSQL = '';
		var $fnExecute = false;
		var $blobEncodeType = false; // false=not required, 'I'=encode to integer, 'C'=encode to char
		var $rsPrefix = 'SQLLayerRecordSet_';
		var $autoCommit = true; 	/// do not modify this yourself - actually private
		var $transOff = 0; 			/// temporarily disable transactions
		var $transCnt = 0; 			/// count of nested transactions
		var $fetchMode=true;
		var $isInterbase = false;

		//
		// PRIVATE VARS
		//
		var $_oldRaiseFn =  false;
		var $_transOK = null;
		var $_connectionID = false;	/// The returned link identifier whenever a successful database connection is made.
		var $_errorMsg = false;		/// A variable which was used to keep the returned last error message.  The value will
		/// then returned by the errorMsg() function
		var $_errorCode = false;	/// Last error code, not guaranteed to be used - only by oci8
		var $_queryID = false;		/// This variable keeps the last created result link identifier

		var $_isPersistentConnection = false;	/// A boolean variable to state whether its a persistent connection or normal connection.
		var $_bindInputArray = false; /// set to true if SQLConnection.Execute() permits binding of array parameters.
		var $_evalAll = false;
		var $_affected = false;
		<{php4/}>*/

		/*<{php5}>
		public $dataProvider = 'native';
		public $databaseType = '';		/// RDBMS currently in use, eg. odbc, mysql, mssql
		public $database = '';			/// Name of database to be used.
		public $host = ''; 			/// The hostname of the database server
		public $user = ''; 			/// The username which is used to connect to the database server.
		public $password = ''; 		/// Password for the username. For security, we no longer store it.
		public $debug = false; 		/// if set to true will output sql statements
		public $maxblobsize = 262144; 	/// maximum size of blobs or large text fields (262144 = 256K)-- some db's die otherwise like foxpro
		public $true = '1'; 			/// string that represents TRUE for a database
		public $false = '0'; 			/// string that represents FALSE for a database
		public $replaceQuote = "\\'"; 	/// string to use to replace quotes
		public $charSet=false; 		/// character set to use - only for interbase, postgres and oci8
		public $forceNewConnect = false;
		public $metaDatabasesSQL = '';
		public $metaTablesSQL = '';
		//--
		public $hasTop = false;			/// support mssql/access SELECT TOP 10 * FROM TABLE
		//--
		public $raiseErrorFn = false; 	/// error function to call
		public $arrayClass = 'SQLLayerRecordSet_array'; /// name of class used to generate array recordsets, which are pre-downloaded recordsets

		public $autoRollback = false; // autoRollback on PConnect().
		public $metaColumnsSQL = '';
		public $fnExecute = false;
		public $blobEncodeType = false; // false=not required, 'I'=encode to integer, 'C'=encode to char
		public $rsPrefix = 'SQLLayerRecordSet_';
		public $autoCommit = true; 	/// do not modify this yourself - actually private
		public $transOff = 0; 			/// temporarily disable transactions
		public $transCnt = 0; 			/// count of nested transactions
		public $fetchMode=true;

		//
		// PRIVATE VARS
		//
		public $_oldRaiseFn =  false;
		public $_transOK = null;
		public $_connectionID = false;	/// The returned link identifier whenever a successful database connection is made.
		public $_errorMsg = false;		/// A variable which was used to keep the returned last error message.  The value will
		/// then returned by the errorMsg() function
		public $_errorCode = false;	/// Last error code, not guaranteed to be used - only by oci8
		public $_queryID = false;		/// This variable keeps the last created result link identifier

		public $_isPersistentConnection = false;	/// A boolean variable to state whether its a persistent connection or normal connection.
		public $_bindInputArray = false; /// set to true if SQLConnection.Execute() permits binding of array parameters.
		public $_evalAll = false;
		public $_affected = false;
		public $isInterbase = false;
		<{php5/}>*/

		/*<{delete}>*/
		var $dataProvider = 'native';
		var $databaseType = '';		/// RDBMS currently in use, eg. odbc, mysql, mssql
		var $database = '';			/// Name of database to be used.
		var $host = ''; 			/// The hostname of the database server
		var $user = ''; 			/// The username which is used to connect to the database server.
		var $password = ''; 		/// Password for the username. For security, we no longer store it.
		var $debug = false; 		/// if set to true will output sql statements
		var $maxblobsize = 262144; 	/// maximum size of blobs or large text fields (262144 = 256K)-- some db's die otherwise like foxpro
		var $true = '1'; 			/// string that represents TRUE for a database
		var $false = '0'; 			/// string that represents FALSE for a database
		var $replaceQuote = "\\'"; 	/// string to use to replace quotes
		var $charSet=false; 		/// character set to use - only for interbase, postgres and oci8
		var $forceNewConnect = false;
		var $metaDatabasesSQL = '';
		var $metaTablesSQL = '';
		//--
		var $hasTop = false;			/// support mssql/access SELECT TOP 10 * FROM TABLE
		//--
		var $raiseErrorFn = false; 	/// error function to call
		var $arrayClass = 'SQLLayerRecordSet_array'; /// name of class used to generate array recordsets, which are pre-downloaded recordsets

		var $autoRollback = false; // autoRollback on PConnect().
		var $metaColumnsSQL = '';
		var $fnExecute = false;
		var $blobEncodeType = false; // false=not required, 'I'=encode to integer, 'C'=encode to char
		var $rsPrefix = 'SQLLayerRecordSet_';
		var $autoCommit = true; 	/// do not modify this yourself - actually private
		var $transOff = 0; 			/// temporarily disable transactions
		var $transCnt = 0; 			/// count of nested transactions
		var $fetchMode=true;
		var $isInterbase = false;

		//
		// PRIVATE VARS
		//
		var $_oldRaiseFn =  false;
		var $_transOK = null;
		var $_connectionID = false;	/// The returned link identifier whenever a successful database connection is made.
		var $_errorMsg = false;		/// A variable which was used to keep the returned last error message.  The value will
		/// then returned by the errorMsg() function
		var $_errorCode = false;	/// Last error code, not guaranteed to be used - only by oci8
		var $_queryID = false;		/// This variable keeps the last created result link identifier

		var $_isPersistentConnection = false;	/// A boolean variable to state whether its a persistent connection or normal connection.
		var $_bindInputArray = false; /// set to true if SQLConnection.Execute() permits binding of array parameters.
		var $_evalAll = false;
		var $_affected = false;
		/*<{delete/}>*/

		/**
		* Constructor for SQLConnection
		*
		* @return void
		*/

		function SQLConnection () {

			die ('Virtual Class -- cannot instantiate');

		}

		/**
		* Connect to the database
		*
		* must be overwriten in the layer child class
		*
		* @access private
		* @return void
		*/

		function _connect () {

		}

		/**
		* Persistant connect to the database.
		*
		* must be overwriten in the layer child class
		*
		* @access private
		* @return void
		*/

		function _pconnect () {

		}

		/**
		* Execute the query.
		*
		* must be overwriten in the layer child class
		*
		* @access private
		* @param $sql
		* @param $arr
		* @return void
		*/

		function _query ($sql, $arr) {

			$t = $sql;
			$t = $arr;

		}

		/**
		* Get the number of rows affected by last query
		* must be overwriten in the layer child class
		*
		* @access private
		* @return integer
		*/

		function _affectedrows () {
			return 0;

		}

		/**
		* Close the connection to database
		* must be overwriten in the layer child class
		*
		* @access private
		* @return void
		*/

		function _close () {

		}

		/**
		*
		* @access private
		* @param $str
		* @return string
		*/

		function _findvers ($str) {
			$arr = '';
			if (preg_match ('/([0-9]+\.([0-9\.])+)/', $str, $arr) ) {
				return $arr[1];
			}
			return '';

		}

		/**
		* returns the state of connection
		*
		* @access public
		* @return boolean
		*/

		function IsConnected () {
			return !empty ($this->_connectionID);

		}

		/**
		* Begin a Transaction.
		*
		* Must be followed by CommitTrans() or RollbackTrans().
		* true if succeeded or false if database does not support transactions
		*
		* @access public
		* @return boolean
		*/

		function BeginTrans () {
			return false;

		}

		/**
		* Commit the transaction.
		*
		* If database does not support transactions, always return true as data always commited
		*
		* @access public
		* @param boolean	$ok	- set to false to rollback transaction, true to commit
		* @return boolean
		*/

		function CommitTrans ($ok = true) {

			$t = $ok;
			return true;

		}

		/**
		* Rollback the transaction
		*
		* If database does not support transactions, rollbacks always fail, so return false
		*
		* @access public
		* @return boolean
		*/

		function RollbackTrans () {
			return false;

		}

		/**
		* Rollback the transaction
		*
		* If database does not support transactions, rollbacks always fail, so return false
		*
		* @access public
		* @return boolean
		*/

		function SetForceNewConnect ($force) {

			$this->forceNewConnect = $force;

		}

		/**
		* Lock a row, will escalate and lock the table if row locking not supported
		* will normally free the lock at the end of the transaction
		*
		* @access public
		* @param string	$table	name of table to lock
		* @param string	$where	where clause to use, eg: "WHERE row=12". If left empty, will escalate to table lock
		* @return boolean
		*/

		function RowLock ($table, $where) {

			$t = $table;
			$t = $where;
			$this->CommitTrans ();
			return false;

		}

		/**
		* Commit a lock.
		*
		* @access public
		* @param string	$table	- name of the table to lock
		* @return boolean
		*/

		function CommitLock ($table) {

			$t = $table;
			return $this->CommitTrans ();

		}

		/**
		* Rollback a lock.
		*
		* @access public
		* @param string	$table
		* @return boolean
		*/

		function RollbackLock ($table) {

			$t = $table;
			return $this->RollbackTrans ();

		}

		/**
		* Connect to the SQL Server.
		*
		* @access private
		* @param string	$argHostname		Host to connect to
		* @param string	$argUsername		Userid to login
		* @param string	$argPassword		Associated password
		* @param string	$argDatabaseName	Database
		* @return boolean
		*/

		function _nconnect ($argHostname, $argUsername, $argPassword, $argDatabaseName) {
			return $this->_connect ($argHostname, $argUsername, $argPassword, $argDatabaseName);

		}

		/**
		* Returns the last errormessage
		*
		* @access public
		* @return string
		*/

		function ErrorMsg () {
			return '!! ' . strtoupper ($this->dataProvider . ' ' . $this->databaseType) . ': ' . $this->_errorMsg;

		}

		/**
		* Returns the last errornumber. Normally 0 means no error.
		*
		* @access public
		* @return number
		*/

		function ErrorNo () {
			return ($this->_errorMsg)?-1 : 0;

		}

		/**
		* Establish a connection to a SQL Server.
		*
		* @access public
		* @param string	$argHostname		Host to connect to
		* @param string	$argUsername		Userid to login
		* @param string	$argPassword		Associated password
		* @param string	$argDatabaseName	Database
		* @param string	$forceNew			Force new connection
		* @return boolean
		*/

		function Connect ($argHostname = '', $argUsername = '', $argPassword = '', $argDatabaseName = '', $forceNew = false) {
			if ($argHostname != '') {
				$this->host = $argHostname;
			}
			if ($argUsername != '') {
				$this->user = $argUsername;
			}
			if ($argPassword != '') {
				$this->password = $argPassword;
				// not stored for security reasons
			}
			if ($argDatabaseName != '') {
				$this->database = $argDatabaseName;
			}
			$this->_isPersistentConnection = false;
			if ($forceNew) {
				if ($this->_nconnect ($this->host, $this->user, $this->password, $this->database) ) {
					return true;
				}
			} else {
				if ($this->_connect ($this->host, $this->user, $this->password, $this->database) ) {
					return true;
				}
			}
			$err = $this->ErrorMsg ();
			if (empty ($err) ) {
				$err = 'Connection error to server "' . $argHostname . '" with user "' . $argUsername . '"';
			}
			if ($fn = $this->raiseErrorFn) {
				$fn ($this->databaseType, 'CONNECT', $this->ErrorNo (), $err, $this->host, $this->database, $this);
			}
			$this->_connectionID = false;
			return false;

		}

		/**
		* Always force a new connection to database - currently only works with oracle
		*
		* @access public
		* @param string	$argHostname		Host to connect to
		* @param string	$argUsername		Userid to login
		* @param string	$argPassword		Associated password
		* @param string	$argDatabaseName	Database
		* @return boolean
		*/

		function NConnect ($argHostname = '', $argUsername = '', $argPassword = '', $argDatabaseName = '') {
			return $this->Connect ($argHostname, $argUsername, $argPassword, $argDatabaseName, true);

		}

		/**
		* Establish persistent connect to database
		*
		* @access public
		* @param string	$argHostname		Host to connect to
		* @param string	$argUsername		Userid to login
		* @param string	$argPassword		Associated password
		* @param string	$argDatabaseName	Database
		* @return boolean
		*/

		function PConnect ($argHostname = '', $argUsername = '', $argPassword = '', $argDatabaseName = '') {
			if (defined ('SQLAYER_NEVER_PERSIST') ) {
				return $this->Connect ($argHostname, $argUsername, $argPassword, $argDatabaseName);
			}
			if ($argHostname != '') {
				$this->host = $argHostname;
			}
			if ($argUsername != '') {
				$this->user = $argUsername;
			}
			if ($argPassword != '') {
				$this->password = $argPassword;
			}
			if ($argDatabaseName != '') {
				$this->database = $argDatabaseName;
			}
			$this->_isPersistentConnection = true;
			if ($this->_pconnect ($this->host, $this->user, $this->password, $this->database) ) {
				return true;
			}
			$err = $this->ErrorMsg ();
			if (empty ($err) ) {
				$err = 'Connection error to server "' . $argHostname . '" with user "' . $argUsername . '"';
			}
			if ($fn = $this->raiseErrorFn) {
				$fn ($this->databaseType, 'PCONNECT', $this->ErrorNo (), $err, $this->host, $this->database, $this);
			}
			$this->_connectionID = false;
			return false;

		}

		/**
		* Improved method of initiating a transaction. Used together with CompleteTrans().
		* Advantages include:
		*
		* a. StartTrans/CompleteTrans is nestable, unlike BeginTrans/CommitTrans/RollbackTrans.
		* 	  Only the outermost block is treated as a transaction.<br />
		* b. CompleteTrans auto-detects SQL errors, and will rollback on errors, commit otherwise.<br />
		* c. All BeginTrans/CommitTrans/RollbackTrans inside a StartTrans/CompleteTrans block
		* 	  are disabled, making it backward compatible.
		*
		* @access public
		* @param string	$errfn	The errorfunction that will be called when an error occurs.
		* @return void
		*/

		function StartTrans ($errfn = 'SQLLayer_TransMonitor') {
			if ($this->transOff>0) {
				$this->transOff++;
				return true;
			}
			$this->_oldRaiseFn = $this->raiseErrorFn;
			$this->raiseErrorFn = $errfn;
			$this->_transOK = true;
			if ($this->transCnt>0) {
				opnErrorHandler (E_WARNING, 'Bad Transaction: StartTrans called within BeginTrans', __FILE__, __LINE__);
				return false;
			}
			$this->BeginTrans ();
			$this->transOff = 1;
			return true;

		}

		/**
		* Used together with StartTrans() to end a transaction. Monitors connection
		* for sql errors, and will commit or rollback as appropriate.
		*
		* @access public
		* @param boolean	$autoComplete if true, monitor sql errors and commit and rollback as appropriate,
		* 			and if set to false force rollback even if no SQL error detected.
		* @return boolean
		*/

		function CompleteTrans ($autoComplete = true) {
			if ($this->transOff>1) {
				$this->transOff -= 1;
				return true;
			}
			$this->raiseErrorFn = $this->_oldRaiseFn;
			$this->transOff = 0;
			if ($this->_transOK && $autoComplete) {
				if (!$this->CommitTrans () ) {
					$this->_transOK = false;
					opnErrorHandler (E_WARNING, 'Smart Commit failed.', __FILE__, __LINE__);
				}
			} else {
				$this->RollbackTrans ();
			}
			return $this->_transOK;

		}

		/**
		* At the end of a StartTrans/CompleteTrans block, perform a rollback.
		*
		* @access public
		* @return void
		*/

		function FailTrans () {
			if ($this->transOff == 0) {
				opnErrorHandler (E_WARNING, 'FailTrans outside StartTrans/CompleteTrans', __FILE__, __LINE__);
			}
			$this->_transOK = false;

		}

		/**
		* Check if transaction has failed, only for Smart Transactions.
		*
		* @access public
		* @return boolean
		*/

		function HasFailedTrans () {
			if ($this->transOff>0) {
				return $this->_transOK == false;
			}
			return false;

		}

		/**
		* Convert database recordset to an array recordset
		* input recordset's cursor should be at beginning, and
		* old $rs will be closed.
		*
		* @access private
		* @param object	$rs		the recordset to copy
		* @param int		$nrows	number of rows to retrieve (optional)
		* @param int		$offset	offset by number of rows (optional)
		* @param boolean	$close
		* @return object
		*/

		function &_rs2rs (&$rs, $nrows = -1, $offset = -1, $close = true) {
			if (!$rs) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$dbtype = $rs->databaseType;
			if (!$dbtype) {
				$rs = &$rs;
				// required to prevent crashing in 4.2.1, but does not happen in 4.3.1 -- why ?
				return $rs;
			}
			if ( ($dbtype == 'array' || $dbtype == 'csv') && $nrows == -1 && $offset == -1) {
				$rs->MoveFirst ();
				$rs = &$rs;
				// required to prevent crashing in 4.2.1, but does not happen in 4.3.1-- why ?
				return $rs;
			}
			$flds = array ();
			$max = $rs->FieldCount ();
			for ($i = 0; $i< $max; $i++) {
				$flds[] = $rs->FetchField ($i);
			}
			$arr = &$rs->GetArrayLimit ($nrows, $offset);
			if ($close) {
				$rs->Close ();
			}
			$arrayClass = $this->arrayClass;
			$rs2 =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ $arrayClass ();
			$rs2->connection = &$this;
			$rs2->sql = $rs->sql;
			$rs2->dataProvider = $this->dataProvider;
			$rs2->InitArrayFields ($arr, $flds);

			# die unsets? wirklich sinnvoll?

			unset ($arr);
			unset ($flds);
			return $rs2;

		}

		/**
		* Execute a query
		*
		* @access private
		* @param string	$sql
		* @param mixed		$inputarr
		* @return object
		*/

		function &_Execute ($sql, $inputarr = false) {
			if ($this->debug) {
				include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'sql/sqllayer-lib.php');
				$this->_queryID = _sqllayer_debug_execute ($this, $sql, $inputarr);
			} else {
				$this->_queryID = @ $this->_query ($sql, $inputarr);
			}
			if ($this->_queryID === false) {
				// error handling if query fails
				$fn = $this->raiseErrorFn;
				if ($fn) {
					$fn ($this->databaseType, 'EXECUTE', $this->ErrorNo (), $this->ErrorMsg (), $sql, $inputarr, $this);
				}

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			if ($this->_queryID === true) {
				// return simplified recordset for inserts/updates/deletes with lower overhead
				$rs =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerRecordSet_empty ();
				$this->_errorMsg = '';
				return $rs;
			}
			// return real recordset from select statement
			$rsclass = $this->rsPrefix . $this->databaseType;
			$rs =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ $rsclass ($this->_queryID, $this->fetchMode);
			$rs->connection = &$this;
			// Pablo suggestion
			$rs->Init ();
			if (is_array ($sql) ) {
				$rs->sql = $sql[0];
			} else {
				$rs->sql = $sql;
			}
			if ($rs->_numOfRows<=0) {
				if (!$rs->EOF) {
					$rs = &$this->_rs2rs ($rs, -1, -1, !is_array ($sql) );
					$rs->_queryID = $this->_queryID;
				}
			}
			return $rs;

		}

		/**
		* Should prepare the sql statement and return the stmt resource.
		*
		* For databases that do not support this, we return the $sql. To ensure
		* compatibility with databases that do not support prepare:
		*
		*	$stmt = $db->Prepare("insert into table (id, name) values (?,?)");
		*	$db->Execute($stmt,array(1,'Jill')) or die('insert failed');
		*	$db->Execute($stmt,array(2,'Joe')) or die('insert failed');
		*
		* @access private
		* @param string	$sql	SQL to send to database
		* @return mixed	FALSE, or the prepared statement, or the original sql if
		* 					the database does not support prepare.
		*/

		function Prepare ($sql) {
			return $sql;

		}

		/**
		* Correctly quotes a string so that all strings are escaped. We prefix and append
		* to the string single-quotes.
		* An example is  $db->qstr("Don't bother",magic_quotes_runtime());
		*
		* @access public
		* @param string	$s			the string to quote
		* @param boolean	$magic_quotes	if $s is GET/POST var, set to get_magic_quotes_gpc().
		* 									his undoes the stupidity of magic quotes for GPC.
		* @return string
		*/

		function qstr ($s, $magic_quotes = false) {
			if (!$magic_quotes) {
				if ($this->replaceQuote[0] == '\\') {
					// only since php 4.0.5
					$s = str_replace (array ('\\',
								"\0"),
								array ('\\\\',
						"\\\0"),
						$s);
				}
				return "'" . str_replace ("'", $this->replaceQuote, $s) . "'";
			}
			// undo magic quotes for "
			$s = str_replace ('\\"', '"', $s);
			if ($this->replaceQuote == "\\'") {
				// ' already quoted, no need to change anything
				return "'$s'";
			}
			// change \' to '' for sybase/mssql
			$s = str_replace ('\\\\', '\\', $s);
			return "'" . str_replace ("\\'", $this->replaceQuote, $s) . "'";

		}

		/**
		* Execute SQL
		*
		* @access public
		* @param string	$sql		SQL Query to execute
		* @param mixed		$inputarr
		* @return object
		*/

		function &Execute ($sql, $inputarr = false) {
			if ($this->fnExecute) {
				$fn = $this->fnExecute;
				$ret = &$fn ($this, $sql, $inputarr);
				if (isset ($ret) ) {
					return $ret;
				}
			}
			if ($inputarr) {
				if (!is_array ($inputarr) ) {
					$inputarr = array ($inputarr);
				}
				$element0 = reset ($inputarr);
				// is_object check because oci8 descriptors can be passed in
				$array_2d = is_array ($element0) && !is_object (reset ($element0) );
				if (!is_array ($sql) && !$this->_bindInputArray) {
					$sqlarr = explode ('?', $sql);
					if (!$array_2d) {
						$inputarr = array ($inputarr);
					}
					foreach ($inputarr as $arr) {
						$sql = '';
						$i = 0;
						foreach ($arr as $v) {
							$sql .= $sqlarr[$i];
							// from Ron Baldwin <ron.baldwin#sourceprose.com>
							// Only quote string types
							$type = gettype ($v);
							if ($type == 'string') {
								$sql .= $this->qstr ($v);
							} elseif ($type == 'double') {
								$sql .= str_replace (',', '.', $v);
								// locales fix so 1.1 does not get converted to 1,1
							} elseif ($v === null) {
								$sql .= 'NULL';
							} else {
								$sql .= $v;
							}
							$i += 1;
						}
						if (isset ($sqlarr[$i]) ) {
							$sql .= $sqlarr[$i];
							if ($i+1 != sizeof ($sqlarr) ) {
								opnErrorHandler (E_WARNING, 'Input Array does not match ?: ' . $sql, __FILE__, __LINE__);
							}
						} elseif ($i != sizeof ($sqlarr) ) {
							opnErrorHandler (E_WARNING, 'Input Array does not match ?: ' . $sql, __FILE__, __LINE__);
						}
						$ret = &$this->_Execute ($sql);
						if (!$ret) {
							return $ret;
						}
					}
				} else {
					if ($array_2d) {
						$stmt = $this->Prepare ($sql);
						foreach ($inputarr as $arr) {
							$ret = &$this->_Execute ($stmt, $arr);
							if (!$ret) {
								return $ret;
							}
						}
					} else {
						$ret = &$this->_Execute ($sql, $inputarr);
					}
				}
			} else {
				$ret = &$this->_Execute ($sql, false);
			}
			return $ret;

		}

		/**
		* Sets the Fetchmode
		*
		* @access public
		* @param bool	$mode
		* @return boolean
		*/

		function SetFetchMode ($mode) {

			$old = $this->fetchMode;
			$this->fetchMode = $mode;
			return $old;

		}

		/**
		* Set the debugmode.
		*
		* @access public
		* @param $debug
		* @return void
		*/

		function SetDebug ($debug) {

			$this->debug = $debug;

		}

		/**
		*
		* @access private
		* @param string	$table
		* @param string	$schema
		* @return void
		*/

		function _findschema (&$table, &$schema) {
			if (!$schema && ($at = strpos ($table, '.') ) !== false) {
				$schema = substr ($table, 0, $at);
				$table = substr ($table, $at+1);
			}

		}

		/**
		* List columns in a database as an array of SQLLayerFieldObjects.
		*
		* @access public
		* @param string	$table	table name to query
		* @param bool		$upper	uppercase table name (required by some databases)
		* @return mixed array or boolean
		*/

		function &MetaColumns ($table, $upper = true) {
			if (!empty ($this->metaColumnsSQL) ) {
				$schema = false;
				$this->_findschema ($table, $schema);
				if ($this->fetchMode !== false) {
					$savem = $this->SetFetchMode (false);
				}
				$rs = $this->Execute (sprintf ($this->metaColumnsSQL, ($upper)?strtoupper ($table) : $table) );
				if (isset ($savem) ) {
					$this->SetFetchMode ($savem);
				}
				if ($rs === false || $rs->EOF) {

					/* return byRef required - so it must be a variable*/

					$false = false;
					return $false;
				}
				$retarr = array ();
				while (! $rs->EOF) {
					$fld =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerFieldObject;
					$fld->name = $rs->fields[0];
					$fld->type = $rs->fields[1];
					if (isset ($rs->fields[3]) && $rs->fields[3]) {
						if ($rs->fields[3]>0) {
							$fld->max_length = $rs->fields[3];
						}
						$fld->scale = $rs->fields[4];
						if ($fld->scale>0) {
							$fld->max_length += 1;
						}
					} else {
						$fld->max_length = $rs->fields[2];
					}
					$retarr[strtolower ($fld->name)] = $fld;
					$rs->MoveNext ();
				}
				$rs->Close ();
				return $retarr;
			}

			/* return byRef required - so it must be a variable*/

			$false = false;
			return $false;

		}

		/**
		* Returns an array with the primary key columns in it.
		*
		* @access public
		* @param string	$table
		* @param mixed		$owner
		* @return mixed boolean or array
		*/

		function MetaPrimaryKeys ($table, $owner = false) {
			// owner not used in base class - see oci8
			$t = $owner;
			$p = array ();
			$objects = &$this->MetaColumns ($table);
			if ($objects) {
				foreach ($objects as $object) {
					if (!empty ($object->primary_key) ) {
						$p[] = $object->name;
					}
				}
			}
			if (sizeof ($p) ) {
				return $p;
			}
			return false;

		}

		/**
		* Returns assoc array where keys are tables, and values are foreign keys
		*
		* @access public
		* @param string	$table
		* @param mixed		$owner
		* @param bool		$upper
		* @return array
		*/

		function MetaForeignKeys ($table, $owner = false, $upper = false) {

			$t = $table;
			$t = $owner;
			$t = $upper;
			return array ();

		}

		/**
		* Will select, getting rows from $offset (1-based), for $nrows.
		* This simulates the MySQL "select * from table limit $offset,$nrows" , and
		* the PostgreSQL "select * from table limit $nrows offset $offset". Note that
		* MySQL and PostgreSQL parameter ordering is the opposite of the other.
		* eg.
		*  SelectLimit('select * from table',3); will return rows 1 to 3 (1-based)
		*  SelectLimit('select * from table',3,2); will return rows 3 to 5 (1-based)
		*
		* Uses SELECT TOP for Microsoft databases (when $this->hasTop is set)
		* BUG: Currently SelectLimit fails with $sql with LIMIT or TOP clause already set
		*
		* @access public
		* @param string	$sql
		* @param int		$nrows		is the row to start calculations from (1-based)
		* @param int		$offset		is the number of rows to get
		* @param mixed		$inputarr	array of bind variables
		* @return object
		*/

		function &SelecLimit ($sql, $nrows = -1, $offset = -1, $inputarr = false) {

			$ret = &$this->_SelectLimit ($sql, $nrows, $offset, $inputarr);
			return $ret;

		}

		/**
		* Will select, getting rows from $offset (1-based), for $nrows.
		* This simulates the MySQL "select * from table limit $offset,$nrows" , and
		* the PostgreSQL "select * from table limit $nrows offset $offset". Note that
		* MySQL and PostgreSQL parameter ordering is the opposite of the other.
		* eg.
		*  SelectLimit('select * from table',3); will return rows 1 to 3 (1-based)
		*  SelectLimit('select * from table',3,2); will return rows 3 to 5 (1-based)
		*
		* Uses SELECT TOP for Microsoft databases (when $this->hasTop is set)
		* BUG: Currently SelectLimit fails with $sql with LIMIT or TOP clause already set
		*
		* @access private
		* @param string	$sql
		* @param int		$nrows		is the row to start calculations from (1-based)
		* @param int		$offset		is the number of rows to get
		* @param mixed		$inputarr	array of bind variables
		* @return object
		*/

		function &_SelectLimit ($sql, $nrows = -1, $offset = -1, $inputarr = false) {
			if ($this->hasTop && $nrows>0) {
				// suggested by Reinhard Balling. Access requires top after distinct
				// Informix requires first before distinct - F Riosa
				$ismssql = (strpos ($this->databaseType, 'mssql') !== false);
				if ($ismssql) {
					$isaccess = false;
				} else {
					$isaccess = (strpos ($this->databaseType, 'access') !== false);
				}
				if ($offset<=0) {
					// access includes ties in result
					if ($isaccess) {
						$sql = preg_replace ('/(^\s*select\s+(distinctrow|distinct)?)/i', '\\1 ' . $this->hasTop . ' ' . $nrows . ' ', $sql);
						$ret = &$this->Execute ($sql, $inputarr);
						return $ret;
						// PHP5 fix
					}
					if ($ismssql) {
						$sql = preg_replace ('/(^\s*select\s+(distinctrow|distinct)?)/i', '\\1 ' . $this->hasTop . ' ' . $nrows . ' ', $sql);
					} else {
						$sql = preg_replace ('/(^\s*select\s)/i', '\\1 ' . $this->hasTop . ' ' . $nrows . ' ', $sql);
					}
				} else {
					$nn = $nrows+ $offset;
					if ($isaccess || $ismssql) {
						$sql = preg_replace ('/(^\s*select\s+(distinctrow|distinct)?)/i', '\\1 ' . $this->hasTop . ' ' . $nn . ' ', $sql);
					} else {
						$sql = preg_replace ('/(^\s*select\s)/i', '\\1 ' . $this->hasTop . ' ' . $nn . ' ', $sql);
					}
				}
			}
			$rs = &$this->Execute ($sql, $inputarr);
			if ($rs && !$rs->EOF) {
				$rs = &$this->_rs2rs ($rs, $nrows, $offset);
			}
			return $rs;

		}

		/**
		* Return first element of first row of sql statement. Recordset is disposed
		* for you.
		*
		* @access public
		* @param string	$sql		SQL statement
		* @param array		$inputarr	input bind array
		* @return mixed boolean or mixed
		*/

		function GetOne ($sql, $inputarr = false) {

			$ret = false;
			$rs = &$this->Execute ($sql, $inputarr);
			if ($rs) {
				if (!$rs->EOF) {
					$ret = reset ($rs->fields);
				}
				$rs->Close ();
			}
			return $ret;

		}

		/**
		*
		* @access public
		* @param string	$sql
		* @param mixed		$inputarr
		* @return array
		*/

		function &GetArray ($sql, $inputarr = false) {

			$rs = &$this->Execute ($sql, $inputarr);
			if (!$rs) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$arr = &$rs->GetArray ();
			$rs->Close ();
			return $arr;

		}

		/**
		* Returns the fetchmode.
		* @return boolean
		*/

		function GetFetchmode () {
			return $this->fetchMode;

		}

		/**
		* Synonym for GetArray
		*
		* @access public
		* @param string	$sql
		* @param mixed		$inputarr
		* @return unknown
		*/

		function &GetAll ($sql, $inputarr = false) {

			$arr = &$this->GetArray ($sql, $inputarr);
			return $arr;

		}

		/**
		* Update a blob column, given a where clause. There are more sophisticated
		* blob handling functions that we could have implemented, but all require
		* a very complex API. Instead we have chosen something that is extremely
		* simple to understand and use.
		*
		* Note: $blobtype supports 'BLOB' and 'CLOB', default is BLOB of course.
		*
		* Usage to update a $blobvalue which has a primary key blob_id=1 into a
		* field blobtable.blobcolumn:
		*
		* 		$conn->UpdateBlob('blobtable', 'blobcolumn', $blobvalue, 'blob_id=1');
		*
		* Insert example:
		*
		* 		$conn->Execute('INSERT INTO blobtable (id, blobcol) VALUES (1, null)');
		* 		$conn->UpdateBlob('blobtable','blobcol',$blob,'id=1');
		*
		* @access public
		* @param string	$table
		* @param string	$column
		* @param string	$val
		* @param string	$where
		* @param string	$blobtype
		* @return boolean
		*/

		function UpdateBlob ($table, $column, $val, $where, $blobtype = 'BLOB') {

			$t = $blobtype;
			return $this->Execute ('UPDATE ' . $table . ' SET ' . $column . '=? WHERE ' . $where, array ($val) ) != false;

		}

		/**
		* @access public
		* @param string	$blob
		* @return unknown
		*/

		function BlobDecode ($blob) {
			return $blob;

		}

		/**
		* @access public
		* @param string	$blob
		* @return unknown
		*/

		function BlobEncode ($blob) {
			return $blob;

		}

		/**
		* Sets the Charset for the database
		*
		* @access public
		* @param string	$charset
		* @return boolean
		*/

		function SetCharset ($charset) {

			$t = $charset;
			return false;

		}

		/**
		* @access public
		* @param string	$field
		* @param string	$ifNull
		* @return string
		*/

		function IfNull ($field, $ifNull) {
			return ' CASE WHEN ' . $field . ' is null THEN ' . $ifNull . ' ELSE ' . $field . ' END ';

		}

		/**
		* Close the connection
		*
		* @access public
		* @return boolean
		*/

		function Close () {

			$rez = $this->_close ();
			$this->_connectionID = false;
			return $rez;

		}

		/**
		*
		* @access public
		* @param string	$sql
		* @param mixed		$inputarr
		* @param bool		$trim
		* @return mixed boolean or array
		*/

		function GetCol ($sql, $inputarr = false, $trim = false) {

			$rv = false;
			$rs = &$this->Execute ($sql, $inputarr);
			if ($rs) {
				$rv = array ();
				if ($trim) {
					while (! $rs->EOF) {
						$rv[] = trim (reset ($rs->fields) );
						$rs->MoveNext ();
					}
				} else {
					while (! $rs->EOF) {
						$rv[] = reset ($rs->fields);
						$rs->MoveNext ();
					}
				}
				$rs->Close ();
			}
			return $rv;

		}

		/**
		* Return the databases that the driver can connect to.
		* Some databases will return an empty array.
		*
		* @access public
		* @return mixed array or boolean
		*/

		function MetaDatabases () {
			if ($this->metaDatabasesSQL) {
				if ($this->fetchMode !== false) {
					$savem = $this->SetFetchMode (false);
				}
				$arr = $this->GetCol ($this->metaDatabasesSQL);
				if (isset ($savem) ) {
					$this->SetFetchMode ($savem);
				}
				return $arr;
			}
			return false;

		}

		/**
		* Return's the tables in the database.
		*
		* @access public
		* @param mixed	$ttype		ttype can either be 'VIEW' or 'TABLE' or false.
		* 							If false, both views and tables are returned.
		* 							"VIEW" returns only views
		* 							"TABLE" returns only tables
		* @param bool	$showSchema	returns the schema/user with the table name, eg. USER.TABLE
		* @param bool	$mask		is the input mask - only supported by oci8 and postgresql
		* @return mixed array or boolean
		*/

		function &MetaTables ($ttype = false, $showSchema = false, $mask = false) {

			$ret = &$this->_MetaTables ($ttype, $showSchema, $mask);
			return $ret;

		}

		/**
		* Return's the tables in the database.
		*
		* @access private
		* @param mixed	$ttype		ttype can either be 'VIEW' or 'TABLE' or false.
		* 							If false, both views and tables are returned.
		* 							"VIEW" returns only views
		* 							"TABLE" returns only tables
		* @param bool	$showSchema	returns the schema/user with the table name, eg. USER.TABLE
		* @param bool	$mask		is the input mask - only supported by oci8 and postgresql
		* @return mixed array or boolean
		*/

		function &_MetaTables ($ttype = false, $showSchema = false, $mask = false) {

			$t = $showSchema;
			$t = $mask;
			if ($mask) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			if ($this->metaTablesSQL) {
				// complicated state saving by the need for backward compat
				if ($this->fetchMode !== false) {
					$savem = $this->SetFetchMode (false);
				}
				$rs = $this->Execute ($this->metaTablesSQL);
				if (isset ($savem) ) {
					$this->SetFetchMode ($savem);
				}
				if ($rs === false) {

					/* return byRef required - so it must be a variable*/

					$false = false;
					return $false;
				}
				$arr = &$rs->GetArray ();
				$arr2 = array ();
				if ($hast = ($ttype && isset ($arr[0][1]) ) ) {
					$showt = strncmp ($ttype, 'T', 1);
				}
				$max = sizeof ($arr);
				for ($i = 0; $i< $max; $i++) {
					if ($hast) {
						if ($showt == 0) {
							if (strncmp ($arr[$i][1], 'T', 1) == 0) {
								$arr2[] = trim ($arr[$i][0]);
							}
						} else {
							if (strncmp ($arr[$i][1], 'V', 1) == 0) {
								$arr2[] = trim ($arr[$i][0]);
							}
						}
					} else {
						$temp = strtolower (trim ($arr[$i][0]));
						$arr2[$temp] = $temp;
					}
				}
				$rs->Close ();
				unset ($arr);
				return $arr2;
			}

			/* return byRef required - so it must be a variable*/

			$false = false;
			return $false;

		}

		/**
		* List indexes on a table as an array.
		*
		* @access public
		* @param string	$table		table name to query
		* @param bool		$primary	true to only show primary keys. Not actually used for most databases
		* @param mixed		$owner
		* @return array
		*/

		function &MetaIndexes ($table, $primary = false, $owner = false) {

			$t = $table;
			$t = $primary;
			$t = $owner;
			$false = array ();
			return $false;

		}

		/**
		* List columns names in a table as an array.
		*
		* @access public
		* @param string	$table	table name to query
		* @param bool		$numIndexes
		* @return mixed boolean or array
		*/

		function &MetaColumnNames ($table, $numIndexes = false) {

			$objarr = &$this->MetaColumns ($table);
			if (!is_array ($objarr) ) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$arr = array ();
			if ($numIndexes) {
				$i = 0;
				foreach ($objarr as $v) {
					$arr[$i++] = $v->name;
				}
			} else {
				foreach ($objarr as $v) {
					$arr[strtolower ($v->name)] = $v->name;
				}
			}
			unset ($objarr);
			return $arr;

		}

		/**
		* Choose a database to connect to. Many databases do not support this.
		*
		* @access public
		* @param string	$dbName	is the name of the database to select
		* @return boolean
		*/

		function SelectDB ($dbName) {

			$t = $dbName;
			return false;

		}

	}
	// class SQLConnection

	/**
	* Internal placeholder for record objects. Used by SQLLayerRecordSet->FetchObj().
	*/

	class SQLLayerFetchObj {

	}

	/**
	* Helper class for FetchFields -- holds info on a column
	*/

	class SQLLayerFieldObject {

		/*<{php4}>
		var $name = '';
		var $max_length = 0;
		var $type = '';
		var $scale = 0;
		var $sub_type = '';
		var $primary_key = false;
		var $auto_increment = false;
		var $binary = false;
		var $unsigned = false;
		var $precision = 0;
		var $unique = false;
		var $not_null = false;
		var $has_default = false; // this one I have done only in mysql and postgres for now ...
		var $default_value; // default, if any, and supported. Check has_default first.
		<{php4/}>*/

		/*<{php5}>
		public $name = '';
		public $max_length = 0;
		public $type = '';
		public $scale = 0;
		public $sub_type = '';
		public $primary_key = false;
		public $auto_increment = false;
		public $binary = false;
		public $unsigned = false;
		public $precision = 0;
		public $unique = false;
		public $not_null = false;
		public $has_default = false; // this one I have done only in mysql and postgres for now ...
		public $default_value; // default, if any, and supported. Check has_default first.
		<{php5/}>*/

		/*<{delete}>*/
		var $name = '';
		var $max_length = 0;
		var $type = '';
		var $scale = 0;
		var $sub_type = '';
		var $primary_key = false;
		var $auto_increment = false;
		var $binary = false;
		var $unsigned = false;
		var $precision = 0;
		var $unique = false;
		var $not_null = false;
		var $has_default = false; // this one I have done only in mysql and postgres for now ...
		var $default_value; // default, if any, and supported. Check has_default first.
		/*<{delete/}>*/

	}
	// class SQLLayerFieldObject

	/**
	* Lightweight recordset when there are no records to be returned
	*/

	class SQLLayerRecordSet_empty {

		/*<{php4}>
		var $dataProvider = 'empty';
		var $databaseType = false;
		var $EOF = true;
		var $_numOfRows = 0;
		var $fields = false;
		var $connection = false;
		<{php4/}>*/

		/*<{php5}>
		public $dataProvider = 'empty';
		public $databaseType = false;
		public $EOF = true;
		public $_numOfRows = 0;
		public $fields = false;
		public $connection = false;
		<{php5/}>*/

		/*<{delete}>*/
		var $dataProvider = 'empty';
		var $databaseType = false;
		var $EOF = true;
		var $_numOfRows = 0;
		var $fields = false;
		var $connection = false;
		/*<{delete/}>*/

		function RecordCount () {
			return 0;

		}

		function Close () {
			return true;

		}

		function FieldCount () {
			return 0;

		}

		function Init () {

		}

	}
	// class SQLLayerRecordSet_empty

	/**
	* RecordSet class that represents the dataset returned by the database.
	* To keep memory overhead low, this class holds only the current row in memory.
	* No prefetching of data is done, so the RecordCount() can return -1 ( which
	* means recordcount not known).
	*/

	class SQLLayerRecordSet {

		/*
		* public variables
		*/

		/*<{php4}>
		var $dataProvider = 'native';
		var $fields = false; 	/// holds the current row data
		var $blobSize = 100; 	/// any varchar/char field this size or greater is treated as a blob
		/// in other words, we use a text area for editing.
		var $canSeek = false; 	/// indicates that seek is supported
		var $EOF = false;		/// Indicates that the current record position is after the last record in a Recordset object.

		var $debug = false;

		var $bind = false; 		/// used by Fields() to hold array - should be private?
		var $fetchMode;			/// default fetch mode
		var $connection = false; /// the parent connection
		var $databaseType = '';
		var $sql = '';

		//private variables

		var $_numOfRows = -1;	// number of rows, or -1
		var $_numOfFields = -1;	// number of fields in recordset
		var $_queryID = -1;		// This variable keeps the result link identifier.
		var $_currentRow = -1;	// This variable keeps the current row in the Recordset.
		var $_closed = false; 	// has recordset been closed
		var $_inited = false; 	// Init() should only be called once

		var $_maxRecordCount = 0;
		var $datetime = false;
		<{php4/}>*/

		/*<{php5}>
		public $dataProvider = 'native';
		public $fields = false; 	/// holds the current row data
		public $blobSize = 100; 	/// any varchar/char field this size or greater is treated as a blob
		/// in other words, we use a text area for editing.
		public $canSeek = false; 	/// indicates that seek is supported
		public $EOF = false;		/// Indicates that the current record position is after the last record in a Recordset object.

		public $debug = false;

		public $bind = false; 		/// used by Fields() to hold array - should be private?
		public $fetchMode;			/// default fetch mode
		public $connection = false; /// the parent connection
		public $databaseType = '';
		public $sql = '';

		//private variables

		public $_numOfRows = -1;	// number of rows, or -1
		public $_numOfFields = -1;	// number of fields in recordset
		public $_queryID = -1;		// This variable keeps the result link identifier.
		public $_currentRow = -1;	// This variable keeps the current row in the Recordset.
		public $_closed = false; 	// has recordset been closed
		public $_inited = false; 	// Init() should only be called once

		public $_maxRecordCount = 0;
		public $datetime = false;
		<{php5/}>*/

		/*<{delete}>*/
		var $dataProvider = 'native';
		var $fields = false; 	/// holds the current row data
		var $blobSize = 100; 	/// any varchar/char field this size or greater is treated as a blob
		/// in other words, we use a text area for editing.
		var $canSeek = false; 	/// indicates that seek is supported
		var $EOF = false;		/// Indicates that the current record position is after the last record in a Recordset object.

		var $debug = false;

		var $bind = false; 		/// used by Fields() to hold array - should be private?
		var $fetchMode;			/// default fetch mode
		var $connection = false; /// the parent connection
		var $databaseType = '';
		var $sql = '';

		//private variables

		var $_numOfRows = -1;	// number of rows, or -1
		var $_numOfFields = -1;	// number of fields in recordset
		var $_queryID = -1;		// This variable keeps the result link identifier.
		var $_currentRow = -1;	// This variable keeps the current row in the Recordset.
		var $_closed = false; 	// has recordset been closed
		var $_inited = false; 	// Init() should only be called once

		var $_maxRecordCount = 0;
		var $datetime = false;
		/*<{delete/}>*/

		/**
		* Constructor
		*
		* @access public
		* @param ressource	$queryID	his is the queryID returned by SQLLayerConnection->_query()
		* @return void
		*/

		function SQLLayerRecordSet ($queryID) {

			$this->_queryID = $queryID;

		}

		/**
		* Must be overwritten in the Recordsetclass.
		*
		* @access private
		* @return void
		*/

		function _initrs () {

		}

		/**
		* Must be overwritten in the Recordsetclass.
		*
		* @access private
		* @return boolean
		*/

		function _fetch () {
			return true;

		}

		/**
		* Must be overwritten in the Recordsetclass.
		*
		* @access private
		* @param int	$number
		* @return boolean
		*/

		function _seek ($number) {

			$t = $number;
			return false;

		}

		/**
		* Must be overwritten in the Recordsetclass.
		*
		* @access private
		* @return boolean
		*/

		function _close () {
			return true;

		}

		/**
		* Get the SQLLayerFieldObject of a specific column.
		*
		* @access public
		* @param int	$fieldoffset
		* @return object
		*/

		function &FetchField ($fieldoffset = -1) {

			$t = $fieldoffset;
			$object =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerFieldObject;
			return $object;

		}

		/**
		* Get the SQLLayerFieldObjects of all columns in an array.
		*
		* @access public
		* @return array
		*/

		function &FieldTypesArray () {

			$arr = array ();
			for ($i = 0, $max = $this->_numOfFields; $i< $max; $i++) {
				$arr[] = $this->FetchField ($i);
			}
			return $arr;

		}

		/**
		* Converts the fields indicies to lowercases when the fields array is an assoc array.
		*
		* @access private
		* @return void
		*/

		function _convert_to_lowercase () {

			$arr1 = array ();
			foreach ($this->fields as $key => $value) {
				$arr1[strtolower ($key)] = $value;
			}
			$this->fields = $arr1;
			unset ($arr1);

		}

		/**
		* Init the Recordset
		*
		* @access public
		* @return void
		*/

		function Init () {
			if (!$this->_inited) {
				$this->_inited = true;
				if ($this->_queryID) {
					@ $this->_initrs ();
				} else {
					$this->_numOfRows = 0;
					$this->_numOfFields = 0;
				}
				if ( ($this->_numOfRows != 0) && $this->_numOfFields && $this->_currentRow == -1) {
					$this->_currentRow = 0;
					if ($this->EOF = ($this->_fetch () === false) ) {
						$this->_numOfRows = 0;
						// _numOfRows could be -1
					}
				} else {
					$this->EOF = true;
				}
			}

		}

		/**
		* return recordset as a 2-dimensional array.
		*
		* @access public the number of rows to return. -1 means every row.
		* @param int	$nRows
		* @return array	- indexed by the rows (0-based) from the recordset
		*/

		function &GetArray ($nRows = -1) {

			$results = array ();
			$cnt = 0;
			while (! $this->EOF && $nRows != $cnt) {
				$results[] = $this->fields;
				$this->MoveNext ();
				$cnt++;
			}
			$o = &$results;
			return $o;

		}

		/**
		* Wrapper for GetArray.
		*
		* @access public
		* @param int	$nRows
		* @return array
		*/

		function &GetAll ($nRows = -1) {

			$arr = &$this->GetArray ($nRows);
			return $arr;

		}

		/**
		* return recordset as a 2-dimensional array.
		* Helper function for SQLLayerConnection->SelectLimit()
		* @param int	$nrows
		* @param int	$offset
		* @return mixed unknown or array
		*/

		function &GetArrayLimit ($nrows, $offset = -1) {
			if ($offset<=0) {
				$arr = &$this->GetArray ($nrows);
				return $arr;
			}
			$this->Move ($offset);
			$results = array ();
			$cnt = 0;
			while (! $this->EOF && $nrows != $cnt) {
				$results[$cnt++] = $this->fields;
				$this->MoveNext ();
			}
			return $results;

		}

		/**
		* Synonym for GetArray() for compatibility with ADO.
		*
		* @access public
		* @param int	$nRows	the number of rows to return. -1 means every row.
		* @return array
		*/

		function &GetRows ($nRows = -1) {

			$arr = &$this->GetArray ($nRows);
			return $arr;

		}

		/**
		* Return whole recordset as a 2-dimensional associative array if there are more than 2 columns.
		* The first column is treated as the key and is not included in the array.
		* If there is only 2 columns, it will return a 1 dimensional array of key-value pairs unless
		* $force_array == true.
		*
		* @access public
		* @param bool	$force_array
		* @param bool	$first2cols
		* @return mixed unknown or array
		*/

		function &GetAssoc ($force_array = false, $first2cols = false) {

			$cols = $this->_numOfFields;
			if ($cols<2) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$numIndex = isset ($this->fields[0]);
			$results = array ();
			if (!$first2cols && ($cols>2 || $force_array) ) {
				if ($numIndex) {
					while (! $this->EOF) {
						$results[trim ($this->fields[0])] = array_slice ($this->fields, 1);
						$this->MoveNext ();
					}
				} else {
					while (! $this->EOF) {
						$results[trim (reset ($this->fields) )] = array_slice ($this->fields, 1);
						$this->MoveNext ();
					}
				}
			} else {
				if ($numIndex) {
					while (! $this->EOF) {
						$results[trim ( ($this->fields[0]) )] = $this->fields[1];
						$this->MoveNext ();
					}
				} else {
					while (! $this->EOF) {
						$results[trim (reset ($this->fields) )] = next ($this->fields);
						$this->MoveNext ();
					}
				}
			}
			return $results;

		}

		/**
		* Fetch a row, returning false if no more rows.
		*
		* @access public
		* @return unknown
		*/

		function &FetchRow () {
			if ($this->EOF) {
				$false = false;
				return $false;
			}
			$arr = $this->fields;
			$this->_currentRow++;
			if (!$this->_fetch () ) {
				$this->EOF = true;
			}
			return $arr;

		}

		/**
		* Move to the first row in the recordset. Many databases do NOT support this.
		*
		* @access public
		* @return boolean
		*/

		function MoveFirst () {
			if ($this->_currentRow == 0) {
				return true;
			}
			return $this->Move (0);

		}

		/**
		* Move to the last row in the recordset.
		*
		* @access public
		* @return boolean
		*/

		function MoveLast () {
			if ($this->_numOfRows >= 0) {
				return $this->Move ($this->_numOfRows-1);
			}
			if ($this->EOF) {
				return false;
			}
			while (! $this->EOF) {
				$f = $this->fields;
				$this->MoveNext ();
			}
			$this->fields = $f;
			$this->EOF = false;
			return true;

		}

		/**
		* Random access to a specific row in the recordset. Some databases do not support
		* access to previous rows in the databases (no scrolling backwards).
		*
		* @access public
		* @param int	$rowNumber	the row to move to (0-based)
		* @return boolean
		*/

		function Move ($rowNumber = 0) {

			$this->EOF = false;
			if ($rowNumber == $this->_currentRow) {
				return true;
			}
			if ($rowNumber >= $this->_numOfRows) {
				if ($this->_numOfRows != -1) {
					$rowNumber = $this->_numOfRows-2;
				}
			}
			if ($this->canSeek) {
				if ($this->_seek ($rowNumber) ) {
					$this->_currentRow = $rowNumber;
					if ($this->_fetch () ) {
						return true;
					}
				} else {
					$this->EOF = true;
					return false;
				}
			} else {
				if ($rowNumber<$this->_currentRow) {
					return false;
				}
				while (! $this->EOF && $this->_currentRow< $rowNumber) {
					$this->_currentRow++;
					if (!$this->_fetch () ) {
						$this->EOF = true;
					}
				}
				return ! ($this->EOF);
			}
			$this->fields = false;
			$this->EOF = true;
			return false;

		}

		/**
		* Move to next record in the recordset
		*
		* @access public
		* @param bool	$autoclose
		* @return boolean
		*/

		function MoveNext ($autoclose = true) {
			if (!$this->EOF) {
				$this->_currentRow++;
				if ($this->_fetch () ) {
					return true;
				}
			}
			if ($autoclose) {
				$this->Close ();
			}
			$this->fields = false;
			$this->EOF = true;
			return false;

		}

		/**
		* Get the value of a field in the current row by column name.
		*
		* @access public
		* @param string	$colname	is the field to access
		* @return unknown
		*/

		function Fields ($colname) {
			return $this->fields[$colname];

		}

		/**
		*
		* @access public
		* @param bool	$upper
		* @return void
		*/

		function GetAssocKeys ($upper = true) {

			$t = $upper;
			$this->bind = array ();
			for ($i = 0, $max = $this->_numOfFields; $i< $max; $i++) {
				$object = &$this->FetchField ($i);
				if ($this->fetchMode) {
					$this->bind[strtolower ($object->name)] = strtolower ($object->name);
				} else {
					$this->bind[strtolower ($object->name)] = $i;
				}
			}

		}

		/**
		* Use associative array to get fields array for databases that do not support
		* associative arrays.
		*
		* @access public
		* @param int	$upper 	0 = lowercase, 1 = uppercase, 2 = whatever is returned by FetchField
		* @return array
		*/

		function &GetRowAssoc ($upper = 0) {

			$ret = &$this->_GetRowAssoc ($upper);
			return $ret;

		}

		/**
		* Use associative array to get fields array for databases that do not support
		* associative arrays.
		*
		* @access private
		* @param int	$upper 	0 = lowercase, 1 = uppercase, 2 = whatever is returned by FetchField
		* @return array
		*/

		function &_GetRowAssoc ($upper = 0) {

			$record = array ();
			if (!$this->bind) {
				$this->GetAssocKeys ($upper);
			}
			foreach ($this->bind as $k => $v) {
				$record[$k] = $this->fields[$v];
			}
			return $record;

		}

		/**
		* Close the Recordset
		*
		* @access public
		* @return boolean
		*/

		function Close () {
			// free connection object - this seems to globally free the object
			// and not merely the reference, so don't do this...
			// $this->connection = false;
			if (!$this->_closed) {
				$this->_closed = true;
				return $this->_close ();
			}
			return true;

		}

		/**
		* The number of rows or -1 if this is not supported
		*
		* @access public
		* @return integer
		*/

		function RecordCount () {
			return $this->_numOfRows;

		}

		/**
		* The number of columns in the recordset. Some databases will set this to 0
		* if no records are returned, others will return the number of columns in the query.
		*
		* @access public
		* @return integer
		*/

		function FieldCount () {
			return $this->_numOfFields;

		}

		/**
		* Get the metatype of the column. This is used for formatting. This is because
		* many databases use different names for the same type, so we transform the original
		* type to our standardised version which uses 1 character codes:
		*
		* @access public
		* @param mixed	$type			is the type passed in. Normally is SQLLayerFieldObject->type.
		* @param int	$len		is the maximum length of that field. This is because we treat character
		* 							fields bigger than a certain size as a 'B' (blob).
		* @param mixed	$fieldobj	is the field object returned by the database driver. Can hold
		* 							additional info (eg. primary_key for mysql).
		* @return mixed unknown or string
		*	C for character < 250 chars
		*	X for teXt (>= 250 chars)
		*	B for Binary
		* 	N for numeric or floating point
		*	D for date
		*	T for timestamp
		* 	L for logical/Boolean
		*	I for integer
		*	R for autoincrement counter/integer
		*/

		function MetaType ($type, $len = -1, $fieldobj = object) {
			if (is_object ($type) ) {
				$fieldobj = $type;
				$type = $fieldobj->type;
				$len = $fieldobj->max_length;
			}
			static $typeMap = array ('VARCHAR' => 'C',
						'VARCHAR2' => 'C',
						'CHAR' => 'C',
						'C' => 'C',
						'STRING' => 'C',
						'NCHAR' => 'C',
						'NVARCHAR' => 'C',
						'VARYING' => 'C',
						'BPCHAR' => 'C',
						'CHARACTER' => 'C',
						'INTERVAL' => 'C',
			// Postgres
			//
			'LONGCHAR' => 'X',
						'TEXT' => 'X',
						'NTEXT' => 'X',
						'M' => 'X',
						'X' => 'X',
						'CLOB' => 'X',
						'NCLOB' => 'X',
						'LVARCHAR' => 'X',
			//
			'BLOB' => 'B',
						'IMAGE' => 'B',
						'BINARY' => 'B',
						'VARBINARY' => 'B',
						'LONGBINARY' => 'B',
						'B' => 'B',
			//
			'YEAR' => 'D',
			// mysql
			'DATE' => 'D',
						'D' => 'D',
			//
			'TIME' => 'T',
						'TIMESTAMP' => 'T',
						'DATETIME' => 'T',
						'TIMESTAMPTZ' => 'T',
						'T' => 'T',
			//
			'BOOL' => 'L',
						'BOOLEAN' => 'L',
						'BIT' => 'L',
						'L' => 'L',
			//
			'COUNTER' => 'R',
						'R' => 'R',
						'SERIAL' => 'R',
			// ifx
			'INT IDENTITY' => 'R',
			//
			'INT' => 'I',
						'INT2' => 'I',
						'INT4' => 'I',
						'INT8' => 'I',
						'INTEGER' => 'I',
						'INTEGER UNSIGNED' => 'I',
						'SHORT' => 'I',
						'TINYINT' => 'I',
						'SMALLINT' => 'I',
						'I' => 'I',
			//
			'LONG' => 'N',
			// interbase is numeric, oci8 is blob
			'BIGINT' => 'N',
			// this is bigger than PHP 32-bit integers
			'DECIMAL' => 'N',
						'DEC' => 'N',
						'REAL' => 'N',
						'DOUBLE' => 'N',
						'DOUBLE PRECISION' => 'N',
						'SMALLFLOAT' => 'N',
						'FLOAT' => 'N',
						'NUMBER' => 'N',
						'NUM' => 'N',
						'NUMERIC' => 'N',
						'MONEY' => 'N',
			// informix 9.2
			'SQLINT' => 'I',
						'SQLSERIAL' => 'I',
						'SQLSMINT' => 'I',
						'SQLSMFLOAT' => 'N',
						'SQLFLOAT' => 'N',
						'SQLMONEY' => 'N',
						'SQLDECIMAL' => 'N',
						'SQLDATE' => 'D',
						'SQLVCHAR' => 'C',
						'SQLCHAR' => 'C',
						'SQLDTIME' => 'T',
						'SQLINTERVAL' => 'N',
						'SQLBYTES' => 'B',
						'SQLTEXT' => 'X');
			$type = strtoupper ($type);
			$tmap = (isset ($typeMap[$type]) )? $typeMap[$type] : 'N';
			switch ($tmap) {
				case 'C':
					// is the char field is too long, return as text field...
					if ($this->blobSize >= 0) {
						if ($len>$this->blobSize) {
							return 'X';
						}
					} elseif ($len>250) {
						return 'X';
				}
				return 'C';
				case 'I':
					if (!empty ($fieldobj->primary_key) ) {
						return 'R';
					}
					return 'I';
				case false:
					return 'N';
				case 'B':
					if (isset ($fieldobj->binary) ) {
						return ($fieldobj->binary)?'B' : 'X';
					}
					return 'B';
				case 'D':
					if (!empty ($this->datetime) ) {
						return 'T';
					}
					return 'D';
				default:
					if ($type == 'LONG' && $this->dataProvider == 'oci8') {
						return 'B';
					}
					return $tmap;
			}

		}

	}
	// class SQLLAYER_Recordset

	class SQLLayerRecordSet_array extends SQLLayerRecordSet {

		/*<{php4}>
		var $databaseType = 'array';
		var $_array;
		var $_types;
		var $_colnames;
		var $_skiprow1;
		var $_fieldarr;
		var $canSeek = true;
		var $affectedrows = false;
		var $compat = false;
		<{php4/}>*/

		/*<{php5}>
		public $databaseType = 'array';
		public $_array;
		public $_types;
		public $_colnames;
		public $_skiprow1;
		public $_fieldarr;
		public $canSeek = true;
		public $affectedrows = false;
		public $compat = false;
		<{php5/}>*/

		/*<{delete}>*/
		var $databaseType = 'array';
		var $_array;
		var $_types;
		var $_colnames;
		var $_skiprow1;
		var $_fieldarr;
		var $canSeek = true;
		var $affectedrows = false;
		var $compat = false;
		/*<{delete/}>*/

		/**
		* Constructor
		*
		* @access public
		* @param int	$fakeid
		* @return void
		*/

		function SQLLayerRecordSet_array ($fakeid = 1) {

			$this->SQLLayerRecordSet ($fakeid);
			$this->fetchMode = true;

		}

		/**
		* Setup the array
		*
		* @access public
		* @param array	$array
		* @param array	$typearr
		* @param bool	$colnames
		* @return void
		*/

		function InitArray ($array, $typearr, $colnames = false) {

			$this->_array = $array;
			$this->_types = $typearr;
			if ($colnames) {
				$this->_skiprow1 = false;
				$this->_colnames = $colnames;
			} else {
				$this->_skiprow1 = true;
				$this->_colnames = $array[0];
			}
			$this->Init ();

		}

		/**
		* Setup the array and datatype field objects
		*
		* @access public
		* @param array	$array
		* @param array	$fieldarr
		* @return void
		*/

		function InitArrayFields (&$array, &$fieldarr) {

			$this->_array = &$array;
			$this->_skiprow1 = false;
			if ($fieldarr) {
				$this->_fieldobjects = &$fieldarr;
			}
			$this->Init ();

		}

		/**
		*
		* @access public
		* @param int	$nRows
		* @return array
		*/

		function &GetArray ($nRows = 1) {
			if ( ($nRows == 1) && ($this->_currentRow<=0) && (!$this->_skiprow1) ) {
				return $this->_array;
			}
			$arr = &parent::GetArray ($nRows);
			return $arr;

		}

		/**
		*
		* @access private
		* @return void
		*/

		function _initrs () {

			$this->_numOfRows = count ($this->_array);
			if ($this->_skiprow1) {
				$this->_numOfRows--;
			}
			$this->_numOfFields = (isset ($this->_fieldobjects) )?count ($this->_fieldobjects) : count ($this->_types);

		}

		/**
		* Get the value of a field in the current row by column name.
		*
		* @access public
		* @param string	$colname
		* @return array
		*/

		function Fields ($colname) {
			if ($this->connection->fetchMode) {
				if (!isset ($this->fields[$colname]) ) {
					$colname = strtolower ($colname);
				}
				return $this->fields[$colname];
			}
			if (!$this->bind) {
				$this->bind = array ();
				for ($i = 0, $max = $this->_numOfFields; $i< $max; $i++) {
					$object = $this->FetchField ($i);
					$this->bind[strtolower ($object->name)] = $i;
				}
			}
			return $this->fields[$this->bind[strtolower ($colname)]];

		}

		/**
		* Get the SQLLayerFieldObject of a specific column.
		*
		* @access public
		* @param int	$fieldOffset
		* @return object
		*/

		function &FetchField ($fieldOffset = -1) {
			if (isset ($this->_fieldobjects) ) {
				return $this->_fieldobjects[$fieldOffset];
			}
			$o =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerFieldObject;
			$o->name = $this->_colnames[$fieldOffset];
			$o->type = $this->_types[$fieldOffset];
			$o->max_length = -1;
			// length not known
			return $o;

		}

		/**
		*
		* @access private
		* @param int	$row
		* @return boolean
		*/

		function _seek ($row) {
			if (sizeof ($this->_array) && 0<=$row && $row<$this->_numOfRows) {
				$this->_currentRow = $row;
				if ($this->_skiprow1) {
					$row += 1;
				}
				$this->fields = $this->_array[$row];
				return true;
			}
			return false;

		}

		/**
		*
		* @access private
		* @return boolean
		*/

		function _fetch () {

			$pos = $this->_currentRow;
			if ($this->_numOfRows<=$pos) {
				if (!$this->compat) {
					$this->fields = false;
				}
				return false;
			}
			if ($this->_skiprow1) {
				$pos += 1;
			}
			$this->fields = $this->_array[$pos];
			return true;

		}

		/**
		*
		* @access private
		* @return boolean
		*/

		function _close () {
			return true;

		}

	}
	// class SQLLayerRecordSet_array
	define ('_OPNSQL_TINYINT', 0);
	define ('_OPNSQL_SMALLINT', 1);
	define ('_OPNSQL_MEDIUMINT', 2);
	define ('_OPNSQL_INT', 3);
	define ('_OPNSQL_BIGINT', 4);
	define ('_OPNSQL_FLOAT', 5);
	define ('_OPNSQL_DOUBLE', 6);
	define ('_OPNSQL_NUMERIC', 7);
	define ('_OPNSQL_DATE', 8);
	define ('_OPNSQL_DATETIME', 9);
	define ('_OPNSQL_TIMESTAMP', 10);
	define ('_OPNSQL_TIME', 11);
	define ('_OPNSQL_CHAR', 12);
	define ('_OPNSQL_VARCHAR', 13);
	define ('_OPNSQL_TINYBLOB', 14);
	define ('_OPNSQL_TINYTEXT', 15);
	define ('_OPNSQL_BLOB', 16);
	define ('_OPNSQL_TEXT', 17);
	define ('_OPNSQL_MEDIUMBLOB', 18);
	define ('_OPNSQL_MEDIUMTEXT', 19);
	define ('_OPNSQL_BIGBLOB', 20);
	define ('_OPNSQL_BIGTEXT', 21);
	define ('_OPNSQL_TABLOCK_ACCESS_SHARE', 0);
	define ('_OPNSQL_TABLOCK_ROW_SHARE', 1);
	define ('_OPNSQL_TABLOCK_ROW_EXCLUSIVE', 2);
	define ('_OPNSQL_TABLOCK_SHUPD_EXCLUSIVE', 3);
	define ('_OPNSQL_TABLOCK_SHARE', 4);
	define ('_OPNSQL_TABLOCK_SHROW_EXCLUSIVE', 5);
	define ('_OPNSQL_TABLOCK_EXCLUSIVE', 6);
	define ('_OPNSQL_TABLOCK_ACCESS_EXCLUSIVE', 7);
	define ('_OPNSQL_NOFIELDERROR', 0);
	define ('_OPNSQL_FIELDERRORNAME', 1);
	define ('_OPNSQL_FIELDERRORLENGTH', 2);
	define ('_OPNSQL_FIELDERRORTYPE', 3);
	define ('_OPNSQL_FIELDERRORNOTNULL', 4);
	define ('_OPNSQL_FIELDERRORHASDEFAULT', 5);
	define ('_OPNSQL_FIELDERRORSCALE', 6);

	class OPN_SQL {

		/*<{php4}>
		var $CanOptimize = false;
		var $CanRepair = false;
		var $HasStatus = false;
		var $DropTable = 'DROP TABLE %s';
		var $IndexCreate = 'create %s index idx%sidx%s on %s %s';
		var $ShowTableStatus = '';
		var $CanAlterColumnAdd= false;
		VAR $CanAlterColumnChange=false;

		var $CanAddRepairField = false;
		var $CanChangeRepairField = false;
		var $CanExecuteColumnAdd =  false;
		var $CanExecuteColumnChange =  false;
		var $_blobcache = false;
		<{php4/}>*/

		/*<{php5}>
		public $CanOptimize = false;
		public $CanRepair = false;
		public $HasStatus = false;
		public $DropTable = 'DROP TABLE %s';
		public $IndexCreate = 'create %s index idx%sidx%s on %s %s';
		public $ShowTableStatus = '';
		public $CanAlterColumnAdd= false;
		public $CanAlterColumnChange=false;

		public $CanAddRepairField = false;
		public $CanChangeRepairField = false;
		public $CanExecuteColumnAdd =  false;
		public $CanExecuteColumnChange =  false;
		public $_blobcache = false;
		<{php5/}>*/

		/*<{delete}>*/
		var $CanOptimize = false;
		var $CanRepair = false;
		var $HasStatus = false;
		var $DropTable = 'DROP TABLE %s';
		var $IndexCreate = 'create %s index idx%sidx%s on %s %s';
		var $ShowTableStatus = '';
		var $CanAlterColumnAdd= false;
		VAR $CanAlterColumnChange=false;

		var $CanAddRepairField = false;
		var $CanChangeRepairField = false;
		var $CanExecuteColumnAdd =  false;
		var $CanExecuteColumnChange =  false;
		var $_blobcache = false;
		/*<{delete/}>*/

		/**
		* Loads and returns the needed OPN SQL-Driver class.
		*
		* @return object The Driverclass
		*/

		function loadDriver ($driver) {
			switch ($driver) {
				case 'mysql':
				case 'mysqlt':
					$class =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opn_mysql ();
					break;
				case 'mysqli':
					$class =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opn_mysqli ();
					break;
				case 'borland_ibase':
				case 'firebird':
				case 'ibase':
					$class =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opn_ibase ();
					break;
				case 'mssql':
					$class =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opn_mssql ();
					break;

				/* case 'oci8':
				case 'oci805':
				case 'oci8po':
				case 'oracle':
				$class = new opn_oracle();
				break; */
				case 'postgres':
				case 'postgres7':
				case 'postgres64':
					$class =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opn_postgres ();
					break;
				case 'sqlite':
					$class =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opn_sqlite ();
					break;
				default:
					opnErrorHandler (E_WARNING, 'Unable to find class for driver ' . $driver, __FILE__, __LINE__);
					$class =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ OPN_SQL;
					break;
			}
			// end switch
			return $class;

		}

		/**
		* OPN_SQL::TableOptimize()
		*
		* @param  $tablename
		* @return string
		*/

		function TableOptimize ($tablename) {

			$t = $tablename;
			return '';

		}

		/**
		* OPN_SQL::TableComment()
		*
		* @param  $tablename
		* @param  $comment
		* @return
		*/

		function TableComment ($tablename, $comment) {

			$t = $tablename;
			$t = $comment;
			return '';

		}

		/**
		* OPN_SQL::TableRepair()
		*
		* @param  $tablename
		* @return
		*/

		function TableRepair ($tablename) {

			$t = $tablename;
			return '';

		}

		/**
		* OPN_SQL::TableDrop()
		*
		* @param  $tablename
		* @return
		*/

		function TableDrop ($tablename) {

			$t = $tablename;
			return '';

		}

		/**
		* OPN_SQL::TableLock()
		*
		* @param  $tablename
		* @param unknown $lockmode
		* @return
		*/

		function TableLock ($tablename, $lockmode = _OPNSQL_TABLOCK_ACCESS_EXCLUSIVE) {

			$t = $tablename;
			$t = $lockmode;
			return '';

		}

		/**
		* OPN_SQL::TableUnLock()
		*
		* @param  $tablename
		* @return
		*/

		function TableUnLock ($tablename) {

			$t = $tablename;
			return '';

		}

		/**
		* OPN_SQL::IndexDrop()
		*
		* @param  $index
		* @param string $table
		* @return
		*/

		function IndexDrop ($index, $table = '') {

			$t = $table;
			$t = $index;
			return '';

		}

		/**
		* OPN_SQL::PrimaryIndexDrop()
		*
		* @param string $table
		* @return
		*/

		function PrimaryIndexDrop ($table) {

			$t = $table;
			return '';

		}

		/**
		* OPN_SQL::CreateIndex()
		*
		* @param  $unique
		* @param  $idxname
		* @param  $idxcounter
		* @param  $table
		* @param  $fields
		* @return
		*/

		function CreateIndex ($unique, $idxname, $idxcounter, $table, $fields) {

			$t = $unique;
			$t = $idxname;
			$t = $idxcounter;
			$t = $table;
			$t = $fields;
			return '';

		}

		/**
		* OPN_SQL::CreateBetween()
		*
		* @param  $field
		* @param  $from
		* @param  $to
		* @param boolean $char
		* @param integer $len
		* @return
		*/

		function CreateBetween ($field, $from, $to, $char = false, $len = 0) {

			$t = $len;
			if (!$char) {
				return '(' . $field . ' BETWEEN ' . $from . ' AND ' . $to . ')';
			}
			return '(' . $field . " BETWEEN '" . $from . "' AND '" . $to . "')";

		}

		/**
		* OPN_SQL::ExecuteColumnAdd()
		*
		* @param  $table
		* @param  $field
		*/

		function ExecuteColumnAdd ($table, $field, $datatype, $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			$t = $table;
			$t = $field;
			$t = $datatype;
			$t = $length;
			$t = $default;
			$t = $null;
			$t = $decimal;
			return '';

		}

		/**
		* OPN_SQL::qstr_array()
		*
		*/

		function qstr_array (&$value, $blobfield = '', $typ = false) {
			if (!is_array ($value) ) {
				if ($blobfield == '') {
					$value = $this->qstr ($value);
				} else {
					$value = $this->qstr ($value, $blobfield);
				}
			} else {
				if ($typ === false) {
					$temp = array_walk ($value, '_opn_qstr', $blobfield);
				} else {
					foreach ($value as $key => $val) {
						if ($typ[$key] == _OOBJ_DTYPE_INT) {
							if ($value[$key] == '') {
								$value[$key] = 0;
							}
						} else {
							_opn_qstr ($value[$key], $key, $blobfield);
						}
					}
				}
			}

		}

		function opn_make_select ($key, &$insert) {
			if (is_array ($key) ) {
				$start = 0;
				foreach ($key as $ourvar) {
					if ($start != 0) {
						$insert .= ' ,';
					}
					$insert .= $ourvar;
					$start++;
				}
			}

		}

		function opn_make_insert ($key, $value, &$insert) {
			if (is_array ($key) ) {
				$start = 0;
				foreach ($key as $ourvar) {
					if (isset($value[$ourvar])) {
						if ($start != 0) {
							$insert .= ' ,';
						}
						$insert .= $value[$ourvar];
						$start++;
					}
				}
			}

		}

		function opn_make_update ($key, $value, &$insert) {
			if (is_array ($key) ) {
				$start = 0;
				foreach ($key as $ourvar) {
					if (isset($value[$ourvar])) {
						if ($start != 0) {
							$insert .= ' ,';
						}
						$insert .= $ourvar . '=' . $value[$ourvar];
						$start++;
					}
				}
			}

		}

		/**
		* OPN_SQL::get_result_array()
		*
		*/

		function get_result_array (&$dat, &$result, $keys = '') {
			if (!is_array ($keys) ) {
				$dat = $result->fields[$keys];
			} else {
				foreach ($keys as $ourvar) {
					$dat[$ourvar] = $result->fields[$ourvar];
				}
			}

		}

		/**
		* OPN_SQL::CreateOpnRegexp()
		*
		* @param  $field
		* @param  $regx
		* @return
		*/

		function CreateOpnRegexp ($field, $regx) {
			if ($regx == '0-9') {
				$regx = '^\[1-9]';
			} elseif ($regx == 'a-z') {
				$regx = '^\[a-z]';
			}
			return '(' . $field . " REGEXP '" . $regx . "')";

		}

		/**
		* OPN_SQL::GetIndex()
		*
		* @param  $table
		* @return
		*/

		function GetIndex ($table) {

			$t = $table;
			return '';

		}

		/**
		* OPN_SQL::isPrimaryIndex()
		*
		* @param  $type
		* @return
		*/

		function isPrimaryIndex ($type) {

			$t = $type;
			return false;

		}

		/**
		* OPN_SQL::TableStatus()
		*
		* @param  $databasename
		* @param  $tablename
		* @return
		*/

		function TableStatus ($databasename, $tablename) {

			$t = $databasename;
			$t = $tablename;
			return '';

		}

		/**
		* OPN_SQL::TableRename()
		*
		* @param  $oldname
		* @param  $newname
		* @return
		*/

		function TableRename ($oldname, $newname) {

			$t = $oldname;
			$t = $newname;
			return '';

		}

		/**
		* OPN_SQL::AddLike()
		*
		* @param  $query
		*/

		function AddLike ($query, $a = '%', $b = '%') {

			global $opnConfig;

			$query = stripslashes ($query);
			$query = str_replace ('%', '', $query);
			$query = $opnConfig['cleantext']->filter_text ($query, true, true, 'nothml');
			$query = $a . $query . $b;
			$query = $this->qstr ($query);
			return $query;

		}

		/**
		* OPN_SQL::AddRepairField()
		*
		* @param  $module
		* @param  $table
		* @param  $field
		* @return
		*/

		function AddRepairField ($module, $table, $field) {

			global $opnConfig;

			$module1 = explode ('/', $module);
			$module1 = $module1[1];
			$file = _OPN_ROOT_PATH . $module . '/plugin/sql/index.php';
			if (file_exists ($file) ) {
				include_once ($file);
				$thefunc = $module1 . '_repair_sql_table';
				$arr = $thefunc ();
				$repairsql = 'ALTER TABLE ' . $opnConfig['tableprefix'] . $table . ' ADD ' . $field . ' ' . $arr['table'][$table][$field];
				$opnConfig['database']->Execute ($repairsql);
			}

		}

		/**
		* Rename a field
		*
		* @param string	$module
		* @param string	$table
		* @param string	$field
		* @param string	$newname
		* @return string
		*/

		function FieldRename ($module, $table, $field, $newname) {

			$t = $module;
			$t = $table;
			$t = $field;
			$t = $newname;
			return '';

		}

		/**
		* OPN_SQL::DropColumn()
		*
		* @param  $table
		* @param  $field
		* @return
		*/

		function DropColumn ($table, $field) {

			global $opnConfig;

			$opnConfig['database']->Execute ('ALTER TABLE ' . $opnConfig['tableprefix'] . $table . ' DROP ' . $field);

		}

		/* OPN_SQL::ChangeTablename()
		*
		* @param  $module
		* @param  $oldname
		* @param  $newname
		* @return
		*/

		function ChangeTablename ($module, $oldname, $newname) {

			global $opnConfig, $opnTables;

			$t = $module;
			$sql = $this->TableRename ($opnConfig['tableprefix'] . $oldname, $opnConfig['tableprefix'] . $newname);
			$opnConfig['database']->Execute ($sql);
			$opnConfig['database']->Execute ('UPDATE ' . $opnConfig['tableprefix'] . 'dbcat' . " SET value1='" . $newname . "' WHERE value1='" . $oldname . "'");
			dbconf_get_tables ($opnTables, $opnConfig);

		}

		/**
		* OPN_SQL::ChangeFieldname()
		*
		* @param  $module
		* @param  $table
		* @param  $oldname
		* @param  $newname
		* @param  $datatype
		* @param  $length
		* @param  $default
		* @param  $null
		* @param  $decimal
		* @return
		*/

		function ChangeFieldname ($module, $table, $oldname, $newname, $datatype = '', $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			$this->FieldRename ($module, $table, $oldname, $newname, $datatype, $length, $default, $null, $decimal);

		}

		/**
		* OPN_SQL::ChangeColumn()
		*
		* @param  $module
		* @param  $table
		* @param  $field
		* @return
		*/

		function ChangeColumn ($module, $table, $field) {

			global $opnConfig;

			$module1 = explode ('/', $module);
			$module1 = $module1[1];
			$file = _OPN_ROOT_PATH . $module . '/plugin/sql/index.php';
			if (file_exists ($file) ) {
				include_once ($file);
				$thefunc = $module1 . '_repair_sql_table';
				$arr = $thefunc ();
				$repairsql = 'ALTER TABLE ' . $opnConfig['tableprefix'] . $table . ' CHANGE ' . $field . ' ' . $field . ' ' . $arr['table'][$table][$field];
				$opnConfig['database']->Execute ($repairsql);
			}

		}

		/**
		* OPN_SQL::GetDBType()
		*
		* @param  $datatype
		* @param integer $length
		* @param unknown $default
		* @param boolean $null
		* @param integer $decimal
		* @return
		*/

		function GetDBType ($datatype, $length = 0, $default = null, $null = false, $decimal = 0) {

			$t = $datatype;
			$t = $length;
			$t = $default;
			$t = $null;
			$t = $decimal;
			return '';

		}

		/**
		* OPN_SQL::GetMetatype()
		*
		* @param  $datatype
		* @return
		*/

		function GetMetatype ($datatype) {

			$t = $datatype;
			return '';

		}

		/**
		* OPN_SQL::GetPrimaryKey()
		*
		* @param  $fields
		* @param string $table
		* @return
		*/

		function GetPrimaryKey ($fields, $table = '') {

			$t = $table;
			$hlp = 'PRIMARY KEY (';
			$j = count ($fields);
			for ($i = 0; $i< $j; $i++) {
				if ($i != 0) {
					$hlp .= ',';
				}
				$hlp .= $fields[$i];
			}
			return $hlp . ')';

		}

		/**
		* OPN_SQL::UpdateBlob()
		*
		* @param  $table
		* @param  $field
		* @param  $value
		* @param  $where
		* @return
		*/

		function UpdateBlob ($table, $field, $value, $where) {

			global $opnConfig;

			$opnConfig['database']->UpdateBlob ($table, $field, $value, $where);

		}

		/**
		* OPN_SQL::isTable()
		*
		* @param  $tablename
		* @return
		*/

		function isTable ($tablename) {

			global $opnConfig, $ininstall;

			static $result = '';
			if ( (!is_array ($result) ) OR ( (isset ($ininstall) ) AND ($ininstall == TRUE) ) ) {
				$result = $opnConfig['database']->MetaTables ();
			}
			return isset ($result[$tablename]);

		}

		function CheckDbField ($dbfield, &$modulfield) {

			$t = $dbfield;
			$t = $modulfield;
			return true;

		}

		/**
		* OPN_SQL::BuildField()
		*
		* @param  $name
		* @param  $maxlenght
		* @param  $type
		* @param boolean $notnull
		* @param boolean $hasdefault
		* @param unknown $defaultvalue
		* @param unknown $scale
		* @return
		*/

		function BuildField ($name, $maxlenght, $type, $notnull = true, $hasdefault = true, $defaultvalue = null, $scale = null) {

			$fld =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerFieldObject;
			$fld->name = $name;
			$fld->max_length = $maxlenght;
			$fld->type = $type;
			$fld->not_null = $notnull;
			$fld->has_default = $hasdefault;
			$fld->default_value = $defaultvalue;
			$fld->scale = $scale;
			return $fld;

		}

		/**
		* Function: get_new_number
		* Purpose : Get the next number for the primary index of a table.
		* Input   : $wichTable = For wich table are the number?
		* 		   $keyField  = Name of the primary index field.
		* Output  : The next number for the primary index.
		*/

		function get_new_number ($wichTable, $keyField) {

			global $opnTables, $opnConfig;

			$get = 'SELECT ' . $keyField . ' AS maxnum FROM ';
			if (isset ($opnTables[$wichTable]) ) {
				$get .= $opnTables[$wichTable];
			} else {
				$get .= $opnConfig['tableprefix'] . $wichTable;
			}
			$retval = 0;
			$get .= ' ORDER BY ' . $keyField . ' DESC';
			$result = $opnConfig['database']->SelectLimit ($get, 1);
			if ($result !== false) {
				if (isset ($result->fields['maxnum']) ) {
					$retval = $result->fields['maxnum']+1;
				} else {
					$retval = 1;
				}
				$result->Close ();
			}
			unset ($result);
			unset ($get);
			if ( (!is_numeric ($retval) ) || ($retval == 0) ) {
				if (is_null ($retval) ) {
					$retval = 1;
				} elseif ( ($retval == '') || ($retval == 0) ) {
					$retval = 1;
				}
			}
			return $retval;

		}

		function ensure_dbwrite ($myselect, $myupdate, $myinsert, $table = '', $keyfield = '') {

			global $opnConfig;

			$result = $opnConfig['database']->Execute ($myselect);
			if (is_object ($result) ) {
				$result_count = $result->RecordCount ();
				$result->Close ();
			} else {
				$result_count = 0;
			}
			unset ($result);
			if ($result_count == 0) {
				if ($table != '') {
					$id = $this->get_new_number ($table, $keyfield);
					$myinsert = sprintf ($myinsert, $id);
					unset ($id);
				}
				$opnConfig['database']->Execute ($myinsert);
			} else {
				$opnConfig['database']->Execute ($myupdate);
			}
			unset ($result_count);

		}

		function opn_dbcoreloader ($dat, $c = 1) {

			global $opnConfig, $opnTables, $root_path, $root_path_datasave;

			$t = $root_path;
			$t = $root_path_datasave;
			unset ($t);
			if ( (isset ($opnTables['opn_script']) ) && (!defined ('_OPN_DBCORELOADER_CODE_' . $dat) ) ) {
				$query = &$opnConfig['database']->Execute ('SELECT src FROM ' . $opnTables['opn_script'] . " WHERE dat='$dat'");
				if ($query !== false) {
					if ($query->RecordCount () == 1) {
						if ($c == 1) {
							if (!function_exists ('open_the_secret') ) {
								include (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'class.crypto.php');
							}
							$i = open_the_secret ($dat, $query->fields['src']);
						} else {
							$i = $query->fields['src'];
						}
						$data = str_replace ('.', '_', $dat);
						if (function_exists ('_code_' . $data . '_int') ) {
							$myfunc = '_code_' . $data . '_int';
							$myfunc ($i);
							eval ($i);
						} else {
							eval (' ?>' . $i . '<?php ');
						}
						define ('_OPN_DBCORELOADER_CODE_' . $dat, 1);
						unset ($i);
					}
					$query->Close ();
				}
				unset ($query);
			}

		}

		/* special do not use this */

		function prefix ($tablename = '') {

			global $opnTables;
			return $opnTables[$tablename];

		}

		function &queryF ($sql, $limit = 0, $start = 0) {

			global $opnConfig;
			if (!empty ($limit) ) {
				return $opnConfig['database']->SelectLimit ($sql, $limit, $start);
			}
			return $opnConfig['database']->Execute ($sql);

		}

		function &query ($sql, $limit = 0, $start = 0) {

			global $opnConfig;
			if (!empty ($limit) ) {
				return $opnConfig['database']->SelectLimit ($sql, $limit, $start);
			}
			return $opnConfig['database']->Execute ($sql);

		}

		function fetchArray ($result) {

			$ar1 = $result->GetArray ();
			echo print_r ($ar1);
			return $ar1[0];

		}

		function fetchRow ($result) {
			return $result->fields;

		}

		/* end special do not use this */

		function CreateTable ($module, $table) {

			global $opnConfig;
			if ($this->_CreateTable ($module, $table, $table) ) {
				$dbcat =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ catalog ($opnConfig, 'dbcat');
				$file = _OPN_ROOT_PATH . $module . '/plugin/sql/index.php';
				$module1 = explode ('/', $module);
				$module1 = $module1[1];
				include_once ($file);
				$tables = array ();
				$myfuncSQLt = $module1 . '_repair_sql_table';
				$tables = $myfuncSQLt ();
				$tables = array_keys ($tables['table']);
				$items = array ();
				$max = count ($tables);
				for ($i = 0; $i< $max; $i++) {
					$items[] = $module1 . ($i+1);
				}
				$max = count ($items);
				for ($i = 0; $i< $max; $i++) {
					if (!$dbcat->catisitemincatalog ($items[$i]) ) {
						$dbcat->catset ($items[$i], $tables[$i]);
					}
				}
				// for
				$dbcat->catsave ();
				$opnTables = array ();
				dbconf_get_tables ($opnTables, $opnConfig);
				unset ($tables);
				unset ($items);
				unset ($dbcat);
			}

		}
		// function

		function DropTableDB ($module, $table) {

			global $opnConfig, $opnTables;

			$drop = $this->TableDrop ($opnConfig['tableprefix'] . $table);
			$opnConfig['database']->Execute ($drop);
			$dbcat =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ catalog ($opnConfig, 'dbcat');
			$items = $dbcat->catlist;
			$items = array_keys ($items);
			$module1 = explode ('/', $module);
			$module1 = $module1[1];
			foreach ($items as $value) {
				if (substr_count ($value, $module1)>0) {
					$dbcat->catremoveitem ($value);
				}
			}
			$file = _OPN_ROOT_PATH . $module . '/plugin/sql/index.php';
			include_once ($file);
			$tables = array ();
			$myfuncSQLt = $module1 . '_repair_sql_table';
			$tables = $myfuncSQLt ();
			$tables = array_keys ($tables['table']);
			$items = array ();
			$max = count ($tables);
			for ($i = 0; $i< $max; $i++) {
				$items[] = $module1 . ($i+1);
			}
			$max = count ($items);
			for ($i = 0; $i< $max; $i++) {
				if (!$dbcat->catisitemincatalog ($items[$i]) ) {
					$dbcat->catset ($items[$i], $tables[$i]);
				}
			}
			// for
			$dbcat->catsave ();
			dbconf_get_tables ($opnTables, $opnConfig);
			unset ($tables);
			unset ($items);
			unset ($dbcat);

		}

		/* function DropTable */

		/**
		*
		* @param $s
		* @param $magic_quotes
		* @return string
		*/

		function qstr ($s, $magic_quotes = false) {

			global $opnConfig;
			if (!is_bool ($magic_quotes) ) {
				$magic_quotes = false;
			}
			// end if
			return $opnConfig['database']->qstr ($s, $magic_quotes);

		}
		// end function

		function UpdateBlobs ($table, $where, $blobtype = 'BLOB') {

			$t = $table;
			$t = $where;
			$t = $blobtype;
			$this->_blobcache = false;
			return true;

		}
		// end function

		/**
		* Resets the blobcache
		*
		*/

		function UnsetBlobcache () {

			$this->_blobcache = false;

		}

		/* Privat */

		function _buildid ($prefix) {

			mt_srand ((double)microtime ()*1000000);
			$idsuffix = mt_rand ();
			return $prefix . '-id-' . $idsuffix;

		}

		function _CreateTable ($module, $name, $table, $nodata = false) {

			include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'class.installer.php');
			$module1 = explode ('/', $module);
			$module1 = $module1[1];
			$file = _OPN_ROOT_PATH . $module . '/plugin/sql/index.php';
			if (file_exists ($file) ) {
				include_once ($file);
				$thefunc = $module1 . '_repair_sql_table';
				$arr = $thefunc ();
				$sql['table'][$name] = $arr['table'][$table];
				$thefunc1 = $module1 . '_repair_sql_index';
				if (function_exists ($thefunc1) ) {
					$arr = $thefunc1 ();
					if (isset ($arr['index'][$table]) == true) {
						$sql['index'][$name] = $arr['index'][$table];
					}
				}
				$thefunc1 = $module1 . '_repair_sql_data';
				if (function_exists ($thefunc1) ) {
					$arr = $thefunc1 ();
					if (isset ($arr['data'][$table]) == true) {
						$sql['data'][$name] = $arr['data'][$table];
					}
				}
				if ($nodata) {
					if (isset ($sql['data']) == true) {
						unset ($sql['data']);
					}
				}
				$installer =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ OPN_AbstractPluginInDeinstaller;
				$installer->opnExecuteSQL ($sql);
				unset ($arr);
				unset ($sql);
				return true;
			}
			return false;

		}

	}
}
// if

?>