<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_DRIVERMYSQL_INCLUDED') ) {
	define ('_OPN_CLASS_DRIVERMYSQL_INCLUDED', 1);

	/**
	* Extends the SQL Connection class for MySQL Database access
	*
	* @var string	$databaseType
	* @var string	$dataProvider
	* @var string	$metaTablesSQL
	* @var string	$metaColumnsSQL
	* @var boolean	$forceNewConnect
	* @var integer	$clientFlags
	*/

	class SQLLayer_mysql extends SQLConnection {

		/*<{php4}>
		var $databaseType = 'mysql';
		var $dataProvider = 'mysql';
		var $metaTablesSQL = 'SHOW TABLES';
		var $metaColumnsSQL = 'SHOW COLUMNS FROM %s';
		var $forceNewConnect = false;
		var $clientFlags = 0;
		<{php4/}>*/

		/*<{php5}>
		public $databaseType = 'mysql';
		public $dataProvider = 'mysql';
		public $metaTablesSQL = 'SHOW TABLES';
		public $metaColumnsSQL = 'SHOW COLUMNS FROM %s';
		public $forceNewConnect = false;
		public $clientFlags = 0;
		<{php5/}>*/

		/*<{delete}>*/
		var $databaseType = 'mysql';
		var $dataProvider = 'mysql';
		var $metaTablesSQL = 'SHOW TABLES';
		var $metaColumnsSQL = 'SHOW COLUMNS FROM %s';
		var $forceNewConnect = false;
		var $clientFlags = 0;
		/*<{delete/}>*/

		/**
		* Constructor for SQLLayer_mysql
		*
		* @access public
		* @return void
		*/

		function SQLLayer_mysql () {
			if (!extension_loaded ('mysql') ) {
				trigger_error ('You must have the mysql extension installed.', E_USER_ERROR);
			}
			$this->fetchMode = true;

		}

		/**
		* IfNull
		*
		* @access public
		* @param string	$field
		* @param string	$ifNull
		* @return string
		*/

		function IfNull ($field, $ifNull) {
			return ' IFNULL(' . $field . ', ' . $ifNull . ') ';

		}

		/**
		* Return's the tables in the database.
		*
		* @access public
		* @param mixed		$ttype
		* @param string	$showSchema
		* @param string	$mask
		* @return array	tables in database
		*/

		function MetaTables ($ttype = false, $showSchema = false, $mask = false) {

			$save = $this->metaTablesSQL;
			if ($showSchema && is_string ($showSchema) ) {
				$this->metaTablesSQL .= ' FROM ' . $showSchema;
			}
			if ($mask) {
				$mask = $this->qstr ($mask);
				$this->metaTablesSQL .= ' LIKE ' . $mask;
			}
			$ret = &$this->_MetaTables ($ttype, $showSchema);
			$this->metaTablesSQL = $save;
			return $ret;

		}

		/**
		* List indexes on a table as an array.
		*
		* @access public
		* @param string	$table
		* @param bool		$primary
		* @param mixed		$owner
		* @return mixed boolean or array
		*/

		function &MetaIndexes ($table, $primary = false, $owner = false) {

			$t = $owner;
			// save old fetch mode
			$false = false;
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			// get index details
			$rs = $this->Execute (sprintf ('SHOW INDEX FROM %s', $table) );
			// restore fetchmode
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}
			if (!is_object ($rs) ) {
				return $false;
			}
			$indexes = array ();
			// parse index data into array
			while ($row = $rs->FetchRow () ) {
				if ($primary == false AND $row[2] == 'PRIMARY') {
					continue;
				}
				if (!isset ($indexes[$row[2]]) ) {
					$indexes[$row[2]] = array ('unique' => ($row[1] == 0),
								'columns' => array () );
				}
				$indexes[$row[2]]['columns'][$row[3]-1] = $row[4];
			}
			unset ($row);
			// sort columns by order in the index
			foreach (array_keys ($indexes) as $index) {
				ksort ($indexes[$index]['columns']);
			}
			return $indexes;

		}

		/**
		* Correctly quotes a string so that all strings are escaped.
		*
		* @access public
		* @param string	$s				- string to quote
		* @param boolean	$magic_quotes	- use magic quotes
		* @return string
		*/

		function qstr ($s, $magic_quotes = false) {
			if (!$magic_quotes) {
				if (function_exists ('mysql_real_escape_string') ) {
					// only in PHP 4.3.0 and up
					if (is_resource ($this->_connectionID) ) {
						return "'" . mysql_real_escape_string ($s, $this->_connectionID) . "'";
					}
				}
				if ($this->replaceQuote[0] == '\\') {
					$s = str_replace (array ('\\',
								"\0"),
								array ('\\\\',
						"\\\0"),
						$s);
				}
				return "'" . str_replace ("'", $this->replaceQuote, $s) . "'";
			}
			// undo magic quotes for "
			$s = str_replace ('\\"', '"', $s);
			return "'$s'";

		}

		/**
		* Get number of affected rows.
		*
		* @access private
		* @return int
		*/

		function _affectedrows () {
			return mysql_affected_rows ($this->_connectionID);

		}

		/**
		* Return the databases that the driver can connect to.
		*
		* @access public
		* @return array
		*/

		function &MetaDatabases () {

			$qid = mysql_list_dbs ($this->_connectionID);
			$arr = array ();
			$i = 0;
			$max = mysql_num_rows ($qid);
			while ($i< $max) {
				$db = mysql_tablename ($qid, $i);
				if ($db != 'mysql') {
					$arr[] = $db;
				}
				$i += 1;
			}
			return $arr;

		}

		/**
		* Connect to the database on $argHostname server
		*
		* @access private
		* @param string	$argHostname	- hostname of the database server
		* @param string	$argUsername	- username to use
		* @param string	$argPassword	- password for username
		* @param string	$argDatabasename	- name of the databese to connect to
		* @return boolean	success
		*/

		function _connect ($argHostname, $argUsername, $argPassword, $argDatabasename) {
			if (version_compare (phpversion (), '4.3.0', 'ge') == 1) {
				$this->_connectionID = mysql_connect ($argHostname, $argUsername, $argPassword, $this->forceNewConnect, $this->clientFlags);
			} elseif (version_compare (phpversion (), '4.2.0') == 1) {
				$this->_connectionID = mysql_connect ($argHostname, $argUsername, $argPassword, $this->forceNewConnect);
			} else {
				$this->_connectionID = mysql_connect ($argHostname, $argUsername, $argPassword);
			}
			if ($this->_connectionID === false) {
				return false;
			}
			if ($argDatabasename) {
				return $this->SelectDB ($argDatabasename);
			}
			return true;

		}

		/**
		* Persitente connection to the database.
		*
		* @access private
		* @param string	$argHostname	- hostname of the database server
		* @param string	$argUsername	- username to use
		* @param string	$argPassword	- password for username
		* @param string	$argDatabasename	- name of the databese to connect to
		* @return boolean	success
		*/

		function _pconnect ($argHostname, $argUsername, $argPassword, $argDatabasename) {
			if (version_compare (phpversion (), '4.3.0') == 1) {
				$this->_connectionID = mysql_pconnect ($argHostname, $argUsername, $argPassword, $this->clientFlags);
			} else {
				$this->_connectionID = mysql_pconnect ($argHostname, $argUsername, $argPassword);
			}
			if ($this->_connectionID === false) {
				return false;
			}
			if ($this->autoRollback) {
				$this->RollbackTrans ();
			}
			if ($argDatabasename) {
				return $this->SelectDB ($argDatabasename);
			}
			return true;

		}

		/**
		* Forced a new connection.
		*
		* @access private
		* @param string	$argHostname	- hostname of the database server
		* @param string	$argUsername	- username to use
		* @param string	$argPassword	- password for username
		* @param string	$argDatabasename	- name of the databese to connect to
		* @return boolean	success
		*/

		function _nconnect ($argHostname, $argUsername, $argPassword, $argDatabasename) {

			$this->forceNewConnect = true;
			return $this->_connect ($argHostname, $argUsername, $argPassword, $argDatabasename);

		}

		/**
		* List columns in a table as an array of SQLLayerFieldObjects.
		*
		* @access public
		* @param string	$table - name of the table to get colums from
		* @return mixed boolean or array
		*/

		function &MetaColumns ($table) {
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			$rs = $this->Execute (sprintf ($this->metaColumnsSQL, $table) );
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}
			if (!is_object ($rs) ) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$retarr = array ();
			while (! $rs->EOF) {
				$fld =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerFieldObject;
				$fld->name = $rs->fields[0];
				$type = $rs->fields[1];
				// split type into type(length):
				$fld->scale = null;
				$query_array1 = '';
				$query_array2 = '';
				if (strpos ($type, ',') && preg_match ("/^(.+)\((\d+),(\d+)/", $type, $query_array1) ) {
					$fld->type = $query_array1[1];
					$fld->max_length = is_numeric ($query_array1[2])? $query_array1[2] : -1;
					$fld->scale = is_numeric ($query_array1[3])? $query_array1[3] : -1;
				} elseif (preg_match ("/^(.+)\((\d+)/", $type, $query_array2) ) {
					$fld->type = $query_array2[1];
					$fld->max_length = is_numeric ($query_array2[2])? $query_array2[2] : -1;
				} else {
					$fld->max_length = -1;
					$fld->type = $type;
				}
				$fld->not_null = ($rs->fields[2] != 'YES');
				$fld->primary_key = ($rs->fields[3] == 'PRI');
				$fld->auto_increment = (strpos ($rs->fields[5], 'auto_increment') !== false);
				$fld->binary = (strpos ($type, 'blob') !== false);
				$fld->unsigned = (strpos ($type, 'unsigned') !== false);
				if (!$fld->binary) {
					$d = $rs->fields[4];
					if ($d != 'NULL') {
						$fld->has_default = true;
						switch ($fld->type) {
							case 'tinyint':
							case 'smallint':
							case 'mediumint':
							case 'int':
							case 'bigint':
								$fld->default_value = (int) $d;
								break;
							case 'float':
							case 'double':
								$fld->default_value = (float) $d;
								break;
							case 'char':
							case 'varchar':
							case 'tinytext':
							case 'text':
							case 'mediumtext':
							case 'longtext':
								$fld->default_value = (string) $d;
								break;
							default:
								$fld->default_value = $d;
								break;
						}
					} else {
						$fld->has_default = false;
					}
				}
				$retarr[strtolower ($fld->name)] = $fld;
				$rs->MoveNext ();
			}
			unset ($fld);
			$rs->Close ();
			return $retarr;

		}

		/**
		* Choose a database to connect to.
		*
		* @access public
		* @param string	$dbName
		* @return boolean
		*/

		function SelectDB ($dbName) {

			$success = false;
			if ($this->_connectionID) {
				$success = @mysql_select_db ($dbName, $this->_connectionID);
				if ($success) {
					$this->databaseName = $dbName;
				}
			}
			return $success;

		}

		/**
		* select records, start getting rows from $offset (1-based), for $nrows.
		* Attention: Parameters use PostgreSQL convention, not MySQL
		*
		* @access public
		* @param string	$sql
		* @param int		$nrows
		* @param int		$offset
		* @param mixed		$inputarr
		* @return object
		*/

		function &SelectLimit ($sql, $nrows = -1, $offset = -1, $inputarr = false) {

			$offsetStr = ($offset >= 0)? $offset . ', ' : '';
			if ($nrows<0) {
				$nrows = '18446744073709551615';
			}
			$rs = &$this->Execute ($sql . ' LIMIT ' . $offsetStr . $nrows, $inputarr);
			return $rs;

		}

		/**
		* Excute the query.
		*
		* @access private
		* @param string	$sql
		* @param mixed		$inputarr
		* @return mixed resource or boolean
		*/

		function _query ($sql, $inputarr) {

			$t = $inputarr;
			return mysql_query ($sql, $this->_connectionID);

		}

		/**
		* returns the last error message from previous database operation
		*
		* @access public
		* @return string
		*/

		function ErrorMsg () {
			if (empty ($this->_connectionID) ) {
				$this->_errorMsg = @mysql_error ();
			} else {
				$this->_errorMsg = @mysql_error ($this->_connectionID);
			}
			return $this->_errorMsg;

		}

		/**
		* returns the last error number from previous database operation
		*
		* @access public
		* @return number
		*/

		function ErrorNo () {
			if (empty ($this->_connectionID) ) {
				$this->_errorCode = @mysql_errno ();
			} else {
				$this->_errorCode = @mysql_errno ($this->_connectionID);
			}
			return $this->_errorCode;

		}

		/**
		* Close the connection to the database
		*
		* @access private
		* @return void
		*/

		function _close () {

			@mysql_close ($this->_connectionID);
			$this->_connectionID = false;

		}

		/**
		* Maximum size of C field
		*
		* @access public
		* @return number
		*/

		function CharMax () {
			return 255;

		}

		/**
		* Maximum size of X field
		*
		* @access public
		* @return number
		*/

		function TextMax () {
			return 4294967295;

		}

	}
	// class SQLLayer_mysql

	/**
	* Extends the RecordSet class that represents the dataset returned by the database
	* for MySQL Database access
	*
	* @var string	$databaseType
	* @var boolean	$canSeek
	*/

	class SQLLayerRecordSet_mysql extends SQLLayerRecordSet {
		var $databaseType = 'mysql';
		var $canSeek = true;

		/**
		* Constructor
		*
		* @access public
		* @param ressource	$queryID
		* @param bool		$mode
		* @return void
		*/

		function SQLLayerRecordSet_mysql ($queryID, $mode = true) {
			if ($mode) {
				$this->fetchMode = MYSQL_ASSOC;
			} else {
				$this->fetchMode = MYSQL_NUM;
			}
			$this->SQLLayerRecordSet ($queryID);

		}

		/**
		* Initialize recordset
		*
		* @access private
		* @return void
		*/

		function _initrs () {

			$this->_numOfRows = @mysql_num_rows ($this->_queryID);
			$this->_numOfFields = @mysql_num_fields ($this->_queryID);

		}

		/**
		* Get the SQLLayerFieldObject of a specific column.
		*
		* @access public
		* @param int	$fieldOffset
		* @return object
		*/

		function &FetchField ($fieldOffset = -1) {
			if ($fieldOffset != -1) {
				$o = @mysql_fetch_field ($this->_queryID, $fieldOffset);
				$f = @mysql_field_flags ($this->_queryID, $fieldOffset);
				$o->max_length = @mysql_field_len ($this->_queryID, $fieldOffset);
				// $o->max_length = -1; // mysql returns the max length less spaces -- so it is unrealiable
				$o->binary = (strpos ($f, 'binary') !== false);
			} elseif ($fieldOffset == -1) {

				/*	The $fieldOffset argument is not provided thus its -1 	*/

				$o = @mysql_fetch_field ($this->_queryID);
				$o->max_length = @mysql_field_len ($this->_queryID, 0);
				// $o->max_length = -1; // mysql returns the max length less spaces -- so it is unrealiable
			}
			return $o;

		}

		/**
		* returns a row as associative array
		*
		* @param bool	$upper
		* @return array
		*/

		function &GetRowAssoc ($upper = true) {
			if ($this->fetchMode == MYSQL_ASSOC && !$upper) {
				return $this->fields;
			}
			$row = &$this->_GetRowAssoc ($upper);
			return $row;

		}

		/**
		* Get the value of a field in the current row by column name.
		*
		* @access public
		* @param string	$colname
		* @return array
		*/

		function Fields ($colname) {
			if ($this->fetchMode != MYSQL_NUM) {
				return @ $this->fields[$colname];
			}
			if (!$this->bind) {
				$this->bind = array ();
				for ($i = 0, $max = $this->_numOfFields; $i< $max; $i++) {
					$o = $this->FetchField ($i);
					$this->bind[strtolower ($o->name)] = $i;
				}
			}
			return $this->fields[$this->bind[strtolower ($colname)]];

		}

		/**
		* Adjusts the result pointer to an arbitary row in the result.
		*
		* @access public
		* @param int	$row
		* @return mixed boolean or number
		*/

		function _seek ($row) {
			if ($this->_numOfRows == 0) {
				return false;
			}
			return @mysql_data_seek ($this->_queryID, $row);

		}

		/**
		* Fetch a record.
		*
		* @access public
		* @return bool
		*/

		function _fetch () {

			$this->fields = @mysql_fetch_array ($this->_queryID, $this->fetchMode);
			if ( ($this->fetchMode == MYSQL_ASSOC) && (is_array ($this->fields) ) ) {
				$this->_convert_to_lowercase ();
			}
			return is_array ($this->fields);

		}

		/**
		* Close the recordset
		*
		* @access private
		* @return void
		*/

		function _close () {

			@mysql_free_result ($this->_queryID);
			$this->_queryID = false;

		}

		/**
		* Get the metatype of the column
		*
		* @access public
		* @param mixed		$t
		* @param int		$len
		* @param object	$fieldobj
		* @return string
		*/

		function MetaType ($t, $len = -1, $fieldobj = object) {
			if (is_object ($t) ) {
				$fieldobj = $t;
				$t = $fieldobj->type;
				$len = $fieldobj->max_length;
			}

			## ich seh nix wo die benutzt wird. Daher mal deaktiviert - Alex

			#			$len = -1; // mysql max_length is not accurate
			switch (strtolower ($t) ) {
				case 'STRING':
				case 'CHAR':
				case 'VARCHAR':
				case 'TINYBLOB':
				case 'TINYTEXT':
				case 'ENUM':
				case 'SET':
					return 'C';
				case 'TEXT':
				case 'LONGTEXT':
				case 'MEDIUMTEXT':
					return 'X';
				// php_mysql extension always returns 'blob' even if 'text'
				// so we have to check whether binary...
				case 'IMAGE':
				case 'LONGBLOB':
				case 'BLOB':
				case 'MEDIUMBLOB':
					return !empty ($fieldobj->binary)?'B' : 'X';
				case 'YEAR':
				case 'DATE':
					return 'D';
				case 'TIME':
				case 'DATETIME':
				case 'TIMESTAMP':
					return 'T';
				case 'INT':
				case 'INTEGER':
				case 'BIGINT':
				case 'TINYINT':
				case 'MEDIUMINT':
				case 'SMALLINT':
					if (!empty ($fieldobj->primary_key) ) {
						return 'R';
					}
					return 'I';
				default:
					return 'N';
			}

		}

	}
	// class SQLLayerRecordSet_mysql

	class opn_mysql extends OPN_SQL {

		/*<{php4}>
		var $DropTable='DROP TABLE IF EXISTS %s';
		var $DropIndex='ALTER TABLE %s DROP INDEX %s';
		var $ShowTableStatus = 'SHOW TABLE STATUS FROM %s';
		<{php4/}>*/

		/*<{php5}>
		public $DropTable='DROP TABLE IF EXISTS %s';
		public $DropIndex='ALTER TABLE %s DROP INDEX %s';
		public $ShowTableStatus = 'SHOW TABLE STATUS FROM %s';
		<{php5/}>*/

		/*<{delete}>*/
		var $DropTable='DROP TABLE IF EXISTS %s';
		var $DropIndex='ALTER TABLE %s DROP INDEX %s';
		var $ShowTableStatus = 'SHOW TABLE STATUS FROM %s';
		/*<{delete/}>*/

		function opn_mysql () {

			$this->CanOptimize = true;
			$this->CanRepair = true;
			$this->HasStatus = true;
			$this->CanAlterColumnAdd = true;
			$this->CanAlterColumnChange = true;
			$this->IndexCreate = 'ALTER TABLE %s ADD %s INDEX idx%sidx%s %s';
			$this->CanAddRepairField = false;
			$this->CanChangeRepairField = false;
			$this->CanExecuteColumnAdd = true;
			$this->CanExecuteColumnChange = true;

		}

		function TableComment ($tablename, $comment) {
			return 'ALTER TABLE ' . $tablename . " COMMENT = '" . $comment . "'";

		}

		function TableOptimize ($tablename) {
			return 'OPTIMIZE TABLE ' . $tablename;

		}

		function TableStatus ($databasename, $tablename) {
			return 'SHOW TABLE STATUS FROM ' . $databasename . ' LIKE ' . $tablename;

		}

		function TableRepair ($tablename) {
			return 'REPAIR TABLE ' . $tablename . ' EXTENDED';

		}

		function ExecuteColumnAdd ($table, $field, $datatype, $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			global $opnConfig;

			$opnConfig['database']->Execute ('alter table ' . $opnConfig['tableprefix'] . $table . ' ADD ' . $field . ' ' . $opnConfig['opnSQL']->GetDBType ($datatype, $length, $default, $null, $decimal) );

		}

		function ExecuteColumnChange ($table, $field, $datatype, $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			global $opnConfig;

			$opnConfig['database']->Execute ('alter table ' . $opnConfig['tableprefix'] . $table . ' change ' . $field . ' ' . $field . ' ' . $opnConfig['opnSQL']->GetDBType ($datatype, $length, $default, $null, $decimal) );

		}

		function TableDrop ($tablename) {
			return 'DROP TABLE IF EXISTS ' . $tablename;

		}

		function TableLock ($tablename, $lockmode = _OPNSQL_TABLOCK_ACCESS_EXCLUSIVE) {

			global $opnConfig;

			$t = $lockmode;
			if ( (isset ($opnConfig['opn_db_lock_use']) ) AND ($opnConfig['opn_db_lock_use'] == 1) ) {
				$opnConfig['database']->Execute ('LOCK TABLES ' . $tablename . ' WRITE');
			}

		}

		function TableRename ($oldname, $newname) {
			return 'ALTER TABLE ' . $oldname . ' RENAME ' . $newname;

		}

		function FieldRename ($module, $table, $oldname, $newname, $datatype = '', $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			global $opnConfig;

			$t = $module;
			$opnConfig['database']->Execute ('ALTER TABLE ' . $opnConfig['tableprefix'] . $table . ' CHANGE ' . $oldname . ' ' . $newname . ' ' . $this->GetDBType ($datatype, $length, $default, $null, $decimal) );

		}

		function IndexDrop ($index, $table = '') {
			return 'ALTER TABLE ' . $table . ' DROP INDEX ' . $index;

		}

		function PrimaryIndexDrop ($table) {
			return 'ALTER TABLE ' . $table . ' DROP PRIMARY KEY';

		}

		function CreateIndex ($unique, $idxname, $idxcounter, $table, $fields) {
			return sprintf ($this->IndexCreate, $table, $unique, $idxname, $idxcounter, $fields);

		}

		function GetIndex ($table) {
			return 'SHOW INDEX FROM ' . $table;

		}

		function TableUnLock ($tablename) {

			global $opnConfig;

			$t = $tablename;
			if ( (isset ($opnConfig['opn_db_lock_use']) ) AND ($opnConfig['opn_db_lock_use'] == 1) ) {
				$opnConfig['database']->Execute ('UNLOCK TABLES');
			}

		}

		function GetDBType ($datatype, $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			global $opnConfig;
			if ( (isset ($opnConfig['return_tableinfo_as_array']) ) && ($opnConfig['return_tableinfo_as_array'] === TRUE) ) {
				if ($decimal == 0) {
					$decimal = null;
				}
				return $opnConfig['opnSQL']->BuildField ('', $length, $datatype, ! $null, !is_null ($default), $default, $decimal);
			}
			$hlp = '';
			if ($length == '') {
				$length = 0;
			}
			if ($datatype == _OPNSQL_TINYINT) {
				$hlp .= 'TINYINT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_SMALLINT) {
				$hlp .= 'SMALLINT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_MEDIUMINT) {
				$hlp .= 'MEDIUMINT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_INT) {
				$hlp .= 'INT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_BIGINT) {
				$hlp .= 'BIGINT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_FLOAT) {
				if ($length != 0) {
					$hlp .= 'FLOAT (' . $length . ',' . $decimal . ')';
				} else {
					$hlp .= 'FLOAT';
				}
			} elseif ($datatype == _OPNSQL_DOUBLE) {
				$hlp .= 'DOUBLE (' . $length . ',' . $decimal . ')';
			} elseif ($datatype == _OPNSQL_NUMERIC) {
				$hlp .= 'NUMERIC (' . $length . ',' . $decimal . ')';
			} elseif ($datatype == _OPNSQL_DATE) {
				$hlp .= 'DATE';
			} elseif ($datatype == _OPNSQL_DATETIME) {
				$hlp .= 'DATETIME';
			} elseif ($datatype == _OPNSQL_TIMESTAMP) {
				$hlp .= 'TIMESTAMP';
			} elseif ($datatype == _OPNSQL_TIME) {
				$hlp .= 'TIME';
			} elseif ($datatype == _OPNSQL_CHAR) {
				$hlp .= 'CHAR (' . $length . ')';
			} elseif ($datatype == _OPNSQL_VARCHAR) {
				$hlp .= 'VARCHAR (' . $length . ')';
			} elseif ($datatype == _OPNSQL_TINYBLOB) {
				$hlp .= 'TINYBLOB';
			} elseif ($datatype == _OPNSQL_TINYTEXT) {
				$hlp .= 'TINYTEXT';
			} elseif ($datatype == _OPNSQL_BLOB) {
				$hlp .= 'BLOB';
			} elseif ($datatype == _OPNSQL_TEXT) {
				$hlp .= 'TEXT';
			} elseif ($datatype == _OPNSQL_MEDIUMBLOB) {
				$hlp .= 'MEDIUMBLOB';
			} elseif ($datatype == _OPNSQL_MEDIUMTEXT) {
				$hlp .= 'MEDIUMTEXT';
			} elseif ($datatype == _OPNSQL_BIGBLOB) {
				$hlp .= 'LONGBLOB';
			} elseif ($datatype == _OPNSQL_BIGTEXT) {
				$hlp .= 'LONGTEXT';
			}
			// if
			if ($default !== NULL) {
				$hlp .= ' DEFAULT ';
				if (strlen ($default) == 0) {
					$hlp .= "''";
				} else {
					$hlp .= "'" . $default . "'";
				}
			}
			if ($null) {
			} else {
				$hlp .= ' NOT NULL';
			}
			return $hlp;

		}

		function GetDbFieldtype (&$modulefield) {
			switch ($modulefield->type) {
				case _OPNSQL_TINYINT:
					$modulefield->type = 'tinyint';
					break;
				case _OPNSQL_SMALLINT:
					$modulefield->type = 'smallint';
					break;
				case _OPNSQL_MEDIUMINT:
					$modulefield->type = 'mediumint';
					break;
				case _OPNSQL_INT:
					$modulefield->type = 'int';
					break;
				case _OPNSQL_BIGINT:
					$modulefield->type = 'bigint';
					break;
				case _OPNSQL_FLOAT:
					$modulefield->type = 'float';
					break;
				case _OPNSQL_DOUBLE:
					$modulefield->type = 'double';
					break;
				case _OPNSQL_NUMERIC:
					$modulefield->type = 'decimal';
					break;
				case _OPNSQL_DATE:
					$modulefield->type = 'date';
					break;
				case _OPNSQL_DATETIME:
					$modulefield->type = 'datetime';
					break;
				case _OPNSQL_TIMESTAMP:
					$modulefield->type = 'timestamp';
					break;
				case _OPNSQL_TIME:
					$modulefield->type = 'time';
					break;
				case _OPNSQL_CHAR:
					$modulefield->type = 'char';
					break;
				case _OPNSQL_VARCHAR:
					$modulefield->type = 'varchar';
					break;
				case _OPNSQL_TINYBLOB:
					$modulefield->type = 'tinyblob';
					break;
				case _OPNSQL_TINYTEXT:
					$modulefield->type = 'tinytext';
					break;
				case _OPNSQL_BLOB:
					$modulefield->type = 'blob';
					break;
				case _OPNSQL_TEXT:
					$modulefield->type = 'text';
					break;
				case _OPNSQL_MEDIUMBLOB:
					$modulefield->type = 'mediumblob';
					break;
				case _OPNSQL_MEDIUMTEXT:
					$modulefield->type = 'mediumtext';
					break;
				case _OPNSQL_BIGBLOB:
					$modulefield->type = 'longblob';
					break;
				case _OPNSQL_BIGTEXT:
					$modulefield->type = 'longtext';
					break;
			}
			// switch

		}

		function CheckDBField ($dbfield, &$modulefield) {

			$this->GetDbFieldtype ($modulefield);
			$help = '';
			if ($dbfield->name != $modulefield->name) {
				$help[] = _OPNSQL_FIELDERRORNAME;
			}
			if ( ($dbfield->max_length != -1) && ($dbfield->max_length != $modulefield->max_length) ) {
				$help[] = _OPNSQL_FIELDERRORLENGTH;
			}
			if ( ($modulefield->type == 'varchar') && ($modulefield->max_length<4) ) {
				$modulefield->type = 'char';
			}
			if ($dbfield->type != $modulefield->type) {
				$help[] = _OPNSQL_FIELDERRORTYPE;
			}
			if ($dbfield->not_null != $modulefield->not_null) {
				$help[] = _OPNSQL_FIELDERRORNOTNULL;
			}
			if ($dbfield->type != 'blob') {
				if ( ($modulefield->has_default) && ($dbfield->has_default != $modulefield->has_default) ) {
					$help[] = _OPNSQL_FIELDERRORHASDEFAULT;
				}
			}
			if ( (!is_null ($dbfield->scale) ) && ($dbfield->scale != $modulefield->scale) ) {
				$help[] = _OPNSQL_FIELDERRORSCALE;
			}
			return $help;

		}

	}
}
// if

?>