<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_DRIVERMYSQLI_INCLUDED') ) {
	define ('_OPN_CLASS_DRIVERMYSQLI_INCLUDED', 1);

	/**
	* Extends the SQL Connection class for (Improved) MySQL Database access
	*
	* @var string	$databaseType
	* @var string	$dataProvider
	* @var string	$metaTablesSQL
	* @var string	$metaColumnsSQL
	* @var boolean	$forceNewConnect
	* @var integer	$clientFlags
	* @var string	$port
	* @var boolean	$socket
	* @var array	$_bindInputArray
	*/

	class SQLLayer_mysqli extends SQLConnection {

		/*<{php4}>
		<{php4/}>*/

		/*<{php5}>
		<{php5/}>*/

		/*<{delete}>*/
		/*<{delete/}>*/
		var $databaseType = 'mysqli';
		var $dataProvider = 'native';
		var $metaTablesSQL = 'SHOW TABLES';
		var $metaColumnsSQL = 'SHOW COLUMNS FROM %s';
		var $forceNewConnect = false;
		var $clientFlags = 0;
		var $port = false;
		var $socket = false;
		var $_bindInputArray = false;

		/**
		* Constructor for SQLLayer_mysqli
		*
		* @access public
		* @return void
		*/

		function SQLLayer_mysqli () {
			if (!extension_loaded ('mysqli') ) {
				trigger_error ('You must have the mysqli extension installed.', E_USER_ERROR);
			}
			$this->fetchMode = true;

		}

		/**
		* Connect to the database.
		*
		* @access private
		* @param string	$argHostname
		* @param string	$argUsername
		* @param string	$argPassword
		* @param string	$argDatabasename
		* @param bool		$persist
		* @return boolean	success
		*/

		function _connect ($argHostname = NULL, $argUsername = NULL, $argPassword = NULL, $argDatabasename = NULL, $persist = false) {

			$t = $persist;
			$this->_connectionID = @mysqli_init ();
			if (is_null ($this->_connectionID) ) {
				// mysqli_init only fails if insufficient memory
				opnErrorHandler (E_WARNING, 'mysqli_init() failed : ' . $this->ErrorMsg (), __FILE__, __LINE__);
				return false;
			}
			// Set connection options
			// Not implemented now
			// mysqli_options($this->_connection,,);
			if (mysqli_real_connect ($this->_connectionID, $argHostname, $argUsername, $argPassword, $argDatabasename, $this->port, $this->socket, $this->clientFlags) ) {
				if ($argDatabasename) {
					return $this->SelectDB ($argDatabasename);
				}
				return true;
			}
			opnErrorHandler (E_WARNING, 'Could\'t connect : ' . $this->ErrorMsg (), __FILE__, __LINE__);
			return false;

		}

		/**
		* Persistent connection to the database (not supported in mysqli).
		*
		* @access private
		* @param string	$argHostname
		* @param string	$argUsername
		* @param string	$argPassword
		* @param string	$argDatabasename
		* @return boolean
		*/

		function _pconnect ($argHostname, $argUsername, $argPassword, $argDatabasename) {
			return $this->_connect ($argHostname, $argUsername, $argPassword, $argDatabasename, true);

		}

		/**
		* Forced a new connection to the database.
		*
		* @access private
		* @param string	$argHostname
		* @param string	$argUsername
		* @param string	$argPassword
		* @param string	$argDatabasename
		* @return boolean
		*/

		function _nconnect ($argHostname, $argUsername, $argPassword, $argDatabasename) {

			$this->forceNewConnect = true;
			return $this->_connect ($argHostname, $argUsername, $argPassword, $argDatabasename);

		}

		/**
		* IfNull
		*
		* @access public
		* @param string	$field
		* @param string	$ifNull
		* @return string
		*/

		function IfNull ($field, $ifNull) {
			return ' IFNULL(' . $field . ', ' . $ifNull . ') ';

		}

		/**
		* Begin a transaction.
		*
		* @access public
		* @return boolean
		*/

		function BeginTrans () {
			if (!$this->transOff) {
				$this->transCnt += 1;
				$this->Execute ('SET AUTOCOMMIT=0');
				$this->Execute ('BEGIN');
			}
			return true;

		}

		/**
		* Commit a transaction.
		*
		* @access public
		* @param boolean	$ok
		* @return boolean
		*/

		function CommitTrans ($ok = true) {
			if (!$this->transOff) {
				if (!$ok) {
					return $this->RollbackTrans ();
				}
				if ($this->transCnt) {
					$this->transCnt -= 1;
				}
				$this->Execute ('COMMIT');
				$this->Execute ('SET AUTOCOMMIT=1');
			}
			return true;

		}

		/**
		* Rollback a transaction.
		*
		* @access public
		* @return boolean
		*/

		function RollbackTrans () {
			if (!$this->transOff) {
				if ($this->transCnt) {
					$this->transCnt -= 1;
				}
				$this->Execute ('ROLLBACK');
				$this->Execute ('SET AUTOCOMMIT=1');
			}
			return true;

		}

		/**
		* Correctly quotes a string so that all strings are escaped. We prefix and append
		* to the string single-quotes.
		*
		* @access public
		* @param string	$s
		* @param boolean		$magic_quotes
		* @return string
		*/

		function qstr ($s, $magic_quotes = false) {
			if (!$magic_quotes) {
				if (version_compare (phpversion (), '5.0.0', 'ge') == 1) {
					return "'" . mysqli_real_escape_string ($this->_connectionID, $s) . "'";
				}
				if ($this->replaceQuote[0] == '\\') {
					$s = str_replace (array ('\\',
								"\0"),
								array ('\\\\',
						"\\\0"),
						$s);
				}
				return "'" . str_replace ("'", $this->replaceQuote, $s) . "'";
			}
			// undo magic quotes for "
			$s = str_replace ('\\"', '"', $s);
			return "'$s'";

		}

		/**
		* Get the number of rows affected by last query
		* Only works for INSERT, UPDATE and DELETE query's
		*
		* @access private
		* @return number
		*/

		function _affectedrows () {

			$result = @mysqli_affected_rows ($this->_connectionID);
			if ($result == -1) {
				opnErrorHandler (E_WARNING, 'mysqli_affected_rows() failed : ' . $this->ErrorMsg (), __FILE__, __LINE__);
			}
			return $result;

		}

		/**
		* Return the databases that the driver can connect to.
		*
		* @access public
		* @return array
		*/

		function &MetaDatabases () {

			$query = 'SHOW DATABASES';
			$ret = &$this->Execute ($query);
			return $ret;

		}

		/**
		* list indexes on a table as an array.
		*
		* @access public
		* @param string	$table
		* @param boolean	$primary
		* @return mixed boolean or array
		*/

		function &MetaIndexes ($table, $primary = false) {
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			// get index details
			$rs = $this->Execute (sprintf ('SHOW INDEXES FROM %s', $table) );
			// restore fetchmode
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}
			if (!is_object ($rs) ) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$indexes = array ();
			// parse index data into array
			while ($row = $rs->FetchRow () ) {
				if ($primary == false && $row[2] == 'PRIMARY') {
					continue;
				}
				if (!isset ($indexes[$row[2]]) ) {
					$indexes[$row[2]] = array ('unique' => ($row[1] == 0),
								'columns' => array () );
				}
				$indexes[$row[2]]['columns'][$row[3]-1] = $row[4];
			}
			// sort columns by order in the index
			$temparr = array_keys ($indexes);
			foreach ($temparr as $index) {
				ksort ($indexes[$index]['columns']);
			}
			return $indexes;

		}

		/**
		* Returns concatenated string
		* Much easier to run "mysqld --ansi" or "mysqld --sql-mode=PIPES_AS_CONCAT" and use || operator
		*
		* @access public
		* @return string
		*/

		function Concat () {

			$arr = func_get_args ();
			$s = implode (',', $arr);
			if (strlen ($s)>0) {
				return 'CONCAT(' . $s . ')';
			}
			return '';

		}

		/**
		* List columns in a database as an array
		*
		* @access public
		* @param string	$table
		* @return mixed boolean or array
		*/

		function &MetaColumns ($table) {
			if (!$this->metaColumnsSQL) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			$rs = $this->Execute (sprintf ($this->metaColumnsSQL, $table) );
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}
			if (!is_object ($rs) ) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$retarr = array ();
			while (! $rs->EOF) {
				$fld =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerFieldObject ();
				$fld->name = $rs->fields[0];
				$type = $rs->fields[1];
				// split type into type(length):
				$fld->scale = null;
				$query_array1 = '';
				$query_array2 = '';
				if (strpos ($type, ',') && preg_match ("/^(.+)\((\d+),(\d+)/", $type, $query_array1) ) {
					$fld->type = $query_array1[1];
					$fld->max_length = is_numeric ($query_array1[2])? $query_array1[2] : -1;
					$fld->scale = is_numeric ($query_array1[3])? $query_array1[3] : -1;
				} elseif (preg_match ("/^(.+)\((\d+)/", $type, $query_array2) ) {
					$fld->type = $query_array2[1];
					$fld->max_length = is_numeric ($query_array2[2])? $query_array2[2] : -1;
				} else {
					$fld->max_length = -1;
					$fld->type = $type;
				}
				$fld->not_null = false;
				if ( ($rs->fields[2] != 'YES') || ($rs->fields[2] != 'NO') ) {
					$fld->not_null = true;
				}
				$fld->primary_key = ($rs->fields[3] == 'PRI');
				$fld->auto_increment = (strpos ($rs->fields[5], 'auto_increment') !== false);
				$fld->binary = (strpos ($type, 'blob') !== false);
				$fld->unsigned = (strpos ($type, 'unsigned') !== false);
				if (!$fld->binary) {
					$d = $rs->fields[4];
					if ($d != 'NULL') {
						$fld->has_default = true;
						switch ($fld->type) {
							case 'tinyint':
							case 'smallint':
							case 'mediumint':
							case 'int':
							case 'bigint':
								$fld->default_value = (int) $d;
								break;
							case 'float':
							case 'double':
								$fld->default_value = (float) $d;
								break;
							case 'char':
							case 'varchar':
							case 'tinytext':
							case 'text':
							case 'mediumtext':
							case 'longtext':
								$fld->default_value = (string) $d;
								break;
							default:
								$fld->default_value = $d;
								break;
						}
					} else {
						$fld->has_default = false;
					}
				}
				$retarr[strtolower ($fld->name)] = $fld;
				$rs->MoveNext ();
			}
			unset ($fld);
			$rs->Close ();
			unset ($rs);
			return $retarr;

		}

		/**
		* Choose a database to connect to
		*
		* @access public
		* @param string	$dbName
		* @return boolean
		*/

		function SelectDB ($dbName) {

			$result = false;
			if ($this->_connectionID) {
				$result = @mysqli_select_db ($this->_connectionID, $dbName);
				if (!$result) {
					opnErrorHandler (E_WARNING, 'Selection of database ' . $dbName . ' failed ' . $this->ErrorMsg (), __FILE__, __LINE__);
				} else {
					$this->databaseName = $dbName;
				}
			}
			return $result;

		}

		/**
		* select records, start getting rows from $offset (1-based), for $nrows.
		*
		* @access public
		* @param string	$sql
		* @param integer	$nrows
		* @param integer	$offset
		* @param array		$inputarr
		* @param boolean	$arg3
		* @return object
		*/

		function &SelectLimit ($sql, $nrows = -1, $offset = -1, $inputarr = false, $arg3 = false) {

			$offsetStr = ($offset >= 0)? $offset . ', ' : '';
			$rs = &$this->Execute ($sql . ' LIMIT ' . $offsetStr . $nrows, $inputarr, $arg3);
			return $rs;

		}

		/**
		* prepare the sql statement and return the stmt resource
		*
		* @access public
		* @param string	$sql
		* @return mixed string or array
		*/

		function Prepare ($sql) {

			# macht ja sonst wenig Sinn - Alex

			#			return $sql;

			$stmt = mysqli_prepare ($sql);
			if (!$stmt) {
				echo $this->ErrorMsg ();
				return $sql;
			}
			return array ($sql,
					$stmt);

		}

		/**
		* Excute the query
		*
		* @access private
		* @param mixed string or array	$sql
		* @param array					$inputarr
		* @return mixed object or boolean
		*/

		function _query ($sql, $inputarr) {
			if (is_array ($sql) ) {
				$stmt = $sql[1];
				$a = '';
				foreach ($inputarr as $v) {
					if (is_string ($v) ) {
						$a .= 's';
					} elseif (is_integer ($v) ) {
						$a .= 'i';
					} else {
						$a .= 'd';
					}
				}
				$fnarr = &array_merge (array ($stmt,
								$a),
								$inputarr);
				$ret = call_user_func_array ('mysqli_stmt_bind_param', $fnarr);
				$ret = mysqli_stmt_execute ($stmt);
				return $ret;
			}
			if (!$mysql_res = mysqli_query ($this->_connectionID, $sql, MYSQLI_STORE_RESULT) ) {
				opnErrorHandler (E_WARNING, 'Query: ' . $sql . ' failed. ' . $this->ErrorMsg (), __FILE__, __LINE__);
				return false;
			}
			return $mysql_res;

		}

		/**
		* returns the last error message from previous database operation
		*
		* @access public
		* @return string
		*/

		function ErrorMsg () {
			if (empty ($this->_connectionID) ) {
				$this->_errorMsg = @mysqli_error ();
			} else {
				$this->_errorMsg = @mysqli_error ($this->_connectionID);
			}
			return $this->_errorMsg;

		}

		/**
		* returns the last error number from previous database operation
		*
		* @access public
		* @return number
		*/

		function ErrorNo () {
			if (empty ($this->_connectionID) ) {
				$this->_errorCode = @mysqli_errno ();
			} else {
				$this->_errorCode = @mysqli_errno ($this->_connectionID);
			}
			return $this->_errorCode;

		}

		/**
		* Close the connection to the database
		*
		* @access private
		* @return void
		*/

		function _close () {

			@mysqli_close ($this->_connectionID);
			$this->_connectionID = false;

		}

		/**
		* Maximum size of C field
		*
		* @access public
		* @return number
		*/

		function CharMax () {
			return 255;

		}

		/**
		* Maximum size of X field
		*
		* @access public
		* @return number
		*/

		function TextMax () {
			return 4294967295;

		}

	}

	/* class SQLLayer_mysqli */

	/**
	* Extends the RecordSet class that represents the dataset returned by the database
	* for (Improved) MySQL Database access
	*
	* @var string	$databaseType
	* @var boolean	$canSeek
	*/

	class SQLLayerRecordSet_mysqli extends SQLLayerRecordSet {
		var $databaseType = 'mysqli';
		var $canSeek = true;

		/**
		* Constructor
		*
		* @access public
		* @param ressource	$id
		* @param boolean	$mode
		* @return void
		*/

		function SQLLayerRecordSet_mysqli ($id, $mode = true) {
			if ($mode) {
				$this->fetchMode = MYSQLI_ASSOC;
			} else {
				$this->fetchMode = MYSQLI_NUM;
			}
			$this->SQLLayerRecordSet ($id);

		}

		/**
		* Initialize recordset
		*
		* @access private
		* @return void
		*/

		function _initrs () {

			$this->_numOfRows = @mysqli_num_rows ($this->_queryID);
			$this->_numOfFields = @mysqli_num_fields ($this->_queryID);

		}

		/**
		* Get the SQLLayerFieldObject of a specific column.
		*
		* @access public
		* @param int	$fieldOffset
		* @return object
		*/

		function &FetchField ($fieldOffset = -1) {

			$fieldnr = $fieldOffset;
			if ($fieldOffset != -1) {
				$fieldOffset = mysqli_field_seek ($this->_queryID, $fieldnr);
			}
			$object = mysqli_fetch_field ($this->_queryID);
			return $object;

		}

		/**
		* returns a row as associative array
		*
		* @access public
		* @param int	$upper
		* @return array
		*/

		function &GetRowAssoc ($upper = true) {
			if ($this->fetchMode == MYSQLI_ASSOC && !$upper) {
				return $this->fields;
			}
			$row = &$this->_GetRowAssoc ($upper);
			return $row;

		}

		/**
		* Get the value of a field in the current row by column name.
		*
		* @access public
		* @param string	$colname
		* @return mixed string or number
		*/

		function Fields ($colname) {
			if ($this->fetchMode != MYSQLI_NUM) {
				return @ $this->fields[$colname];
			}
			if (!$this->bind) {
				$this->bind = array ();
				for ($i = 0, $max = $this->_numOfFields; $i< $max; $i++) {
					$object = $this->FetchField ($i);
					$this->bind[strtolower ($object->name)] = $i;
				}
			}
			return $this->fields[$this->bind[strtolower ($colname)]];

		}

		/**
		* Adjusts the result pointer to an arbitary row in the result
		*
		* @access private
		* @param integer	$row
		* @return boolean
		*/

		function _seek ($row) {
			if ($this->_numOfRows == 0 || $row<0) {
				return false;
			}
			$success = mysqli_data_seek ($this->_queryID, $row);
			$this->EOF = false;
			return $success;

		}

		/**
		* set $fields to row values
		*
		* @access private
		* @return bool	success
		*/

		function _fetch () {

			$this->fields = mysqli_fetch_array ($this->_queryID, $this->fetchMode);
			if ( ($this->fetchMode == MYSQLI_ASSOC) && (is_array ($this->fields) ) ) {
				$this->_convert_to_lowercase ();
			}
			return is_array ($this->fields);

		}

		/**
		* Close the recordset
		*
		* @access private
		* @return void
		*/

		function _close () {

			mysqli_free_result ($this->_queryID);
			$this->_queryID = false;

		}

		/**
		* Get the metatype of the column
		*
		* @access public
		* @param mixed object or string	$t
		* @param int						$len
		* @param object					$fieldobj
		* @return string
		*/

		function MetaType ($t, $len = -1, $fieldobj = object) {
			if (is_object ($t) ) {
				$fieldobj = $t;
				$t = $fieldobj->type;
				$len = $fieldobj->max_length;
			}
			$len = -1;
			// mysql max_length is not accurate
			switch (strtolower ($t) ) {
				case MYSQLI_TYPE_TINY_BLOB:
				case MYSQLI_TYPE_CHAR:
				case MYSQLI_TYPE_STRING:
				case MYSQLI_TYPE_ENUM:
				case MYSQLI_TYPE_SET:
				case 253:
					if ($len<=$this->blobSize) {
						return 'C';
					}
					return 'X';
				case MYSQLI_TYPE_BLOB:
				case MYSQLI_TYPE_LONG_BLOB:
				case MYSQLI_TYPE_MEDIUM_BLOB:
					return !empty ($fieldobj->binary)?'B' : 'X';
				case MYSQLI_TYPE_DATE:
				case MYSQLI_TYPE_YEAR:
					return 'D';
				case MYSQLI_TYPE_DATETIME:
				case MYSQLI_TYPE_NEWDATE:
				case MYSQLI_TYPE_TIME:
				case MYSQLI_TYPE_TIMESTAMP:
					return 'T';
				case MYSQLI_TYPE_INT24:
				case MYSQLI_TYPE_LONG:
				case MYSQLI_TYPE_LONGLONG:
				case MYSQLI_TYPE_SHORT:
				case MYSQLI_TYPE_TINY:
					if (!empty ($fieldobj->primary_key) ) {
						return 'R';
					}
					return 'I';
				default:
					if (!is_numeric ($t) ) {
						echo '<p>--- Error in type matching ' . $t . ' -----</p>';
					}
					return 'N';
			}

		}
		// function

	}

	/* class SQLLayerRecordSet_mysqli */

	class opn_mysqli extends OPN_SQL {

		/*<{php4}>
		var $DropTable='DROP TABLE IF EXISTS %s';
		var $DropIndex='ALTER TABLE %s DROP INDEX %s';
		var $ShowTableStatus = 'SHOW TABLE STATUS FROM %s';
		<{php4/}>*/

		/*<{php5}>
		public $DropTable='DROP TABLE IF EXISTS %s';
		public $DropIndex='ALTER TABLE %s DROP INDEX %s';
		public $ShowTableStatus = 'SHOW TABLE STATUS FROM %s';
		<{php5/}>*/

		/*<{delete}>*/
		var $DropTable='DROP TABLE IF EXISTS %s';
		var $DropIndex='ALTER TABLE %s DROP INDEX %s';
		var $ShowTableStatus = 'SHOW TABLE STATUS FROM %s';
		/*<{delete/}>*/

		function opn_mysqli () {

			$this->CanOptimize = true;
			$this->CanRepair = true;
			$this->HasStatus = true;
			$this->CanAlterColumnAdd = true;
			$this->CanAlterColumnChange = true;
			$this->IndexCreate = 'ALTER TABLE %s ADD %s INDEX idx%sidx%s %s';
			$this->CanAddRepairField = false;
			$this->CanChangeRepairField = false;
			$this->CanExecuteColumnAdd = true;
			$this->CanExecuteColumnChange = true;

		}

		function TableComment ($tablename, $comment) {
			return 'ALTER TABLE ' . $tablename . " COMMENT = '" . $comment . "'";

		}

		function TableOptimize ($tablename) {
			return 'OPTIMIZE TABLE ' . $tablename;

		}

		function TableStatus ($databasename, $tablename) {
			return 'SHOW TABLE STATUS FROM ' . $databasename . ' LIKE ' . $tablename;

		}

		function TableRepair ($tablename) {
			return 'REPAIR TABLE ' . $tablename . ' EXTENDED';

		}

		function ExecuteColumnAdd ($table, $field, $datatype, $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			global $opnConfig;

			$opnConfig['database']->Execute ('alter table ' . $opnConfig['tableprefix'] . $table . ' ADD ' . $field . ' ' . $opnConfig['opnSQL']->GetDBType ($datatype, $length, $default, $null, $decimal) );

		}

		function ExecuteColumnChange ($table, $field, $datatype, $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			global $opnConfig;

			$opnConfig['database']->Execute ('alter table ' . $opnConfig['tableprefix'] . $table . ' change ' . $field . ' ' . $field . ' ' . $opnConfig['opnSQL']->GetDBType ($datatype, $length, $default, $null, $decimal) );

		}

		function TableDrop ($tablename) {
			return 'DROP TABLE IF EXISTS ' . $tablename;

		}

		function TableLock ($tablename, $lockmode = _OPNSQL_TABLOCK_ACCESS_EXCLUSIVE) {

			global $opnConfig;

			$t = $lockmode;
			if ( (isset ($opnConfig['opn_db_lock_use']) ) AND ($opnConfig['opn_db_lock_use'] == 1) ) {
				$opnConfig['database']->Execute ('LOCK TABLES ' . $tablename . ' WRITE');
			}

		}

		function TableRename ($oldname, $newname) {
			return 'ALTER TABLE ' . $oldname . ' RENAME ' . $newname;

		}

		function FieldRename ($module, $table, $oldname, $newname, $datatype = '', $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			global $opnConfig;

			$t = $module;
			$opnConfig['database']->Execute ('ALTER TABLE ' . $opnConfig['tableprefix'] . $table . ' CHANGE ' . $oldname . ' ' . $newname . ' ' . $this->GetDBType ($datatype, $length, $default, $null, $decimal) );

		}

		function IndexDrop ($index, $table = '') {
			return 'ALTER TABLE ' . $table . ' DROP INDEX ' . $index;

		}

		function PrimaryIndexDrop ($table) {
			return 'ALTER TABLE ' . $table . ' DROP PRIMARY KEY';

		}

		function CreateIndex ($unique, $idxname, $idxcounter, $table, $fields) {
			return sprintf ($this->IndexCreate, $table, $unique, $idxname, $idxcounter, $fields);

		}

		function GetIndex ($table) {
			return 'SHOW INDEX FROM ' . $table;

		}

		function TableUnLock ($tablename) {

			global $opnConfig;

			$t = $tablename;
			if ( (isset ($opnConfig['opn_db_lock_use']) ) AND ($opnConfig['opn_db_lock_use'] == 1) ) {
				$opnConfig['database']->Execute ('UNLOCK TABLES');
			}

		}

		function GetDBType ($datatype, $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			global $opnConfig;
			if ( (isset ($opnConfig['return_tableinfo_as_array']) ) && ($opnConfig['return_tableinfo_as_array'] === TRUE) ) {
				if ($decimal == 0) {
					$decimal = null;
				}
				return $opnConfig['opnSQL']->BuildField ('', $length, $datatype, ! $null, !is_null ($default), $default, $decimal);
			}
			$hlp = '';
			if ($length == '') {
				$length = 0;
			}
			if ($datatype == _OPNSQL_TINYINT) {
				$hlp .= 'TINYINT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_SMALLINT) {
				$hlp .= 'SMALLINT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_MEDIUMINT) {
				$hlp .= 'MEDIUMINT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_INT) {
				$hlp .= 'INT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_BIGINT) {
				$hlp .= 'BIGINT (' . $length . ')';
			} elseif ($datatype == _OPNSQL_FLOAT) {
				if ($length != 0) {
					$hlp .= 'FLOAT (' . $length . ',' . $decimal . ')';
				} else {
					$hlp .= 'FLOAT';
				}
			} elseif ($datatype == _OPNSQL_DOUBLE) {
				$hlp .= 'DOUBLE (' . $length . ',' . $decimal . ')';
			} elseif ($datatype == _OPNSQL_NUMERIC) {
				$hlp .= 'NUMERIC (' . $length . ',' . $decimal . ')';
			} elseif ($datatype == _OPNSQL_DATE) {
				$hlp .= 'DATE';
			} elseif ($datatype == _OPNSQL_DATETIME) {
				$hlp .= 'DATETIME';
			} elseif ($datatype == _OPNSQL_TIMESTAMP) {
				$hlp .= 'TIMESTAMP';
			} elseif ($datatype == _OPNSQL_TIME) {
				$hlp .= 'TIME';
			} elseif ($datatype == _OPNSQL_CHAR) {
				$hlp .= 'CHAR (' . $length . ')';
			} elseif ($datatype == _OPNSQL_VARCHAR) {
				$hlp .= 'VARCHAR (' . $length . ')';
			} elseif ($datatype == _OPNSQL_TINYBLOB) {
				$hlp .= 'TINYBLOB';
			} elseif ($datatype == _OPNSQL_TINYTEXT) {
				$hlp .= 'TINYTEXT';
			} elseif ($datatype == _OPNSQL_BLOB) {
				$hlp .= 'BLOB';
			} elseif ($datatype == _OPNSQL_TEXT) {
				$hlp .= 'TEXT';
			} elseif ($datatype == _OPNSQL_MEDIUMBLOB) {
				$hlp .= 'MEDIUMBLOB';
			} elseif ($datatype == _OPNSQL_MEDIUMTEXT) {
				$hlp .= 'MEDIUMTEXT';
			} elseif ($datatype == _OPNSQL_BIGBLOB) {
				$hlp .= 'LONGBLOB';
			} elseif ($datatype == _OPNSQL_BIGTEXT) {
				$hlp .= 'LONGTEXT';
			}
			// if
			if ($default !== NULL) {
				$hlp .= ' DEFAULT ';
				if (strlen ($default) == 0) {
					$hlp .= "''";
				} else {
					$hlp .= "'" . $default . "'";
				}
			}
			if ($null) {
			} else {
				$hlp .= ' NOT NULL';
			}
			return $hlp;

		}

		function GetDbFieldtype (&$modulefield) {
			switch ($modulefield->type) {
				case _OPNSQL_TINYINT:
					$modulefield->type = 'tinyint';
					break;
				case _OPNSQL_SMALLINT:
					$modulefield->type = 'smallint';
					break;
				case _OPNSQL_MEDIUMINT:
					$modulefield->type = 'mediumint';
					break;
				case _OPNSQL_INT:
					$modulefield->type = 'int';
					break;
				case _OPNSQL_BIGINT:
					$modulefield->type = 'bigint';
					break;
				case _OPNSQL_FLOAT:
					$modulefield->type = 'float';
					break;
				case _OPNSQL_DOUBLE:
					$modulefield->type = 'double';
					break;
				case _OPNSQL_NUMERIC:
					$modulefield->type = 'decimal';
					break;
				case _OPNSQL_DATE:
					$modulefield->type = 'date';
					break;
				case _OPNSQL_DATETIME:
					$modulefield->type = 'datetime';
					break;
				case _OPNSQL_TIMESTAMP:
					$modulefield->type = 'timestamp';
					break;
				case _OPNSQL_TIME:
					$modulefield->type = 'time';
					break;
				case _OPNSQL_CHAR:
					$modulefield->type = 'char';
					break;
				case _OPNSQL_VARCHAR:
					$modulefield->type = 'varchar';
					break;
				case _OPNSQL_TINYBLOB:
					$modulefield->type = 'tinyblob';
					break;
				case _OPNSQL_TINYTEXT:
					$modulefield->type = 'tinytext';
					break;
				case _OPNSQL_BLOB:
					$modulefield->type = 'blob';
					break;
				case _OPNSQL_TEXT:
					$modulefield->type = 'text';
					break;
				case _OPNSQL_MEDIUMBLOB:
					$modulefield->type = 'mediumblob';
					break;
				case _OPNSQL_MEDIUMTEXT:
					$modulefield->type = 'mediumtext';
					break;
				case _OPNSQL_BIGBLOB:
					$modulefield->type = 'longblob';
					break;
				case _OPNSQL_BIGTEXT:
					$modulefield->type = 'longtext';
					break;
			}
			// switch

		}

		function CheckDBField ($dbfield, &$modulefield) {

			$this->GetDbFieldtype ($modulefield);
			$help = '';
			if ($dbfield->name != $modulefield->name) {
				$help[] = _OPNSQL_FIELDERRORNAME;
			}
			if ( ($dbfield->max_length != -1) && ($dbfield->max_length != $modulefield->max_length) ) {
				$help[] = _OPNSQL_FIELDERRORLENGTH;
			}
			if ( ($modulefield->type == 'varchar') && ($modulefield->max_length<4) ) {
				$modulefield->type = 'char';
			}
			if ($dbfield->type != $modulefield->type) {
				$help[] = _OPNSQL_FIELDERRORTYPE;
			}
			if ($dbfield->not_null != $modulefield->not_null) {
				$help[] = _OPNSQL_FIELDERRORNOTNULL;
			}
			if ($dbfield->type != 'blob') {
				if ( ($modulefield->has_default) && ($dbfield->has_default != $modulefield->has_default) ) {
					$help[] = _OPNSQL_FIELDERRORHASDEFAULT;
				}
			}
			if ( (!is_null ($dbfield->scale) ) && ($dbfield->scale != $modulefield->scale) ) {
				$help[] = _OPNSQL_FIELDERRORSCALE;
			}
			return $help;

		}

	}
}

?>