<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/

/* based on GeoClass by multimediamotz, Stefan Motz
thank you for letting us use this software */
if (!defined ('_OPN_MAINFILE_INCLUDED') ) {
	include ('../../../mainfile.php');
}
getLanguageDef (_OPN_ROOT_PATH . 'modules/geodb/class/language/');
include_once (_OPN_ROOT_PATH . 'modules/geodb/class/class.geoobject.php');

/**
* Unit constants
*
* _GEODB_UNIT_KM => kilometers
* _GEODB_UNIT_MI => miles
* _GEODB_UNIT_IN => inches
* _GEODB_UNIT_SM => sea-miles
*/

define ('_GEODB_UNIT_KM', 1);
define ('_GEODB_UNIT_MI', 2);
define ('_GEODB_UNIT_IN', 3);
define ('_GEODB_UNIT_SM', 4);
define ('_GEODB_UNIT_DEFAULT', _GEODB_UNIT_KM);

/**
* This constant contains the radius of the earth in kilometers
* _GEODB_EARTH_RADIUS is set to the mean value: 6371. km
* equatorial radius as of WGS84: 6378.137 km
*/

define ('_GEODB_EARTH_RADIUS', 6371.0);
define ('_GEODB_ORIENTATION', 'ORIENTATION_SHORT');
$cfgStrings['ORIENTATION_SHORT'] = explode (',', _GEODB_ORIENTATION_SHORT);
$cfgStrings['ORIENTATION_LONG'] = explode (',', _GEODB_ORIENTATION_LONG);
$cfgStrings[_GEODB_UNIT_KM] = 'km';
$cfgStrings[_GEODB_UNIT_MI] = 'miles';
$cfgStrings[_GEODB_UNIT_IN] = 'inch';
$cfgStrings[_GEODB_UNIT_SM] = 'sm';

/**
* The main "geodb" class is simply a container class with some static
* methods for creating Geo objects as well as some utility functions
* common to all parts of Geo.
*
* The object model of Geo is as follows (indentation means inheritance):
*
* Geo			 The main Geo class. This is simply a utility class
*				 with some "static" methods for creating Geo objects as
*				 well as common utility functions for other Geo classes.
*
* Geo_Common	  The base for each source implementation.
* |
* +-Geo_DB		The source implementation for a common database.
*   |
*   +-Geo_DB_Nima The source implementation for a Nima database.
*				 Inherits Geo_DB. When calling Geo::setupSource('DB_Nima'),
*				 the object returned is an instance of this class.
*
* geodb_map		 This class draws maps of GeoObjects and e00 files
*
* geodb_object	  Object with name, longitude, latitude
*				 and additional information (depends on the source
*				 implementation)
*
* @access   public
* @package  Geo
*/

class geodb {

	/**
	* Creates a new geodb_map object
	*
	* @access  public
	* @param   int	 $x
	* @param   int	 $y
	* @return  object   a newly created Geo object, or a Error object on error
	*/

	function &setupMap ($x, $y = -1) {

		include_once (_OPN_ROOT_PATH . 'modules/geodb/class/class.geomap.php');
		$obj = & new geodb_map ($x, $y);
		return $obj;

	}

	/**
	* Converts degrees/minutes/seconds to degrees
	*
	* Converts a string which represents a latitude/longitude as degree/minutes/seconds
	* to a float degree value. If no valid string is passed it will return 0.
	*
	* @access  public
	* @param   string  $dms  latitude/longitude as degree/minutes/seconds
	* @return  float   degree
	*/

	function dms2deg ($dms) {

		global $cfgStrings;

		$negativeSigns = array ($cfgStrings['ORIENTATION_SHORT'][4],
					$cfgStrings['ORIENTATION_SHORT'][6],
					"-");
		$negativeSignsString = $cfgStrings['ORIENTATION_SHORT'][4] . $cfgStrings['ORIENTATION_SHORT'][6];
		if (strlen ($dms) == 6) {
			$dms = '0' . $dms;
		} elseif (strlen ($dms) == 5) {
			$dms = '00' . $dms;
		}
		$searchPattern = "|\s*([" . $negativeSignsString . "\-\+]?)\s*(\d{1,3})[\\s]*(\d{1,2})[\'\s]*(\d{1,2})([\,\.]*)(\d*)[\'\"\s]*([" . $negativeSignsString . "\-\+]?)|i";
		$result = '';
		if (preg_match ($searchPattern, $dms, $result) ) {
			if (in_array (strtoupper ($result[1]), $negativeSigns) || in_array (strtoupper ($result[7]), $negativeSigns) ) {
				$algSign = -1;
			} else {
				$algSign = 1;
			}
			if ( ( (1*$result[2])>360) || ($result[3] >= 60) || ($result[4] >= 60) ) {
				opnErrorHandler (E_WARNING, 'Values out of range', __FILE__, __LINE__);
				return false;
			}
			return $algSign* ($result[2]+ ( ($result[3]+ ( ($result[4] . '.' . $result[6])*10/6)/100)*10/6)/100);
		}
		trigger_error ('No DMS-Format (Like 51 24\' 32.123\'\' W)');
		return FALSE;

	}

	/**
	* Converts a float value to degrees/minutes/seconds
	*
	* Converts a float value to degrees/minutes/second (e.g. 50.1833300 to 50 10' 60'') '
	* The seconds could contain the number of decimal places one passes to the optional
	* parameter $decPlaces. The direction (N, S, W, E) must be added manually
	* (e.g. $output = "E ".deg2dms(7.441944); )
	*
	* @access  public
	* @param   float   $degFloat
	* @param   int	 $decPlaces
	* @return  string  degrees minutes seconds
	*/

	function deg2dms ($degFloat, $decPlaces = 0) {

		$degree = abs (floor ($degFloat) );
		$minSec = 60* ($degFloat- $degree);
		$minutes = floor ($minSec);
		$seconds = round ( ($minSec- $minutes)*60, $decPlaces);
		return $degree . ' ' . $minutes . "' " . $seconds . "''";

	}

	/**
	* Returns the radius of the earth
	*
	* Returns the radius of the earth in the given unit.
	* _GEODB_EARTH_RADIUS is set to the mean value: 6371. km
	* equatorial radius as of WGS84: 6378.137 km
	*
	* @access  public
	* @param   int	 $unit  use the _GEODB_UNIT_* constants
	*/

	function getEarthRadius ($unit = _GEODB_UNIT_DEFAULT) {
		switch ($unit) {
			case _GEODB_UNIT_KM:
				// kilometers
				return _GEODB_EARTH_RADIUS;
			case _GEODB_UNIT_MI:
				// Miles
				return _GEODB_EARTH_RADIUS*0.621371;
			case _GEODB_UNIT_IN:
				// Zoll/Inch
				return _GEODB_EARTH_RADIUS*39370.08;
			case _GEODB_UNIT_SM:
				// See-Miles
				return _GEODB_EARTH_RADIUS*0.5399568;
			default:
				return _GEODB_EARTH_RADIUS;
		}

	}

	/**
	* Returns an array with all countries from database
	*
	* Returns all know countries from databse. The values are
	* return in parameter by ref
	*
	* @access  public
	* @param   array	 $countriesarr
	*/

	function get_countryarray (&$countriesarr) {

		global $opnConfig, $opnTables;

		$sql = 'SELECT adm0, adm0name FROM ' . $opnTables['geodb_adm0'];
		$result = &$opnConfig['database']->Execute ($sql);
		$countriesarr = array ();
		if (!$result === false) {
			while (! $result->EOF) {
				$temp = $result->fields['adm0'];
				$countriesarr[$temp] = $result->fields['adm0name'];
				$result->MoveNext ();
			}
		}

	}

	/**
	* Returns an array with all states from database
	*
	* Returns all know states from databse. The values are
	* return in parameter by ref
	*
	* @access  public
	* @param   array	 $statearr
	*/

	function get_statearray (&$statearr) {

		global $opnConfig, $opnTables;

		$sql = 'SELECT adm0, adm1, adm1name FROM ' . $opnTables['geodb_adm1'];
		$result = &$opnConfig['database']->Execute ($sql);
		$statearr = array ();
		if (!$result === false) {
			while (! $result->EOF) {
				$temp = $result->fields['adm0'];
				$temp2 = $result->fields['adm1'];
				$statearr[$temp][$temp2] = $result->fields['adm1name'];
				$result->MoveNext ();
			}
		}

	}

	/**
	* Returns number of records in locations that match
	* to the search criteria
	*
	*
	* @access  public
	* @param   string	 $lookupfor
	*/

	function counthits ($lookupfor = '') {

		global $opnConfig, $opnTables;

		$reccount = 0;
		if (preg_match ('/^[0-9]{1,5}$/', $lookupfor) ) {
			$targetfield = 'plz';
		} else {
			$targetfield = 'locationsname';
		}
		$like_search = $opnConfig['opnSQL']->AddLike ($lookupfor);
		$sql = 'SELECT COUNT(locationsid) AS counter FROM ' . $opnTables['geodb_locations'] . ' WHERE ' . $targetfield . ' LIKE ' . $like_search;
		$result = &$opnConfig['database']->SelectLimit ($sql, 1);
		if ( (is_object ($result) ) && (isset ($result->fields['counter']) ) ) {
			$reccount = $result->fields['counter'];
		}
		return $reccount;

	}

	/**
	* Returns records in locations that match
	* to the search criteria as array
	*
	* @access  public
	* @param   array		$hitsarr
	* @param   string	 $lookupfor
	*/

	function gethitsarray (&$hitsarr, $lookupfor = '') {

		global $opnConfig, $opnTables;
		if (preg_match ('/^[0-9]{1,5}$/', $lookupfor) ) {
			$targetfield = 'plz';
		} else {
			$targetfield = 'locationsname';
		}
		$fields = array ('locationsid',
				'locationsname',
				'typ',
				'adm0',
				'adm1',
				'adm2',
				'adm3',
				'adm4',
				'ort',
				'plz');
		$like_search = $opnConfig['opnSQL']->AddLike ($lookupfor);
		$sql = 'SELECT ' . implode (',', $fields) . ' FROM ' . $opnTables['geodb_locations'] . ' WHERE ' . $targetfield . ' LIKE ' . $like_search;
		$result = &$opnConfig['database']->Execute ($sql);
		$hitsarr = array ();
		if (!$result === false) {
			$count = 0;
			while (! $result->EOF) {
				foreach ($fields AS $field) {
					$hitsarr[$count][$field] = $result->fields ($field);
				}
				$count++;
				$result->MoveNext ();
			}
		}

	}

	/**
	* Find GeoObject
	*
	* Returns a GeoObject which fits the $locationid
	*
	* @access  public
	* @param   string	$locationid
	* @return   object	 $mylocation
	*/

	function getlocationarray ($locationid = '') {

		global $opnConfig, $opnTables;

		$fields = array ('locationsid',
				'locationsname',
				'typ',
				'adm0',
				'adm1',
				'adm2',
				'adm3',
				'adm4',
				'ort',
				'breite',
				'laenge',
				'gs',
				'kfz',
				'plz');
		$sql = 'SELECT ' . implode (',', $fields) . ' FROM ' . $opnTables['geodb_locations'] . ' WHERE locationsid="' . $locationid . '"';
		$result = &$opnConfig['database']->SelectLimit ($sql, 1);
		$mylocation = $this->transformQueryResult ($result, $fields);
		return $mylocation[0];

	}

	/**
	* Find GeoObjects near an overgiven GeoObject
	*
	* Searches for GeoObjects, which are in a specified radius around the passed GeoBject.
	* Default is radius of 100 (100 of specified unit, see configuration and maxHits of 50
	* Returns an array of GeoDB-objects which lie in ther radius of the passed GeoObject.
	*
	* @access  public
	* @param   object  &$geoObject
	* @param   int	 $maxRadius
	* @param   int	 $maxHits
	* @return  array
	*/

	function findCloseByGeoObjects (&$geoObject, $maxRadius = 100, $maxHits = 50) {

		global $opnConfig, $opnTables;

		$fields = array ('locationsid',
				'locationsname',
				'typ',
				'adm0',
				'adm1',
				'adm2',
				'adm3',
				'adm4',
				'ort',
				'breite',
				'laenge',
				'gs',
				'kfz',
				'plz');
		$query = 'SELECT ';
		$query .= implode (', ', $fields);
		$query .= ', ' . $this->getDistanceFormula ($geoObject) . ' AS e';
		$query .= ' FROM  ' . $opnTables['geodb_locations'];
		$query .= ' WHERE ' . $this->getDistanceFormula ($geoObject) . ' < ' . $maxRadius;
		$query .= ' ORDER BY e ASC';
		$fields[] = 'e';
		if ($maxHits !== FALSE) {
			$queryResult = &$opnConfig['database']->SelectLimit ($query, $maxHits);
		} else {
			$queryResult = &$opnConfig['database']->Execute ($query);
		}
		return $this->transformQueryResult ($queryResult, $fields);

	}

	/**
	* Returns the formula which evaluates the distance between the passed GeoObject and
	* the elements in the database.
	*
	* @access  private
	* @param   geodb_object  &$geoObject
	* @return  string
	*/

	function getDistanceFormula (&$geoObject) {

		$formula = 'IFNULL(';
		$formula .= '(ACOS((SIN(' . $geoObject->latitudeRad . ')*SIN(RADIANS(breite))) + ';
		$formula .= '(COS(' . $geoObject->latitudeRad . ')*COS(RADIANS(breite))*COS(RADIANS(laenge)-' . $geoObject->longitudeRad . '))) * ';
		$formula .= geodb::getEarthRadius () . ')';
		$formula .= ',0)';
		return $formula;

	}

	/**
	* Transforms a db-query-result to an array of GeoObjects.
	*
	* @access  private
	* @param   DB_Result  &$result
	* @param   array  		$fields
	* @return  object	  GeoObjects
	*/

	function transformQueryResult (&$result, $fields) {

		$foundGeoObjects = array ();
		if (!$result === false) {
			$count = 0;
			while (! $result->EOF) {
				$location = array ();
				foreach ($fields AS $field) {
					$location[$count][$field] = $result->fields ($field);
				}
				$foundGeoObjects[] = new geodb_object ($location[$count]['locationsname'], $location[$count]['breite'], $location[$count]['laenge'], true, $location[$count]);
				$count++;
				$result->MoveNext ();
			}
		}
		return $foundGeoObjects;

	}

	/**
	* Find GeoObjects
	*
	* Returns an array of GeoObjects which fits the $searchConditions
	*
	* @access  public
	* @param   mixed	$searchConditions  string or array
	* @return  array
	*/

	function findGeoObject ($searchConditions = array () ) {

		global $opnConfig,
								$opnTables;
		$fields = array ('locationsid',
				'locationsname',
				'typ',
				'adm0',
				'adm1',
				'adm2',
				'adm3',
				'adm4',
				'ort',
				'breite',
				'laenge',
				'gs',
				'kfz',
				'plz');
		if (is_array ($searchConditions) ) {
			foreach ($searchConditions AS $key => $val) {
				$where[] = $key . " = '" . $val . "'";
			}
			$query = 'SELECT ' . implode (', ', $fields) . ' FROM ' . $opnTables['geodb_locations'] . ' WHERE ' . join (' AND ', $where) . ' ORDER BY ort';
		} else {
			$like_search = $opnConfig['opnSQL']->AddLike ($searchConditions);
			$query = 'SELECT ' . implode (', ', $fields) . ' FROM ' . $opnTables['geodb_locations'] . " WHERE locationsname LIKE $like_search ORDER BY ort";
		}
		$queryResult = &$opnConfig['database']->Execute ($query);
		return $this->transformQueryResult ($queryResult);

	}

	/**
	* Find GeoObjects near an overgiven GeoObject
	*
	* Searches for GeoObjects, which are in a specified radius around the passed GeoBject.
	* Default is radius of 100 (100 of specified unit, see configuration and maxHits of 50
	* Returns an array of GeoDB-objects which lie in ther radius of the passed GeoObject.
	*
	* @access  public
	* @param   object  &$geoObject
	* @param   int	 $maxRadius
	* @param   int	 $maxHits
	* @return  array
	*/

	function findCloseByZips (&$geoObject, $maxRadius = 100, $maxHits = 50) {

		global $opnConfig, $opnTables;

		$fields = array ('plz');
		$query = 'SELECT ';
		$query .= implode (', ', $fields);
		$query .= ', ' . $this->getDistanceFormula ($geoObject) . ' AS e';
		$query .= ' FROM  ' . $opnTables['geodb_locations'];
		$query .= ' WHERE ' . $this->getDistanceFormula ($geoObject) . ' < ' . $maxRadius;
		$query .= ' ORDER BY e ASC';
		if ($maxHits !== FALSE) {
			$result = &$opnConfig['database']->SelectLimit ($query, $maxHits);
		} else {
			$result = &$opnConfig['database']->Execute ($query);
		}
		$zips = '';
		while (! $result->EOF) {
			$zips .= $result->fields ('plz') . ', ';
			$result->MoveNext ();
		}
		$zips = str_replace ('?,', '', $zips);
		return rtrim (rtrim ($zips), ',');

	}

	/**
	* Return Info of a gfx for an overgiven GeoObject
	*
	* Searches for Image, if not found (cached) a new one is created.
	* Returns an array with informations like path, url and img tag.
	*
	* @access  public
	* @param   object  &$geoObject
	* @param   object	 &$location
	* @param   int	 $range
	* @return  array
	*/

	function getImage (&$mygeodb, &$location, $range = '') {

		global $opnConfig;

		$imgarr = array ();
		$imgarr['DE'] = array ('imgwidth' => 188,
					'imgheight' => 235,
					'x1' => 5.5,
					'x2' => 15.5,
					'y1' => 47.2,
					'y2' => 55.1);
		$imgarr['AT'] = array ('imgwidth' => 250,
					'imgheight' => 135,
					'x1' => 9.4,
					'x2' => 17.3,
					'y1' => 46.3,
					'y2' => 49.1);
		$imgarr['CH'] = array ('imgwidth' => 235,
					'imgheight' => 150,
					'x1' => 5.7,
					'x2' => 10.8,
					'y1' => 45.6,
					'y2' => 47.9);
		$thiscountry = $location->dbValues['adm0'];
		$rangetext = '';
		if ($range != '') {
			$rangetext = '_r' . $range;
		}
		$myImagename = $opnConfig['datasave']['geodb']['path'] . $location->dbValues['locationsid'] . $rangetext . '.png';
		if (!file_exists ($myImagename) ) {
			$mycountry = strtolower ($location->dbValues['adm0']);
			$mycountrymap = $opnConfig['datasave']['geodb']['path'] . $mycountry . '.png';
			if (!file_exists ($mycountrymap) ) {
				$map2 = $mygeodb->setupMap ($imgarr[$location->dbValues['adm0']]['imgwidth'], $imgarr[$location->dbValues['adm0']]['imgheight']);
				$map2->setRange ($imgarr[$location->dbValues['adm0']]['x1'], $imgarr[$location->dbValues['adm0']]['x2'], $imgarr[$location->dbValues['adm0']]['y1'], $imgarr[$location->dbValues['adm0']]['y2']);
				$map2->MapSetColor ('borders', $map2->allocateColor (192, 192, 192) );
				$map2->MapSetColor ('states', $map2->allocateColor (212, 212, 212) );
				$map2->addDataFile ($opnConfig['datasave']['geodb']['path'] . $mycountry . '_staatline.e00', 'states');
				$map2->addDataFile ($opnConfig['datasave']['geodb']['path'] . $mycountry . '_ponet.e00', 'borders');
				$map2->saveImage ($mycountrymap);
			} else {
				$map2 = $mygeodb->setupMap ($mycountrymap);
				$map2->setRange ($imgarr[$thiscountry]['x1'], $imgarr[$thiscountry]['x2'], $imgarr[$thiscountry]['y1'], $imgarr[$thiscountry]['y2']);
			}
			$map2->addGeoObject ($location, 'red', 6);
			if ( ($range != '') && (is_numeric ($range) ) ) {
				$map2->addGeoObject ($location, 'black', ($range/3), true);
			}
			$map2->saveImage ($myImagename);
		}
		$thisimage['path'] = $myImagename;
		$thisimage['url'] = $opnConfig['datasave']['geodb']['url'] . '/' . $location->dbValues['locationsid'] . $rangetext . '.png';
		$thisimage['tag'] = '<img class="imgtag" src="' . $thisimage['url'] . '" width="' . $imgarr[$thiscountry]['imgwidth'] . '" height="' . $imgarr[$thiscountry]['imgheight'] . '" alt="' . $location->dbValues['locationsname'] . '" />';
		return $thisimage;

	}

}

?>